@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   5/5/00
 *
-* $Revision: 1.28 $
+* $Revision: 1.30 $
 *
 * File Description: 
 *
@@ -74,6 +74,12 @@ NLM_EXTERN CONN PubSeqFetchOpenConnection (
   Int4 flags
 );
 
+NLM_EXTERN CONN PubSeqFetchTraceOpenConnection (
+  Uint4 tid,
+  Int2 retcode,
+  Int4 flags
+);
+
 NLM_EXTERN CONN GiRevHistOpenConnection (
   Int4 uid,
   Int4 num,
@@ -135,6 +141,12 @@ NLM_EXTERN PubmedEntryPtr PubMedSynchronousQuery (
   Int4 uid
 );
 
+NLM_EXTERN SeqEntryPtr PubSeqSynchronousQueryTI (
+  Uint4 tid,
+  Int2 retcode,
+  Int4 flags
+);
+
 NLM_EXTERN SeqEntryPtr PubSeqSynchronousQuery (
   Int4 uid,
   Int2 retcode,
