@@ -1,4 +1,4 @@
-/* $Id: blast_format.c,v 1.114 2007/01/19 14:32:31 madden Exp $
+/* $Id: blast_format.c,v 1.116 2008/08/26 19:52:23 camacho Exp $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -32,7 +32,7 @@
  */
 
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: blast_format.c,v 1.114 2007/01/19 14:32:31 madden Exp $";
+static char const rcsid[] = "$Id: blast_format.c,v 1.116 2008/08/26 19:52:23 camacho Exp $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/api/blast_format.h>
@@ -617,6 +617,66 @@ GetOldAlignType(EBlastProgramType prog, Boolean* db_is_na)
     return align_type;
 }
 
+/** Deletes matrix generated by s_LoadMatrix
+ * @param matrix data to be freed [in]
+ */
+const int k_MatrixSize = 128;
+static void s_DeleteMatrix(Int4** matrix)
+{
+    int i;
+    if (matrix == NULL)
+      return;
+
+    for(i=0; i<k_MatrixSize; i++)
+         MemFree(matrix[i]);
+
+    MemFree(matrix);
+
+    return;
+}
+
+/** Adopted from api/jzcoll.c:load_default_matrix
+ * We take the compressed matrix and move it 
+ * to a format that includes all ASCII letters,
+ * though most cells will be blank.  this is understodd
+ * by txalign.c and does not seem worth fixing now.
+ * @param name matrix name (e.g., BLOSUM90) [in]
+ */
+static Int4** s_LoadMatrix(const char* name)
+{
+    Int4** ss;
+    int i, j;
+    int dim = 0;
+    const char* pchars = NULL;
+    SNCBIPackedScoreMatrix* pack_matrix = 
+       NCBISM_GetStandardMatrix(name?name:BLAST_DEFAULT_MATRIX);
+
+    if (pack_matrix == NULL)
+        return NULL;
+
+    pchars = pack_matrix->symbols;
+    dim = strlen(pchars);
+
+    ss = (Int4**) MemNew((size_t)k_MatrixSize * sizeof (Int4*));
+    if (ss == NULL)
+       return NULL;
+
+    for(i = 0; i<k_MatrixSize; ++i)
+           ss[i] = (Int4*) MemNew((size_t)k_MatrixSize * sizeof (Int4));
+
+    for(i = 0; i < k_MatrixSize; i++)
+           for(j = 0; j < k_MatrixSize;j++)
+                 ss[i][j] = -1000;
+
+    for(i = 0; i < dim; ++i)
+           for(j = 0; j < dim; ++j)
+                 ss[pchars[i]][pchars[j]] = pack_matrix->scores[(i*dim)+j];
+
+    return ss;
+}
+
+
+
 static Int2
 s_BLAST_PrintDatabaseInfo(const BlastFormattingInfo* format_info)
 {
@@ -834,7 +894,7 @@ Int2 BLAST_FormatResults(SBlastSeqalignArray* seqalign_arr, Int4 num_queries,
          } 
          if (outfp) {
             BlastPruneSapStruct* prune;
-            Int4** matrix = NULL;
+            Int4** matrix = s_LoadMatrix(sum_returns->search_params->matrix);
             ObjMgrSetHold();
             init_buff_ex(85);
             PrintDefLinesFromSeqAlignEx2(seqalign, 80, outfp, 
@@ -849,9 +909,6 @@ Int2 BLAST_FormatResults(SBlastSeqalignArray* seqalign_arr, Int4 num_queries,
                        format_options->number_of_alignments, NULL);
             seqannot->data = prune->sap;
 
-             /** @todo FIXME: matrix must be filled here if it is not default 
-                (BLOSUM62). Matrix name can be retrieved from 
-                sum_returns->search_params->matrix. */
             if(format_info->search_options->score_options->is_ooframe) {
                OOFShowBlastAlignment(prune->sap, mask_loc, outfp, 
                                      format_options->align_options, NULL);
@@ -863,6 +920,7 @@ Int2 BLAST_FormatResults(SBlastSeqalignArray* seqalign_arr, Int4 num_queries,
                   format_options->align_options, matrix, mask_loc, 
                   FormatScoreFunc);
             }
+            s_DeleteMatrix(matrix);
             seqannot->data = seqalign;
             prune = BlastPruneSapStructDestruct(prune);
             ObjMgrClearHold();
