@@ -1,4 +1,4 @@
-/* $Id: blast_aascan.c,v 1.7 2007/04/11 15:55:30 kazimird Exp $
+/* $Id: blast_aascan.c,v 1.12 2008/11/04 14:54:34 kazimird Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,9 +33,76 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] =
-    "$Id: blast_aascan.c,v 1.7 2007/04/11 15:55:30 kazimird Exp $";
+    "$Id: blast_aascan.c,v 1.12 2008/11/04 14:54:34 kazimird Exp $";
 #endif                          /* SKIP_DOXYGEN_PROCESSING */
 
+
+/** 
+ * @brief Determines the scanner's offsets taking the database masking
+ * restrictions into account (if any). This function should be called from the
+ * ScanSubject routines only.
+ * 
+ * @param subject The subject sequence [in]
+ * @param offset offset into the subject sequence about to be scanned [in]
+ * @param word_length the lookup table word length [in]
+ * @param s_first starting offset to scan [out]
+ * @param s_last ending offset to scan
+ * 
+ * @return TRUE if the scanning should proceed, FALSE otherwise (caller must
+ * return 0)
+ */
+static NCBI_INLINE Boolean
+s_DetermineScanningOffsets(const BLAST_SequenceBlk* subject,
+                           Int4* offset,
+                           Int4 word_length,
+                           Uint1** s_first,
+                           Uint1** s_last)
+{
+    Uint4 index;
+
+    ASSERT(subject->num_seq_ranges >= 1);
+
+    for (index = 0; index < subject->num_seq_ranges; index++) {
+        /* if offset is after the end of the segment, go to the next
+         * segment. */
+        if (*offset > subject->seq_ranges[index].right) {
+            continue;
+        }
+
+        /* if offset is before the beginning of the segment, advance it. */
+        if (*offset < subject->seq_ranges[index].left) {
+            *offset = subject->seq_ranges[index].left;
+        }
+
+        *s_first = subject->sequence + *offset;
+        *s_last  = subject->sequence + 
+            subject->seq_ranges[index].right - 
+            word_length;
+
+        /* if we fell off the end of the last segment, try the next one. */
+        if (*s_first > *s_last) {
+            continue;
+        }
+        /* otherwise, we've found a valid region to scan; break out. */
+        else {
+            break;
+        }
+
+    } /* end for */
+
+    /* if we didn't find any more valid ranges to scan, set the
+       expected exit conditions and return. */
+    if (index == subject->num_seq_ranges) {
+        *offset = subject->length - word_length + 1;
+        return FALSE;
+    }
+
+    ASSERT(index < subject->num_seq_ranges);
+    ASSERT(subject->seq_ranges[index].left <= subject->seq_ranges[index].right);
+    ASSERT(*s_first <= *s_last);
+    return TRUE;
+}
+
 /**
  * Scans the subject sequence from "offset" to the end of the sequence.
  * Copies at most array_size hits.
@@ -57,21 +124,28 @@ static Int4 s_BlastAaScanSubject(const LookupTableWrap * lookup_wrap,
                                  Int4 array_size)
 {
     Int4 index;
-    Uint1 *s;
-    Uint1 *s_first;
-    Uint1 *s_last;
+    Uint1 *s = NULL;
+    Uint1 *s_first = NULL;
+    Uint1 *s_last = NULL;
     Int4 numhits = 0;           /* number of hits found for a given subject
                                    offset */
     Int4 totalhits = 0;         /* cumulative number of hits found */
     PV_ARRAY_TYPE *pv;
     BlastAaLookupTable *lookup;
+    AaLookupBackboneCell *bbc;
+    Int4 *ovfl;
 
     ASSERT(lookup_wrap->lut_type == eAaLookupTable);
     lookup = (BlastAaLookupTable *) lookup_wrap->lut;
-
-    s_first = subject->sequence + *offset;
-    s_last = subject->sequence + subject->length - lookup->word_length;
+    ASSERT(lookup->bone_type == eBackbone);
     pv = lookup->pv;
+    bbc = (AaLookupBackboneCell *) lookup->thick_backbone;
+    ovfl = (Int4 *) lookup->overflow;
+
+    if (!s_DetermineScanningOffsets(subject, offset, lookup->word_length,
+                                    &s_first, &s_last)) {
+        return 0;
+    }
 
     /* prime the index */
     index = ComputeTableIndex(lookup->word_length - 1,
@@ -85,7 +159,7 @@ static Int4 s_BlastAaScanSubject(const LookupTableWrap * lookup_wrap,
 
         /* if there are hits... */
         if (PV_TEST(pv, index, PV_ARRAY_BTS)) {
-            numhits = lookup->thick_backbone[index].num_used;
+            numhits = bbc[index].num_used;
 
             ASSERT(numhits != 0);
 
@@ -94,22 +168,21 @@ static Int4 s_BlastAaScanSubject(const LookupTableWrap * lookup_wrap,
                 /* ...then copy the hits to the destination */
             {
                 Int4 *src;
-                Int4 i;
                 if (numhits <= AA_HITS_PER_CELL)
                     /* hits live in thick_backbone */
-                    src = lookup->thick_backbone[index].payload.entries;
+                    src = bbc[index].payload.entries;
                 else
                     /* hits live in overflow array */
-                    src =
-                        &(lookup->
-                          overflow[lookup->thick_backbone[index].payload.
-                                   overflow_cursor]);
+                    src = &(ovfl[bbc[index].payload.overflow_cursor]);
 
                 /* copy the hits. */
+                {
+                Int4 i;
+                Int4 s_off = s - subject->sequence;
                 for (i = 0; i < numhits; i++) {
                     offset_pairs[i + totalhits].qs_offsets.q_off = src[i];
-                    offset_pairs[i + totalhits].qs_offsets.s_off =
-                        s - subject->sequence;
+                    offset_pairs[i + totalhits].qs_offsets.s_off = s_off;
+                }
                 }
 
                 totalhits += numhits;
@@ -126,7 +199,92 @@ static Int4 s_BlastAaScanSubject(const LookupTableWrap * lookup_wrap,
 
     /* if we get here, we fell off the end of the sequence */
     *offset = s - subject->sequence;
+    return totalhits;
+}
 
+/** same function for small lookup table */
+static Int4 s_BlastSmallAaScanSubject(const LookupTableWrap * lookup_wrap,
+                                 const BLAST_SequenceBlk * subject,
+                                 Int4 * offset,
+                                 BlastOffsetPair * NCBI_RESTRICT offset_pairs,
+                                 Int4 array_size)
+{
+    Int4 index;
+    Uint1 *s = NULL;
+    Uint1 *s_first = NULL;
+    Uint1 *s_last = NULL;
+    Int4 numhits = 0;           /* number of hits found for a given subject
+                                   offset */
+    Int4 totalhits = 0;         /* cumulative number of hits found */
+    PV_ARRAY_TYPE *pv;
+    BlastAaLookupTable *lookup;
+    AaLookupSmallboneCell *bbc;
+    Uint2 *ovfl;
+
+    ASSERT(lookup_wrap->lut_type == eAaLookupTable);
+    lookup = (BlastAaLookupTable *) lookup_wrap->lut;
+    ASSERT(lookup->bone_type == eSmallbone);
+    pv = lookup->pv;   
+    bbc = (AaLookupSmallboneCell *) lookup->thick_backbone;
+    ovfl = (Uint2 *) lookup->overflow;
+
+    if (!s_DetermineScanningOffsets(subject, offset, lookup->word_length,
+                                    &s_first, &s_last)) {
+        return 0;
+    }
+
+    /* prime the index */
+    index = ComputeTableIndex(lookup->word_length - 1,
+                              lookup->charsize, s_first);
+
+    for (s = s_first; s <= s_last; s++) {
+        /* compute the index value */
+        index = ComputeTableIndexIncremental(lookup->word_length, 
+                                             lookup->charsize,
+                                             lookup->mask, s, index);
+
+        /* if there are hits... */
+        if (PV_TEST(pv, index, PV_ARRAY_BTS)) {
+            numhits = bbc[index].num_used;
+
+            ASSERT(numhits != 0);
+
+            /* ...and there is enough space in the destination array, */
+            if (numhits <= (array_size - totalhits))
+                /* ...then copy the hits to the destination */
+            {
+                Uint2 *src;
+                if (numhits <= AA_HITS_PER_CELL)
+                    /* hits live in thick_backbone */
+                    src = bbc[index].payload.entries;
+                else
+                    /* hits live in overflow array */
+                    src = &(ovfl[bbc[index].payload.overflow_cursor]);
+
+                /* copy the hits. */
+                {
+                Int4 i;
+                Int4 s_off = s - subject->sequence;
+                for (i = 0; i < numhits; i++) {
+                    offset_pairs[i + totalhits].qs_offsets.q_off = src[i];
+                    offset_pairs[i + totalhits].qs_offsets.s_off = s_off;
+                }
+                }
+
+                totalhits += numhits;
+            } else
+                /* not enough space in the destination array; return early */
+            {
+                break;
+            }
+        } else
+            /* no hits found */
+        {
+        }
+    }
+
+    /* if we get here, we fell off the end of the sequence */
+    *offset = s - subject->sequence;
     return totalhits;
 }
 
@@ -152,10 +310,11 @@ static Int4 s_BlastCompressedAaScanSubject(
                               BlastOffsetPair * NCBI_RESTRICT offset_pairs,
                               Int4 array_size)
 {
-    Int4 index;
-    Uint1 *s;
-    Uint1 *s_first;
-    Uint1 *s_last;
+    Int4 index=0;
+    Int4 preshift; /* used for 2-stage index calculation */
+    Uint1 *s = NULL;
+    Uint1 *s_first = NULL;
+    Uint1 *s_last = NULL;
     Int4 numhits = 0;     /* number of hits found for one subject offset */
     Int4 totalhits = 0;         /* cumulative number of hits found */
     PV_ARRAY_TYPE *pv;
@@ -173,51 +332,73 @@ static Int4 s_BlastCompressedAaScanSubject(
     ASSERT(lookup_wrap->lut_type == eCompressedAaLookupTable);
     lookup = (BlastCompressedAaLookupTable *) lookup_wrap->lut;
     word_length = lookup->word_length;
+
+    if (!s_DetermineScanningOffsets(subject, offset, word_length, &s_first,
+                                    &s_last)) {
+        return 0;
+    }
+
     compressed_alphabet_size = lookup->compressed_alphabet_size;
     scaled_compress_table = lookup->scaled_compress_table;
     recip = lookup->reciprocal_alphabet_size;
-    s_first = subject->sequence + *offset;
-    s_last = subject->sequence + subject->length - word_length;
     pv = lookup->pv;
     pv_array_bts = lookup->pv_array_bts;
 
     /* prime the index */
-    index = s_ComputeCompressedIndex(word_length - 1, s_first,
-                                     compressed_alphabet_size,
-                                     &skip, lookup);
+    for(s = s_first; s <= s_last; s++){
+        index = s_ComputeCompressedIndex(word_length - 1, s,
+                                         compressed_alphabet_size,
+                                         &skip, lookup);
+        if(!skip)
+          break;
+    }
 
-    next_char = ((subject->length > word_length)? s_first[word_length-1] : 0);
+    next_char = ((s <= s_last)? s[word_length-1] : 0);
+    preshift = (Int4)((((Int8)index) * recip) >> 32); 
 
-    for (s = s_first; s <= s_last; s++) {
+    /* main scanning loop */
+    for (; s <= s_last; s++) {
        /* compute the index value */
 
        compressed_char = scaled_compress_table[next_char];
        next_char = s[word_length];
-
-       if (compressed_char < 0) {
-          /* skip all words containing this (ignored) letter */
-          skip = word_length; 
-          continue;
-       } 
-       else {
-          /* we have to remove the oldest letter from the
-             index and add in the next letter. The latter is easy,
-             but since the compressed alphabet size is not a
-             power of two the former requires a remainder and
-             multiply, assuming the old letter is in the high part
-             of the index. For this reason, we reverse the order
-             of the letters and keep the oldest in the low part
-             of index, so that a single divide (implemented via
-             reciprocal multiplication) does the removal */
-
-          index = (Int4)((((Int8)index) * recip) >> 32) + compressed_char;
-
-          if (skip) {
-             if (--skip) /* still skipping? */
-                continue;
-          }
-       }
       
+       if(compressed_char < 0){ /* flush (rare) "bad" character(s) */
+         preshift = 0;
+         s++;
+         for(skip = word_length-1; skip && (s <= s_last) ; s++){
+           compressed_char = scaled_compress_table[next_char];
+           next_char = s[word_length];
+           
+           if(compressed_char < 0){ /* not again! */
+             skip = word_length-1;
+             preshift = 0;
+             continue;
+           }
+           
+           index = preshift + compressed_char;
+           preshift = (Int4)((((Int8)( index )) * recip) >> 32);
+           skip--;
+         }
+         
+         s--; /*undo the following increment*/
+         continue;
+       }
+
+       /* we have to remove the oldest letter from the
+          index and add in the next letter. The latter is easy,
+          but since the compressed alphabet size is not a
+          power of two the former requires a remainder and
+          multiply, assuming the old letter is in the high part
+          of the index. For this reason, we reverse the order
+          of the letters and keep the oldest in the low part
+          of index, so that a single divide (implemented via
+          reciprocal multiplication) does the removal.
+          Index calculation done in two steps to let the CPU do 
+          out-of-order execution. */
+       
+       index = preshift + compressed_char;
+       preshift = (Int4)((((Int8)( index )) * recip) >> 32);
 
        /* if there are hits */
        if (PV_TEST(pv, index, pv_array_bts)) {
@@ -455,7 +636,12 @@ void BlastChooseProteinScanSubject(LookupTableWrap *lookup_wrap)
 {
     if (lookup_wrap->lut_type == eAaLookupTable) {
         BlastAaLookupTable *lut = (BlastAaLookupTable *)(lookup_wrap->lut);
-        lut->scansub_callback = (void *)s_BlastAaScanSubject;
+        /* normal backbone */
+        if(lut->bone_type == eBackbone)
+           lut->scansub_callback = (void *)s_BlastAaScanSubject;
+        /* small bone*/
+        else
+           lut->scansub_callback = (void *)s_BlastSmallAaScanSubject;
     }
     else if (lookup_wrap->lut_type == eCompressedAaLookupTable) {
         BlastCompressedAaLookupTable *lut = 
