@@ -1,4 +1,4 @@
-/* $Id: blast_def.h,v 1.79 2007/03/20 14:55:37 kazimird Exp $
+/* $Id: blast_def.h,v 1.88 2009/02/03 18:14:30 kazimird Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -35,6 +35,7 @@
 
 #include <algo/blast/core/ncbi_std.h>
 #include <algo/blast/core/blast_export.h>
+#include <algo/blast/core/blast_program.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -42,18 +43,18 @@ extern "C" {
 
 /****************************** Constants *********************************/
 
-extern const int kDustLevel;  /**< Level parameter used by dust. */
-extern const int kDustWindow; /**< Window parameter used by dust. */
-extern const int kDustLinker; /**< Parameter used by dust to link together close low-complexity segments. */
+NCBI_XBLAST_EXPORT extern const int kDustLevel;  /**< Level parameter used by dust. */
+NCBI_XBLAST_EXPORT extern const int kDustWindow; /**< Window parameter used by dust. */
+NCBI_XBLAST_EXPORT extern const int kDustLinker; /**< Parameter used by dust to link together close low-complexity segments. */
 
-extern const int kSegWindow;  /**< Window that SEG examines at once. */
-extern const double kSegLocut;   /**< Locut parameter for SEG. */
-extern const double kSegHicut;   /**< Hicut parameter for SEG. */
+NCBI_XBLAST_EXPORT extern const int kSegWindow;  /**< Window that SEG examines at once. */
+NCBI_XBLAST_EXPORT extern const double kSegLocut;   /**< Locut parameter for SEG. */
+NCBI_XBLAST_EXPORT extern const double kSegHicut;   /**< Hicut parameter for SEG. */
 
 /** Maximum number of HPSs to be saved in an ungapped search.
  * Value defined in blast_options.c
  */
-extern const int kUngappedHSPNumMax; 
+NCBI_XBLAST_EXPORT extern const int kUngappedHSPNumMax; 
 
 /******************** Preprocessor definitions ******************************/
 
@@ -150,6 +151,47 @@ typedef struct SSeqRange {
    Int4 right;  /**< right endpoint of range (zero based) */
 } SSeqRange;
 
+/** Create a new SSeqRange structure with both fields initialized
+ * @param start the start of the range [in]
+ * @param stop the end of the range [in]
+ */
+NCBI_XBLAST_EXPORT
+SSeqRange SSeqRangeNew(Int4 start, Int4 stop);
+
+/** Determine if two ranges intersect
+ * @param a first range to compare [in]
+ * @param b second range to compare [in]
+ * @return TRUE if they intersect, otherwise FALSE 
+ */
+static NCBI_INLINE
+Boolean SSeqRangeIntersectsWith(const SSeqRange* a, const SSeqRange* b)
+{
+    if ( !a || !b ) {
+        return FALSE;
+    }
+
+    if ( (b->right < a->left) || (b->left > a->right) )
+        return FALSE;
+
+    return TRUE;
+}
+
+/** Returns the index of the range, such that this element is the
+ * first range that either contains the target or if no such range exists, the
+ * index of the first range, such that the target is less than this range.
+ * @pre ranges array is sorted on the starting coordinates (i.e.:
+ * SSeqRange::left)
+ * @param ranges array of SSeqRange structures to search [in]
+ * @param num_ranges number of elements in the ranges array [in]
+ * @param target element to look for [in]
+ * @return the index of interest in the ranges array or -1 if the function was
+ * called with invalid parameters
+ */
+NCBI_XBLAST_EXPORT
+Int4
+SSeqRangeArrayLessThanOrEqual(const SSeqRange* ranges, Int4 num_ranges,
+                              Int4 target);
+
 /** Used to hold a set of positions, mostly used for filtering. 
  * oid holds the index of the query sequence.
 */
@@ -190,6 +232,8 @@ typedef struct BLAST_SequenceBlk {
                                sequence as that byte is a NULL sentinel byte.*/
    Int4     length;         /**< Length of sequence. */
    Int2 frame; /**< Frame of the query, needed for translated searches */
+   Int2 subject_strand; /**< Strand of the subject sequence for translated searches. 
+                          Uses the same values as ENa_strand. */
    Int4 oid; /**< The ordinal id of the current sequence */
    Boolean sequence_allocated; /**< TRUE if memory has been allocated for 
                                   sequence */
@@ -215,6 +259,12 @@ typedef struct BLAST_SequenceBlk {
                               owned by the genetic code singleton. 
                               @sa gencode_singleton.h
                               */
+   /* BEGIN: Data members needed for masking subjects from a BLAST database */
+   SSeqRange* seq_ranges;   /**< Ranges of the sequence to search */
+   Uint4 num_seq_ranges;    /**< Number of elements in seq_ranges */
+   Boolean seq_ranges_allocated;   /**< TRUE if memory has been allocated for
+                                      seq_ranges */
+   /* END: Data members needed for masking subjects from a BLAST database */
 } BLAST_SequenceBlk;
 
 /** Information about a single pattern occurence in the query. */
@@ -232,8 +282,21 @@ typedef struct SPHIQueryInfo {
                                         structures. */
     Int4 allocated_size; /**< Allocated size of the occurrences array. */
     double probability; /**< Estimated probability of the pattern */
+    char* pattern;   /**< Pattern used, saved here for formatting purposes. */
 } SPHIQueryInfo;
 
+
+/** Information about target translations. */
+typedef struct SBlastTargetTranslation {
+   EBlastProgramType program_number; /**< Program being run. */
+   const Uint1* gen_code_string; /**< Genetic code string for translation. */
+   Uint1** translations; /**< two dimensional array for translations. */
+   Boolean partial; /**< specifies that nucleotide sequence is too long to translated. */
+   Int4 num_frames; /**< how many frames, one dimension of translation_buffer. */
+   Int4* range; /**< start and stop of translated sequences. */
+   BLAST_SequenceBlk* subject_blk; /**< target sequence being translated. */
+} SBlastTargetTranslation;
+
 /************************* Progress monitoring/interruptible API *************/
 
 /** Enumeration for the stages in the BLAST search */
@@ -268,14 +331,17 @@ typedef Boolean (*TInterruptFnPtr) (SBlastProgress* progress_info);
  * [in]
  * Implemented in blast_util.c 
  */
+NCBI_XBLAST_EXPORT
 SBlastProgress* SBlastProgressNew(void* user_data);
 
 /** Deallocates a SBlastProgress structure.
  * Implemented in blast_util.c */
+NCBI_XBLAST_EXPORT
 SBlastProgress* SBlastProgressFree(SBlastProgress* progress_info);
 
 /** Resets the progress structure to its original state (as if newly allocated)
  * for a fresh start without touching the user_data field */
+NCBI_XBLAST_EXPORT
 void SBlastProgressReset(SBlastProgress* progress_info);
 
 #ifdef __cplusplus
