@@ -1,4 +1,4 @@
-/* $Id: blast_filter.c,v 1.89 2007/07/05 14:25:35 kazimird Exp $
+/* $Id: blast_filter.c,v 1.94 2008/11/03 20:59:44 kazimird Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -30,7 +30,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: blast_filter.c,v 1.89 2007/07/05 14:25:35 kazimird Exp $";
+    "$Id: blast_filter.c,v 1.94 2008/11/03 20:59:44 kazimird Exp $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/blast_util.h>
@@ -125,6 +125,55 @@ s_ParseRepeatOptions(const char* repeat_options, char** dbname)
     return 0;
 }
 
+/** Parses window masker options string.
+ * @param winmask_options Input character string [in]
+ * @param dbname Database name for window masker filtering [out]
+ * @param taxid Taxonomic ID for window masker filtering [out]
+ */
+static Int2  
+s_ParseWindowMaskerOptions(const char  * winmask_options,
+                           char       ** dbname,
+                           int         * taxid)
+{
+    char* ptr = NULL;
+    
+    ASSERT(dbname);
+    *dbname = NULL;
+    
+    if (!winmask_options)
+        return 0;
+    
+    ptr = strstr(winmask_options, "-d");
+    
+    if (ptr) {
+        char * endp = 0;
+        
+        ptr += 2;
+        while (*ptr == ' ' || *ptr == '\t')
+            ++ptr;
+        
+        *dbname = strdup(ptr);
+        
+        for(endp = *dbname; *endp; ++endp) {
+            if (*endp == ' ' || *endp == '\t') {
+                *endp = (char)0;
+                break;
+            }
+        }
+    } else {
+        ptr = strstr(winmask_options, "-t");
+        
+        if (ptr) {
+            ptr += 2;
+            while (*ptr == ' ' || *ptr == '\t')
+                ++ptr;
+            *taxid = atoi(ptr);
+        }
+    }
+
+    return 0;
+}
+
 /** Parses options used for dust.
  * @param ptr buffer containing instructions. [in]
  * @param level sets level for dust. [out]
@@ -248,11 +297,141 @@ s_ParseSegOptions(const char *ptr, Int4* window, double* locut, double* hicut)
 	return 0;
 }
 
+/// Wrapper around strcat to ensure we don't do buffer overflows :)
+/// @param dest string to concatenate to [in|out]
+/// @param dest_size size of the dest array, modified if dest is grown [in|out]
+/// @param string2append string to append to dest [in]
+/// @return the concatenated string or NULL if we run out of memory
+static char*
+s_SafeStrCat(char** dest, unsigned int* dest_size, const char* string2append)
+{
+    size_t dest_length = strlen(*dest);
+    size_t string2append_length = strlen(string2append);
+    if ((dest_length + string2append_length + 1) > *dest_size) {
+        size_t target_size = MAX(string2append_length, dest_length) * 2;
+        *dest = (char*)realloc((void*)*dest, target_size);
+        if (*dest) {
+            (*dest_size) = target_size;
+        } else {
+            sfree(*dest);
+            return 0;
+        }
+    }
+    strcat(*dest, string2append);
+    return *dest;
+}
 
+char*
+BlastFilteringOptionsToString(const SBlastFilterOptions* filtering_options)
+{
+    char* retval = NULL;
+    unsigned int retval_size = 0;
+
+    if (filtering_options == NULL) {
+        return strdup("F");
+    }
+
+    retval_size = 64;   /* Usually this will suffice */
+    retval = (char*) calloc(retval_size, sizeof(char));
+
+    if (filtering_options->dustOptions) {
+        if (filtering_options->dustOptions->level == kDustLevel &&
+            filtering_options->dustOptions->window == kDustWindow &&
+            filtering_options->dustOptions->linker == kDustLinker) {
+            if (!s_SafeStrCat(&retval, &retval_size, "L;")) {
+                return 0;
+            }
+        } else {
+            char buffer[24] = { '\0' };
+            snprintf(buffer, sizeof(buffer), "D %d %d %d;",
+                     filtering_options->dustOptions->level,
+                     filtering_options->dustOptions->window,
+                     filtering_options->dustOptions->linker);
+            if (!s_SafeStrCat(&retval, &retval_size, buffer)) {
+                return 0;
+            }
+        }
+    }
+
+    if (filtering_options->segOptions) {
+        if (filtering_options->segOptions->window == kSegWindow &&
+            filtering_options->segOptions->locut == kSegLocut &&
+            filtering_options->segOptions->hicut == kSegHicut) {
+            if (!s_SafeStrCat(&retval, &retval_size, "L;")) {
+                return 0;
+            }
+        } else {
+            char buffer[24] = { '\0' };
+            snprintf(buffer, sizeof(buffer), "S %d %1.1f %1.1f;",
+                     filtering_options->segOptions->window,
+                     filtering_options->segOptions->locut,
+                     filtering_options->segOptions->hicut);
+            if (!s_SafeStrCat(&retval, &retval_size, buffer)) {
+                return 0;
+            }
+        }
+    }
+
+    if (filtering_options->repeatFilterOptions) {
+        if (filtering_options->repeatFilterOptions->database) {
+            if (!s_SafeStrCat(&retval, &retval_size, "R -d ")) {
+                return 0;
+            }
+            if (!s_SafeStrCat(&retval, &retval_size,  
+                         filtering_options->repeatFilterOptions->database)) {
+                return 0;
+            }
+            if (!s_SafeStrCat(&retval, &retval_size, ";")) {
+                return 0;
+            }
+        } else {
+            if (!s_SafeStrCat(&retval, &retval_size, "R;")) {
+                return 0;
+            }
+        }
+    }
+
+    if (filtering_options->windowMaskerOptions) {
+        if (filtering_options->windowMaskerOptions->taxid != 0) {
+            char buffer[24] = { '\0' };
+            snprintf(buffer, sizeof(buffer), "W -t %d;",
+                     filtering_options->windowMaskerOptions->taxid);
+            if (!s_SafeStrCat(&retval, &retval_size, buffer)) {
+                return 0;
+            }
+        } else if (filtering_options->windowMaskerOptions->database) {
+            if (!s_SafeStrCat(&retval, &retval_size, "W -d ")) {
+                return 0;
+            }
+            if (!s_SafeStrCat(&retval, &retval_size, 
+                         filtering_options->windowMaskerOptions->database)) {
+                return 0;
+            }
+            if (!s_SafeStrCat(&retval, &retval_size, ";")) {
+                return 0;
+            }
+        }
+    }
+
+    /* Mask at hash is a modifier for other filtering options, as such it
+     * doesn't make sense to apply it by itself */
+    if (SBlastFilterOptionsMaskAtHash(filtering_options) && 
+        strlen(retval) != 0 ) {
+        if (!s_SafeStrCat(&retval, &retval_size, "m;")) {
+            return 0;
+        }
+    }
+
+    return strlen(retval) == 0 
+        ? s_SafeStrCat(&retval, &retval_size, "F") 
+        : retval;
+}
 
 Int2
-BlastFilteringOptionsFromString(EBlastProgramType program_number, const char* instructions, 
-       SBlastFilterOptions* *filtering_options, Blast_Message* *blast_message)
+BlastFilteringOptionsFromString(EBlastProgramType program_number, 
+                                const char* instructions, 
+                                SBlastFilterOptions* *filtering_options, 
+                                Blast_Message* *blast_message)
 {
         Boolean mask_at_hash = FALSE; /* the default. */
         char* buffer;
@@ -262,7 +441,8 @@ BlastFilteringOptionsFromString(EBlastProgramType program_number, const char* in
         SSegOptions* segOptions = NULL;
         SDustOptions* dustOptions = NULL;
         SRepeatFilterOptions* repeatOptions = NULL;
-   
+        SWindowMaskerOptions * winmaskOptions = NULL;
+        
         *filtering_options = NULL;
         if (blast_message)
             *blast_message = NULL;
@@ -313,7 +493,7 @@ BlastFilteringOptionsFromString(EBlastProgramType program_number, const char* in
 			ptr = s_LoadOptionsToBuffer(ptr+1, buffer);
 			if (buffer[0] != NULLB)
                         {
-                                int window, level, linker;
+                                int window = 0, level = 0, linker = 0;
 				status = s_ParseDustOptions(buffer, &level, &window, &linker);
                                 if (status)
                                 {
@@ -355,6 +535,35 @@ BlastFilteringOptionsFromString(EBlastProgramType program_number, const char* in
                              }
                         }
                 }
+                else if (*ptr == 'W')
+                {
+                    SWindowMaskerOptionsNew(&winmaskOptions);
+                    
+                    ptr = s_LoadOptionsToBuffer(ptr+1, buffer);
+                    if (buffer[0] != NULLB) {
+                        char* dbname = NULL;
+                        int taxid = 0;
+                        
+                        status = s_ParseWindowMaskerOptions(buffer, &dbname, &taxid);
+                        if (status) {
+                            winmaskOptions = SWindowMaskerOptionsFree(winmaskOptions);
+                            sprintf(error_buffer, "Error parsing filter string: %s", buffer);
+                            if (blast_message)
+                                Blast_MessageWrite(blast_message, eBlastSevError, kBlastMessageNoContext,
+                                                   error_buffer);
+                            
+                            sfree(buffer);
+                            return status;
+                        }
+                        if (dbname) {
+                            sfree(winmaskOptions->database);
+                            winmaskOptions->database = dbname;
+                        }
+                        if (taxid) {
+                            winmaskOptions->taxid = taxid;
+                        }
+                    }
+                }
 		else if (*ptr == 'L' || *ptr == 'T')
 		{ /* do low-complexity filtering; dust for blastn, otherwise seg.*/
                         if (program_number == eBlastTypeBlastn)
@@ -382,6 +591,7 @@ BlastFilteringOptionsFromString(EBlastProgramType program_number, const char* in
         (*filtering_options)->dustOptions = dustOptions;
         (*filtering_options)->segOptions = segOptions;
         (*filtering_options)->repeatFilterOptions = repeatOptions;
+        (*filtering_options)->windowMaskerOptions = winmaskOptions;
         (*filtering_options)->mask_at_hash = mask_at_hash;
 
         return status;
@@ -440,35 +650,71 @@ static BlastSeqLoc* s_BlastSeqLocNodeDup(BlastSeqLoc* source)
     return BlastSeqLocNew(NULL, source->ssr->left, source->ssr->right);
 }
 
-/** Prepend node to the head of the list and return the new head of the list */
-static BlastSeqLoc* s_BlastSeqLocPrepend(BlastSeqLoc* head, BlastSeqLoc* node)
+/** Calculates number of links in a chain of BlastSeqLoc's.
+ * @param var Chain of BlastSeqLoc structures [in]
+ * @return Number of links in the chain.
+ */
+static Int4 s_BlastSeqLocLen(const BlastSeqLoc* var)
 {
-    if ( !node ) {
-        return NULL;
+    BlastSeqLoc* itr = NULL;
+    Int4 retval = 0;
+   
+    for (itr = (BlastSeqLoc*)var; itr; itr = itr->next, retval++) {
+        ;
     }
-    node->next = head;
-    return node;
+    return retval;
+}
+
+/** Converts a BlastSeqLoc list to an array of pointers, each pointing to an
+ * element of the list passed in to this function and the last element points
+ * to NULL 
+ * @param list List to convert to an array of pointers [in]
+ * @param count number of elements populated in the array [out]
+ */
+static BlastSeqLoc**
+s_BlastSeqLocListToArrayOfPointers(const BlastSeqLoc* list, Int4* count)
+{
+    BlastSeqLoc* tmp,** retval;
+    Int4 i;
+    *count = 0;
+
+    if (list == NULL) 
+       return NULL;
+
+    *count = s_BlastSeqLocLen(list);
+    retval = (BlastSeqLoc**) calloc(((size_t)(*count)+1), sizeof(BlastSeqLoc*));
+    for (tmp = (BlastSeqLoc*)list, i = 0; tmp != NULL && i < *count; i++) {
+        retval[i] = tmp;
+        tmp = tmp->next;
+    }
+    return retval;
 }
 
 /** Reverse elements in the list 
- * @param head pointer to pointer to the head of the list. After this call,
- * this is set to NULL [in|out]
- * @return the new head of the list or NULL if argument is NULL
+ * @param head pointer to pointer to the head of the list. [in|out]
+ * (this is not declared static so that it can be tested in the unit tests
  */
-static BlastSeqLoc* s_BlastSeqLocListReverse(BlastSeqLoc** head)
+NCBI_XBLAST_EXPORT
+void BlastSeqLocListReverse(BlastSeqLoc** head)
 {
-    BlastSeqLoc* retval = NULL;     /* return value */
-    BlastSeqLoc* itr = NULL;        /* iterator */
+    BlastSeqLoc** ptrs = NULL;  /* array of pointers to BlastSeqLoc elements */
+    Int4 num_elems = 0, i = 0;
 
     if ( !head ) {
-        return NULL;
+        return;
     }
 
-    for (itr = *head; itr; itr = itr->next) {
-        retval = s_BlastSeqLocPrepend(retval, s_BlastSeqLocNodeDup(itr));
+    ptrs = s_BlastSeqLocListToArrayOfPointers(*head, &num_elems);
+    if (num_elems == 0) {
+        return;
     }
-    *head = BlastSeqLocFree(*head);
-    return retval;
+    ASSERT(ptrs);
+    *head = ptrs[num_elems-1];
+    for (i = num_elems-1; i > 0; i--) {
+        ptrs[i]->next = ptrs[i-1];
+    }
+    ptrs[0]->next = NULL;
+    sfree(ptrs);
 }
 
 BlastSeqLoc* BlastSeqLocNodeFree(BlastSeqLoc* loc)
@@ -715,98 +961,49 @@ static int s_SeqRangeSortByStartPosition(const void *vp1, const void *vp2)
       return 0;
 }
 
-/** Calculates number of links in a chain of BlastSeqLoc's.
- * @param var Chain of BlastSeqLoc structures [in]
- * @return Number of links in the chain.
- */
-static Int4 s_BlastSeqLocLen(BlastSeqLoc* var)
+void
+BlastSeqLocCombine(BlastSeqLoc** mask_loc, Int4 link_value)
 {
-     Int4 count=0;
-   
-     while (var)
-     {
-         count++;
-         var = var->next;
-     }
+    BlastSeqLoc** ptrs = NULL;
+    Int4 i = 0, num_elems = 0;
 
-     return count;
-}
+    /* Break up the list into an array of pointers and sort it */
+    ptrs = s_BlastSeqLocListToArrayOfPointers(*mask_loc, &num_elems);
+    if (num_elems == 0) {
+        return;
+    }
+    ASSERT(ptrs);
+    qsort(ptrs, (size_t)num_elems, sizeof(*ptrs), 
+          s_SeqRangeSortByStartPosition);
 
-/** Sort a list of BlastSeqLoc structures
- * @param list List of BlastSeqLoc's [in]
- * @param compar Comparison function to use [in]
- * @return Sorted list.  
- */
-static BlastSeqLoc* 
-s_BlastSeqLocSort (BlastSeqLoc* list,
-                 int (*compar )(const void *, const void *))
-{
-        BlastSeqLoc* tmp,** head;
-        Int4 count, i;
+    /* Merge the overlapping elements */
+    {
+        BlastSeqLoc* curr_tail = *mask_loc = ptrs[0];
+        for (i = 0; i < num_elems - 1; i++) {
+            const SSeqRange* next_ssr = ptrs[i+1]->ssr;
+            const Int4 stop = curr_tail->ssr->right;
 
-        if (list == NULL) 
-           return NULL;
-
-        count = s_BlastSeqLocLen (list);
-        head = (BlastSeqLoc* *) calloc (((size_t) count + 1), sizeof (BlastSeqLoc*));
-        for (tmp = list, i = 0; tmp != NULL && i < count; i++) {
-                head [i] = tmp;
-                tmp = tmp->next;
+            if ((stop + link_value) > next_ssr->left) {
+                curr_tail->ssr->right = MAX(stop, next_ssr->right);
+                ptrs[i+1] = BlastSeqLocNodeFree(ptrs[i+1]);
+            } else {
+                curr_tail = ptrs[i+1];
+            }
         }
+    }
 
-        qsort (head, (size_t) count, sizeof (BlastSeqLoc*), compar);
-        for (i = 0; i < count; i++) {
-                tmp = head [i];
-                tmp->next = head [i + 1];
+    /* Rebuild the linked list */
+    {
+        BlastSeqLoc* tail = *mask_loc;
+        for (i = 1; i < num_elems; i++) {
+            if (ptrs[i]) {
+                tail->next = ptrs[i];
+                tail = ptrs[i];
+            }
         }
-        list = head [0];
-        sfree (head);
-
-        return list;
-}
-
-BlastSeqLoc*
-BlastSeqLocCombine(BlastSeqLoc* mask_loc, Int4 link_value)
-{
-   BlastSeqLoc* retval = NULL;
-   BlastSeqLoc* retval_tail = NULL;
-   Int4 start, stop;	/* Used to merge overlapping SeqLoc's. */
-   BlastSeqLoc* loc_head=NULL,* loc_var=NULL;
-   
-   if ( !mask_loc ) {
-      return NULL;
-   }
-
-   /* Copy the BlastSeqLoc-s and sort them by starting position. */
-   loc_head = loc_var = s_BlastSeqLocSort(BlastSeqLocListDup(mask_loc),
-                                          s_SeqRangeSortByStartPosition);
-   if ( !loc_head ) {
-       return NULL;
-   }
-   start = loc_head->ssr->left;
-   stop = loc_head->ssr->right;
-
-   for (; loc_var; loc_var = loc_var->next) {
-       SSeqRange* ssr = loc_var->next ? loc_var->next->ssr : NULL;
-
-       if (ssr && ((stop + link_value) > ssr->left)) {
-          stop = MAX(stop, ssr->right);
-       } else {
-          /* Cache the tail of the list to avoid the overhead of traversing the
-           * list when appending to it */
-          retval_tail = BlastSeqLocNew((retval_tail ? &retval_tail : &retval), 
-                                       start, stop);
-          if (loc_var->next) {
-              start = ssr->left;
-              stop = ssr->right;
-          }
-       }
-   }
-
-   /* Free memory allocated for the temporary list of BlastSeqLoc-s */
-   BlastSeqLocFree(loc_head);
-
-   return retval;
+        tail->next = NULL;
+    }
+    sfree(ptrs);
 }
 
 Int2 
@@ -853,8 +1050,7 @@ BLAST_ComplementMaskLocations(EBlastProgramType program_number,
       }
       
       if (BlastIsReverseStrand(kIsNucl, context)) {
-         mask_loc->seqloc_array[context] = 
-             s_BlastSeqLocListReverse(&mask_loc->seqloc_array[context]);
+         BlastSeqLocListReverse(&mask_loc->seqloc_array[context]);
       }
       loc = mask_loc->seqloc_array[context];
 
@@ -958,23 +1154,13 @@ BlastSetUp_Filter(EBlastProgramType program_number,
 	return status;
 }
 
-BlastSeqLoc*
-BlastSeqLocReverse(const BlastSeqLoc* filter_in, Int4 query_length)
+void
+BlastSeqLocReverse(BlastSeqLoc* masks, Int4 query_length)
 {
-   BlastSeqLoc* filter_out = NULL;   /* Return variable. */
-
-   while (filter_in)
-   {
-        Int4 start, stop;
-        SSeqRange* loc = filter_in->ssr;
-        
-        start = query_length - 1 - loc->right;
-        stop = query_length - 1 - loc->left;
-        BlastSeqLocNew(&filter_out, start, stop);
-        filter_in = filter_in->next;
-   }
-
-   return filter_out;
+    for(; masks; masks = masks->next) {
+        masks->ssr->left    = query_length - 1 - masks->ssr->right;
+        masks->ssr->right   = query_length - 1 - masks->ssr->left;
+    }
 }
 
 /** Calculates the mask locations one context at a time.
@@ -999,7 +1185,6 @@ s_GetFilteringLocationsForOneContext(BLAST_SequenceBlk* query_blk,
     Int2 status = 0;
     Int4 query_length = 0;      /* Length of query described by SeqLocPtr. */
     Int4 context_offset;
-    BlastSeqLoc *filter_slp = NULL;     /* SeqLocPtr computed for filtering. */
     Uint1 *buffer;              /* holds sequence for plus strand or protein. */
 
     const Boolean kIsNucl = (program_number == eBlastTypeBlastn);
@@ -1018,16 +1203,14 @@ s_GetFilteringLocationsForOneContext(BLAST_SequenceBlk* query_blk,
                                query_length, 
                                0, 
                                filter_options, 
-                               &filter_slp, 
+                               filter_out, 
                                blast_message);
     if (status)
          return status;
 
-    if (BlastIsReverseStrand(kIsNucl, context) == TRUE)
-    {  /* Reverse this as it's on minus strand. */
-          BlastSeqLoc* tmp = BlastSeqLocReverse(filter_slp, query_length);
-          filter_slp = BlastSeqLocFree(filter_slp);
-          filter_slp = tmp;
+    if (BlastIsReverseStrand(kIsNucl, context) == TRUE) {  
+        /* Reverse this as it's on minus strand. */
+        BlastSeqLocReverse(*filter_out, query_length);
     }
 
     /* Extract the mask locations corresponding to this query 
@@ -1045,17 +1228,16 @@ s_GetFilteringLocationsForOneContext(BLAST_SequenceBlk* query_blk,
             ASSERT(context < query_blk->lcase_mask->total_size);
             lcase_mask_slp = query_blk->lcase_mask->seqloc_array[context];
             /* Set location list to NULL, to allow safe memory deallocation, 
-              ownership transferred to filter_slp below. */
+              ownership transferred to filter_out below. */
             query_blk->lcase_mask->seqloc_array[context] = NULL;
         }
 
         /* Attach the lower case mask locations to the filter locations and 
            combine them */
-        BlastSeqLocAppend(&filter_slp, lcase_mask_slp);
+        BlastSeqLocAppend(filter_out, lcase_mask_slp);
     }
 
-    *filter_out = BlastSeqLocCombine(filter_slp, 0);
-    filter_slp = BlastSeqLocFree(filter_slp);
+    BlastSeqLocCombine(filter_out, 0);
 
 	return 0;
 }
