@@ -1,4 +1,4 @@
-/* $Id: blast_hits.h,v 1.108 2008/02/14 15:55:42 kazimird Exp $
+/* $Id: blast_hits.h,v 1.110 2009/01/07 16:29:41 kazimird Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -241,8 +241,8 @@ Blast_HSPInit(Int4 query_start, Int4 query_end,
  */
 NCBI_XBLAST_EXPORT
 Boolean 
-Blast_HSPReevaluateWithAmbiguitiesGapped(BlastHSP* hsp, Uint1* query_start, 
-   Uint1* subject_start, const BlastHitSavingParameters* hit_params, 
+Blast_HSPReevaluateWithAmbiguitiesGapped(BlastHSP* hsp, const Uint1* query_start, 
+   const Uint1* subject_start, const BlastHitSavingParameters* hit_params, 
    const BlastScoringParameters* score_params, BlastScoreBlk* sbp);
 
 /** Reevaluate the HSP's score and percent identity after taking into
@@ -259,7 +259,7 @@ Blast_HSPReevaluateWithAmbiguitiesGapped(BlastHSP* hsp, Uint1* query_start,
 NCBI_XBLAST_EXPORT
 Boolean 
 Blast_HSPReevaluateWithAmbiguitiesUngapped(BlastHSP* hsp, 
-   Uint1* query_start, Uint1* subject_start,
+   const Uint1* query_start, const Uint1* subject_start,
    const BlastInitialWordParameters* word_params, 
    BlastScoreBlk* sbp, Boolean translated);
 
@@ -294,9 +294,10 @@ Blast_HSPGetNumIdentities(const Uint1* query,
  *                    HSP length thresholds.
  * @return FALSE if HSP passes the test, TRUE if it should be deleted.
  */ 
+NCBI_XBLAST_EXPORT
 Boolean
 Blast_HSPTestIdentityAndLength(EBlastProgramType program_number, 
-                               BlastHSP* hsp, Uint1* query, Uint1* subject, 
+                               BlastHSP* hsp, const Uint1* query, const Uint1* subject, 
                                const BlastScoringOptions* score_options,
                                const BlastHitSavingOptions* hit_options);
 
@@ -305,7 +306,9 @@ Blast_HSPTestIdentityAndLength(EBlastProgramType program_number,
  * @param options HitSavingoptions object [in]
  * @return number of HSPs to save. 
  */
-Int4 BlastHspNumMax(Boolean gapped_calculation, const BlastHitSavingOptions* options);
+NCBI_XBLAST_EXPORT
+Int4
+BlastHspNumMax(Boolean gapped_calculation, const BlastHitSavingOptions* options);
 
 /** Calculate length of an HSP as length in query plus length of gaps in 
  * query. If gap information is unavailable, return maximum between length in
@@ -365,6 +368,16 @@ NCBI_XBLAST_EXPORT
 void
 Blast_HSPAdjustSubjectOffset(BlastHSP* hsp, Int4 start_shift);
 
+
+/** Returns a buffer with a protein translated from nucleotide.
+ * @target_t SBlastTargetTranslation* with information about translation [in]
+ * @hsp The hit to work on [in]
+ * @translated_length length of the protein sequence [in]
+*/
+NCBI_XBLAST_EXPORT
+const Uint1*
+Blast_HSPGetTargetTranslation(SBlastTargetTranslation* target_t, const BlastHSP* hsp, Int4* translated_length);
+
 /********************************************************************************
           HSPList API
 ********************************************************************************/
@@ -391,9 +404,11 @@ Boolean
 Blast_HSPList_IsEmpty(const BlastHSPList* hsp_list);
 
 /** Returns a duplicate (deep copy) of the given hsp list. */
+NCBI_XBLAST_EXPORT
 BlastHSPList* BlastHSPListDup(const BlastHSPList* hsp_list);
 
 /** Swaps the two HSP lists via structure assignment */
+NCBI_XBLAST_EXPORT
 void Blast_HSPListSwap(BlastHSPList* list1, BlastHSPList* list2);
 
 /** Saves HSP information into a BlastHSPList structure
@@ -452,6 +467,7 @@ Int2 Blast_HSPListGetBitScores(BlastHSPList* hsp_list,
  * @param hsp_list List of HSPs [in] [out]
  * @param sbp Scoring block with statistical parameters [in]
  */
+NCBI_XBLAST_EXPORT
 void Blast_HSPListPHIGetBitScores(BlastHSPList* hsp_list, BlastScoreBlk* sbp);
     
 /** Discard the HSPs above the e-value threshold from the HSP list 
@@ -567,6 +583,7 @@ void Blast_HSPListAdjustOffsets(BlastHSPList* hsp_list, Int4 offset);
  * @param gapped_calculation not an ungapped alignment [in]
  * @param sbp used for round_down Boolean
  */
+NCBI_XBLAST_EXPORT
 void Blast_HSPListAdjustOddBlastnScores(BlastHSPList* hsp_list, 
                                         Boolean gapped_calculation, 
                                         const BlastScoreBlk* sbp);
@@ -764,6 +781,7 @@ struct BlastHSPStream;
  *                      hit list sizes [in]
  * @return The generated collection of HSP results
  */
+NCBI_XBLAST_EXPORT
 BlastHSPResults*
 Blast_HSPResultsFromHSPStream(struct BlastHSPStream* hsp_stream, 
                               size_t num_queries, 
@@ -809,6 +827,7 @@ Blast_HSPResultsFromHSPStreamWithLimit(struct BlastHSPStream* hsp_stream,
  * @return Array of pointers to BlastHSPResults structures, corresponding to 
  *         different pattern occurrences.
  */
+NCBI_XBLAST_EXPORT
 BlastHSPResults** 
 PHIBlast_HSPResultsSplit(const BlastHSPResults* results, 
                          const SPHIQueryInfo* pattern_info);
@@ -818,6 +837,7 @@ PHIBlast_HSPResultsSplit(const BlastHSPResults* results,
  * do not overlap by more than half the pattern match length. 
  * @param query_info Query information structure, containing pattern info. [in]
  */
+NCBI_XBLAST_EXPORT
 Int4
 PhiBlastGetEffectiveNumberOfPatterns(const BlastQueryInfo *query_info);
 
