@@ -1,4 +1,4 @@
-/* $Id: blast_nalookup.c,v 1.8 2008/01/31 23:55:42 kazimird Exp $
+/* $Id: blast_nalookup.c,v 1.10 2008/07/17 17:55:44 kazimird Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -36,7 +36,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] =
-    "$Id: blast_nalookup.c,v 1.8 2008/01/31 23:55:42 kazimird Exp $";
+    "$Id: blast_nalookup.c,v 1.10 2008/07/17 17:55:44 kazimird Exp $";
 #endif                          /* SKIP_DOXYGEN_PROCESSING */
 
 /** bitfield used to detect ambiguities in uncompressed
@@ -317,7 +317,14 @@ static Int4 s_BlastSmallNaLookupFinalize(Int4 **thin_backbone,
     return 0;
 }
 
-static BlastSeqLoc* s_SeqLocListInvert(BlastSeqLoc* locations, Int4 length)
+/** Changes the list of locations into a list of 
+   the intervals between locations (the inverse).
+   @param locations input list [in]
+   @param length (query) sequence length [in]
+   @return inverted BlastSeqLoc
+*/
+
+static BlastSeqLoc* s_SeqLocListInvert(const BlastSeqLoc* locations, Int4 length)
 {
      BlastSeqLoc* retval = NULL;
      BlastSeqLoc* tail = NULL;  /* Tail of the list. */
@@ -671,7 +678,10 @@ s_FillDiscMBTable(BLAST_SequenceBlk* query, BlastSeqLoc* location,
       and the optimal templates for one combination of word size
       and template length. */
    if (kTwoTemplates) {
-      second_template_type = mb_lt->second_template_type = template_type + 1;
+      /* Use the temporaray to avoid annoying ICC warning. */
+      int temp_int = template_type + 1;
+      second_template_type = 
+           mb_lt->second_template_type = (EDiscTemplateType) temp_int;
 
       mb_lt->hashtable2 = (Int4*)calloc(mb_lt->hashsize, sizeof(Int4));
       mb_lt->next_pos2 = (Int4*)calloc(query->length + 1, sizeof(Int4));
@@ -786,15 +796,13 @@ s_FillDiscMBTable(BLAST_SequenceBlk* query, BlastSeqLoc* location,
  * @param query the query sequence [in]
  * @param location locations on the query to be indexed in table [in]
  * @param mb_lt the (already allocated) megablast lookup table structure [in|out]
- * @param lookup_options specifies the word_size [in]
  * @return zero on success, negative number on failure. 
  */
 
 static Int2 
 s_FillContigMBTable(BLAST_SequenceBlk* query, 
         BlastSeqLoc* location,
-        BlastMBLookupTable* mb_lt, 
-        const LookupTableOptions* lookup_options)
+        BlastMBLookupTable* mb_lt) 
 
 {
    BlastSeqLoc* loc;
@@ -972,7 +980,7 @@ Int2 BlastMBLookupTableNew(BLAST_SequenceBlk* query, BlastSeqLoc* location,
    else {
         /* contiguous megablast */
         mb_lt->scan_step = mb_lt->word_length - mb_lt->lut_word_length + 1;
-        status = s_FillContigMBTable(query, location, mb_lt, lookup_options);
+        status = s_FillContigMBTable(query, location, mb_lt);
    }
 
    if (status > 0) {
