@@ -1,4 +1,4 @@
-/* $Id: blast_options.c,v 1.207 2007/10/25 15:55:36 kazimird Exp $
+/* $Id: blast_options.c,v 1.211 2009/02/03 18:14:30 kazimird Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -34,7 +34,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: blast_options.c,v 1.207 2007/10/25 15:55:36 kazimird Exp $";
+    "$Id: blast_options.c,v 1.211 2009/02/03 18:14:30 kazimird Exp $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/blast_options.h>
@@ -91,6 +91,33 @@ Int2 SSegOptionsNew(SSegOptions* *seg_options)
     return 0;
 }
 
+Int2 SWindowMaskerOptionsNew(SWindowMaskerOptions ** winmask_options)
+{
+    if (winmask_options) {
+        *winmask_options = (SWindowMaskerOptions*) calloc(1, sizeof(SWindowMaskerOptions));
+        if (*winmask_options == NULL)
+            return BLASTERR_MEMORY;
+        
+        (*winmask_options)->taxid = 0;
+        (*winmask_options)->database = NULL;
+        return 0;
+    }
+    return 1;
+}
+
+SWindowMaskerOptions* SWindowMaskerOptionsFree(SWindowMaskerOptions* winmask_options)
+{
+    if (winmask_options)
+    {
+        if (winmask_options->database)
+        {
+            sfree(winmask_options->database);
+        }
+        sfree(winmask_options);
+    }
+    return NULL;
+}
+
 SRepeatFilterOptions* SRepeatFilterOptionsFree(SRepeatFilterOptions* repeat_options)
 {
     if (repeat_options)
@@ -133,13 +160,37 @@ Int2 SRepeatFilterOptionsResetDB(SRepeatFilterOptions* *repeat_options, const ch
     return status;
 }
 
+Int2 SWindowMaskerOptionsResetDB(SWindowMaskerOptions ** winmask_options, const char* db)
+{
+    Int2 status=0;
+    
+    if (*winmask_options == NULL)
+        status = SWindowMaskerOptionsNew(winmask_options);
+    
+    if (status)
+        return status;
+    
+    sfree((*winmask_options)->database);
+    
+    if (db) {
+        (*winmask_options)->database = strdup(db);
+    }
+    
+    return status;
+}
+
 SBlastFilterOptions* SBlastFilterOptionsFree(SBlastFilterOptions* filter_options)
 {
     if (filter_options)
     {
-        filter_options->dustOptions = SDustOptionsFree(filter_options->dustOptions);
-        filter_options->segOptions = SSegOptionsFree(filter_options->segOptions);
-        filter_options->repeatFilterOptions = SRepeatFilterOptionsFree(filter_options->repeatFilterOptions);
+        filter_options->dustOptions = 
+            SDustOptionsFree(filter_options->dustOptions);
+        filter_options->segOptions = 
+            SSegOptionsFree(filter_options->segOptions);
+        filter_options->repeatFilterOptions = 
+            SRepeatFilterOptionsFree(filter_options->repeatFilterOptions);
+        filter_options->windowMaskerOptions =
+            SWindowMaskerOptionsFree(filter_options->windowMaskerOptions);
         sfree(filter_options);
     }
 
@@ -272,6 +323,46 @@ static SRepeatFilterOptions* s_MergeRepeatOptions(const SRepeatFilterOptions* op
       return retval;
 }
 
+/** Merges together two sets of window masker options, choosing the most non-default one.
+ * 
+ * @param opt1 first set to be merged [in]
+ * @param opt2 second set to be merged [in]
+ * @return the merged options.
+ */
+static SWindowMaskerOptions*
+s_MergeWindowMaskerOptions(const SWindowMaskerOptions* opt1,
+                           const SWindowMaskerOptions* opt2)
+{
+    SWindowMaskerOptions* retval = NULL;
+    const SWindowMaskerOptions* src = NULL;
+    Boolean have1 = FALSE, have2 = FALSE;
+    
+    have1 = opt1 && (opt1->database || opt1->taxid);
+    have2 = opt2 && (opt2->database || opt2->taxid);
+    
+    if (! (have1 || have2))
+        return NULL;
+    
+    if (have1 && ! have2) {
+        src = opt1;
+    } else if (! have1 && have2) {
+        src = opt2;
+    } else {
+        // We have data structures with some kind of content, so
+        // prefer structure 2 as repeat filter options do.
+        src = opt2;
+    }
+    
+    ASSERT(src);
+    ASSERT(src->database || src->taxid);
+    
+    SWindowMaskerOptionsNew(&retval);
+    SWindowMaskerOptionsResetDB(& retval, src->database);
+    retval->taxid = src->taxid;
+    
+    return retval;
+}
+
 Int2 SBlastFilterOptionsMerge(SBlastFilterOptions** combined, const SBlastFilterOptions* opt1,
        const SBlastFilterOptions* opt2)
 {
@@ -298,11 +389,22 @@ Int2 SBlastFilterOptionsMerge(SBlastFilterOptions** combined, const SBlastFilter
          s_MergeSegOptions(opt1 ? opt1->segOptions : NULL, opt2 ? opt2->segOptions : NULL);
      retval->repeatFilterOptions = 
          s_MergeRepeatOptions(opt1 ? opt1->repeatFilterOptions : NULL, opt2 ? opt2->repeatFilterOptions : NULL);
+     retval->windowMaskerOptions = 
+         s_MergeWindowMaskerOptions(opt1 ? opt1->windowMaskerOptions : NULL, opt2 ? opt2->windowMaskerOptions : NULL);
 
      return 0;
 }
 
-
+Boolean SBlastFilterOptionsNoFiltering(const SBlastFilterOptions* filter_options)
+{
+       if (filter_options == NULL)
+          return TRUE;
+      
+       return filter_options->dustOptions == NULL &&
+           filter_options->segOptions == NULL &&
+           filter_options->repeatFilterOptions == NULL &&
+           filter_options->windowMaskerOptions == NULL;
+}
 
 Boolean SBlastFilterOptionsMaskAtHash(const SBlastFilterOptions* filter_options)
 {
@@ -714,16 +816,14 @@ BlastScoringOptionsValidate(EBlastProgramType program_number,
 	if (options == NULL)
 		return BLASTERR_INVALIDPARAM;
 
-   if (program_number == eBlastTypeTblastx && 
-              options->gapped_calculation)
-   {
-      Blast_MessageWrite(blast_msg, eBlastSevError, kBlastMessageNoContext,
-         "Gapped search is not allowed for tblastx");
+        if (program_number == eBlastTypeTblastx && options->gapped_calculation)
+        {
+            Blast_MessageWrite(blast_msg, eBlastSevError, kBlastMessageNoContext,
+               "Gapped search is not allowed for tblastx");
 		return BLASTERR_OPTION_PROGRAM_INVALID;
-   }
+        }
 
-	if (program_number == eBlastTypeBlastn ||
-        program_number == eBlastTypePhiBlastn)
+	if (program_number == eBlastTypeBlastn || program_number == eBlastTypePhiBlastn)
 	{
 		if (options->penalty >= 0)
 		{
@@ -731,6 +831,14 @@ BlastScoringOptionsValidate(EBlastProgramType program_number,
                             "BLASTN penalty must be negative");
 			return BLASTERR_OPTION_VALUE_INVALID;
 		}
+
+                if (options->gapped_calculation && !BLAST_CheckRewardPenaltyScores(options->reward, options->penalty))
+                {
+			Blast_MessageWrite(blast_msg, eBlastSevWarning, kBlastMessageNoContext,
+                            "BLASTN reward/penalty combination not supported for gapped search");
+			return BLASTERR_OPTION_VALUE_INVALID;
+                }
+
                 if (options->gapped_calculation && options->gap_open > 0 && options->gap_extend == 0) 
                 {
                         Blast_MessageWrite(blast_msg, eBlastSevWarning, kBlastMessageNoContext,
@@ -770,16 +878,14 @@ BlastScoringOptionsValidate(EBlastProgramType program_number,
 	       }
 	}
 
-	if (program_number != eBlastTypeBlastx && 
-       program_number != eBlastTypeTblastn && options->is_ooframe)
+	if (program_number != eBlastTypeBlastx && program_number != eBlastTypeTblastn && options->is_ooframe)
 	{
-          Blast_MessageWrite(blast_msg, eBlastSevWarning, kBlastMessageNoContext,
-             "Out-of-frame only permitted for blastx and tblastn");
-          return  BLASTERR_OPTION_PROGRAM_INVALID;
+            Blast_MessageWrite(blast_msg, eBlastSevWarning, kBlastMessageNoContext,
+               "Out-of-frame only permitted for blastx and tblastn");
+            return  BLASTERR_OPTION_PROGRAM_INVALID;
 	}
 
 	return 0;
-
 }
 
 Int2 
@@ -1343,9 +1449,9 @@ Int2 PSIBlastOptionsValidate(const PSIBlastOptions* psi_options,
         return retval;
     }
 
-    if (psi_options->pseudo_count <= 0) {
+    if (psi_options->pseudo_count < 0) {
         Blast_MessageWrite(blast_msg, eBlastSevError, kBlastMessageNoContext,
-                           "Pseudo count must be greater than 0");
+                           "Pseudo count must be greater than or equal to 0");
         return retval;
     }
 
