@@ -1,4 +1,4 @@
-/* $Id: blast_options.h,v 1.146 2007/05/22 20:55:36 kazimird Exp $
+/* $Id: blast_options.h,v 1.152 2008/11/03 20:59:44 kazimird Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -96,7 +96,8 @@ extern "C" {
  *  will go into the BLAST lookup table when it is generated 
  */
 #define BLAST_WORD_THRESHOLD_BLASTP 11 /**< default neighboring threshold
-                                         (blastp/rpsblast) */
+                                         (blastp and for rpsblast at RPS-BLAST
+                                         database creation time) */
 #define BLAST_WORD_THRESHOLD_BLASTN 0 /**< default threshold (blastn) */
 #define BLAST_WORD_THRESHOLD_BLASTX 12 /**< default threshold (blastx) */
 #define BLAST_WORD_THRESHOLD_TBLASTN 13 /**< default neighboring threshold 
@@ -132,7 +133,7 @@ extern "C" {
 /** default dropoff for the final gapped extension with traceback */
 #define BLAST_GAP_X_DROPOFF_FINAL_PROT 25 /**< default dropoff (all protein-
                                                based gapped extensions) */
-#define BLAST_GAP_X_DROPOFF_FINAL_NUCL 50 /**< default dropoff for nucleotide
+#define BLAST_GAP_X_DROPOFF_FINAL_NUCL 100 /**< default dropoff for nucleotide
                                                gapped extensions) */
 #define BLAST_GAP_X_DROPOFF_FINAL_TBLASTX 0 /**< default dropoff for tblastx */
 
@@ -147,7 +148,7 @@ extern "C" {
                                   for */
 /** Defaults for PSI-BLAST options */
 #define PSI_INCLUSION_ETHRESH 0.002 /**< Inclusion threshold for PSI BLAST */
-#define PSI_PSEUDO_COUNT_CONST 9 /**< Pseudo-count constant for PSI-BLAST */
+#define PSI_PSEUDO_COUNT_CONST 0 /**< Pseudo-count constant for PSI-BLAST */
 
 /** Default genetic code for query and/or database */
 #define BLAST_GENETIC_CODE 1  /**< Use the standard genetic code for converting
@@ -184,8 +185,6 @@ typedef struct LookupTableOptions {
    Int4 mb_template_length; /**< Length of the discontiguous words */
    Int4 mb_template_type; /**< Type of a discontiguous word template */
    char* phi_pattern;  /**< PHI-BLAST pattern */
-   Int4 max_num_patterns; /**< Maximal number of patterns allowed for 
-                             PHI-BLAST */
    EBlastProgramType program_number; /**< indicates blastn, blastp, etc. */
 } LookupTableOptions;
 
@@ -216,6 +215,15 @@ typedef struct SRepeatFilterOptions {
     char* database;   /**< Nucleotide database for mini BLAST search. */
 } SRepeatFilterOptions;
 
+/** Filtering options for organism-specific filtering with Window
+    Masker.  The taxid and filename are alternative means of choosing
+    which Window Masker database to use.
+ */
+typedef struct SWindowMaskerOptions {
+    int          taxid;    /**< Select masking database for this TaxID. */
+    const char * database; /**< Use winmasker database at this location. */
+} SWindowMaskerOptions;
+
 /** All filtering options */
 typedef struct SBlastFilterOptions {
     Boolean mask_at_hash;         /**< mask query only for lookup table creation */
@@ -223,6 +231,7 @@ typedef struct SBlastFilterOptions {
     SSegOptions* segOptions;      /**< low-complexity filtering for proteins sequences 
             (includes translated nucleotides). */
     SRepeatFilterOptions* repeatFilterOptions;  /**< for organism specific repeat filtering. */
+    SWindowMaskerOptions* windowMaskerOptions;  /**< organism specific filtering with window masker. */
 } SBlastFilterOptions;
 
 
@@ -243,8 +252,6 @@ typedef struct BlastInitialWordOptions {
    double gap_trigger; /**< Score in bits for starting gapped extension */
    Int4 window_size; /**< Maximal allowed distance between 2 hits in case 2 
                         hits are required to trigger the extension */
-   Boolean ungapped_extension; /**< Should the ungapped extension be 
-                                  performed? */
    double x_dropoff; /**< X-dropoff value (in bits) for the ungapped 
                          extension */
    EBlastProgramType program_number; /**< indicates blastn, blastp, etc. */
@@ -308,8 +315,6 @@ typedef struct BlastHitSavingOptions {
    Int4 hsp_num_max; /**< Maximal number of HSPs to save for one database 
                         sequence */
    Int4 total_hsp_limit; /**< Maximal total number of HSPs to keep */
-   Int4 hsp_range_max; /**< Maximal number of HSPs to save in a region: 
-                          used for culling only */
    Int4 culling_limit; /**< If the query range of an HSP is contained in
                             at least this many higher-scoring HSPs, throw
                             away the HSP as redundant (turned off if zero) */
@@ -426,7 +431,7 @@ typedef struct PSIBlastOptions {
  *  to a subset.
  */
 typedef struct BlastDatabaseOptions {
-   Int4 genetic_code;  /**< Genetic code to use for translation, 
+   Int4 genetic_code;   /**< Genetic code to use for translation, 
                              tblast[nx] only */
 } BlastDatabaseOptions;
 
@@ -441,24 +446,28 @@ typedef struct BlastDatabaseOptions {
  * @param dust_options object to free
  * @return NULL pointer
  */
+NCBI_XBLAST_EXPORT
 SDustOptions* SDustOptionsFree(SDustOptions* dust_options);
 
 /** Allocates memory for SDustOptions, fills in defaults.
  * @param dust_options options that are being returned [in|out]
  * @return zero on sucess
  */
+NCBI_XBLAST_EXPORT
 Int2 SDustOptionsNew(SDustOptions* *dust_options);
 
 /** Frees SSegOptions.
  * @param seg_options object to free [in]
  * @return NULL pointer
  */
+NCBI_XBLAST_EXPORT
 SSegOptions* SSegOptionsFree(SSegOptions* seg_options);
 
 /** Allocates memory for SSegOptions, fills in defaults. [in|out]
  * @param seg_options options that are being returned [in|out]
  * @return zero on sucess
  */
+NCBI_XBLAST_EXPORT
 Int2 SSegOptionsNew(SSegOptions* *seg_options);
 
 /** Resets name of db for repeat filtering.
@@ -466,24 +475,51 @@ Int2 SSegOptionsNew(SSegOptions* *seg_options);
  * @param dbname name of the database(s) [in]
  * @return zero on sucess
  */
+NCBI_XBLAST_EXPORT
 Int2 SRepeatFilterOptionsResetDB(SRepeatFilterOptions* *repeat_options, const char* dbname);
 
+/** Resets name of db for window masker filtering.
+ * @param winmask_options options block constaining field to be reset [in|out]
+ * @param dbname name of the database(s) [in]
+ * @return zero on sucess
+ */
+NCBI_XBLAST_EXPORT
+Int2 SWindowMaskerOptionsResetDB(SWindowMaskerOptions ** winmask_options,
+                                 const char            * dbname);
+
 /** Frees SRepeatFilterOptions.
  * @param repeat_options object to free [in]
  * @return NULL pointer
  */
+NCBI_XBLAST_EXPORT
 SRepeatFilterOptions* SRepeatFilterOptionsFree(SRepeatFilterOptions* repeat_options);
 
+/** Frees SWindowMaskerOptions.
+ * @param winmask_options object to free [in]
+ * @return NULL pointer
+ */
+NCBI_XBLAST_EXPORT SWindowMaskerOptions*
+SWindowMaskerOptionsFree(SWindowMaskerOptions * winmask_options);
+
 /** Allocates memory for SRepeatFilterOptions, fills in defaults.
  * @param repeat_options options that are being returned [in|out]
  * @return zero on sucess
  */
-Int2 SRepeatFilterOptionsNew(SRepeatFilterOptions* *repeat_options);
+NCBI_XBLAST_EXPORT
+Int2 SRepeatFilterOptionsNew(SRepeatFilterOptions ** repeat_options);
+
+/** Allocates memory for SWindowMaskerOptions, fills in defaults.
+ * @param winmask_options options that are being returned [in|out]
+ * @return zero on sucess
+ */
+NCBI_XBLAST_EXPORT
+Int2 SWindowMaskerOptionsNew(SWindowMaskerOptions ** winmask_options);
 
 /** Frees SBlastFilterOptions and all subservient structures.
  * @param filter_options object to free
  * @return NULL pointer
  */
+NCBI_XBLAST_EXPORT
 SBlastFilterOptions* SBlastFilterOptionsFree(SBlastFilterOptions* filter_options);
 
 /**  Merges two sets of options together, taking the non-default one as preferred.  if
@@ -493,6 +529,7 @@ SBlastFilterOptions* SBlastFilterOptionsFree(SBlastFilterOptions* filter_options
  * @param opt2 second set of options [in]
  * @return zero on success. 
  */
+NCBI_XBLAST_EXPORT
 Int2 SBlastFilterOptionsMerge(SBlastFilterOptions** combined, const SBlastFilterOptions* opt1,
        const SBlastFilterOptions* opt2);
 
@@ -510,12 +547,22 @@ typedef enum EFilterOptions {
  * @param type specify either dust or seg (now) with EFilterOptions [in]
  * @return zero on sucess
  */
+NCBI_XBLAST_EXPORT
 Int2 SBlastFilterOptionsNew(SBlastFilterOptions* *filter_options, EFilterOptions type);
 
+/** Queries whether no masking is required
+ * @param filter_options the object to be queried [in]
+ * @return TRUE if no filtering is required or argument is NULL, FALSE
+ * otherwise
+ */
+NCBI_XBLAST_EXPORT
+Boolean SBlastFilterOptionsNoFiltering(const SBlastFilterOptions* filter_options);
+
 /** Queries whether masking should be done only for the lookup table or for the entire search.
  * @param filter_options the object to be queried [in]
  * @return TRUE or FALSE, FALSE if filter_options is NULL.
  */
+NCBI_XBLAST_EXPORT
 Boolean SBlastFilterOptionsMaskAtHash(const SBlastFilterOptions* filter_options);
 
 /** Validates filter options to ensure that program and options are consistent
@@ -525,6 +572,7 @@ Boolean SBlastFilterOptionsMaskAtHash(const SBlastFilterOptions* filter_options)
  * @param blast_message error or warning (optional) [out] 
  * @return zero on success
  */
+NCBI_XBLAST_EXPORT
 Int2 SBlastFilterOptionsValidate(EBlastProgramType program_number, const SBlastFilterOptions* filter_options, 
        Blast_Message* *blast_message);
 
@@ -604,10 +652,11 @@ BlastExtensionOptionsFree(BlastExtensionOptions* options);
 /** Allocate memory for BlastExtensionOptions and fill with default values.
  * @param program Program number (blastn, blastp, etc.) [in]
  * @param options The options that are being returned [out]
+ * @param gapped The search is gapped [in]
 */
 NCBI_XBLAST_EXPORT
 Int2
-BlastExtensionOptionsNew(EBlastProgramType program, BlastExtensionOptions* *options);
+BlastExtensionOptionsNew(EBlastProgramType program, BlastExtensionOptions* *options, Boolean gapped);
 
 /** Fill non-default values in the BlastExtensionOptions structure.
  * @param options The options structure [in] [out]
@@ -688,6 +737,7 @@ Int2 BlastScoringOptionsDup(BlastScoringOptions* *new_opt, const BlastScoringOpt
  * @param matrix_name New matrix name. If NULL, old matrix name is left 
  *                    as is. [in]
  */
+NCBI_XBLAST_EXPORT
 Int2 BlastScoringOptionsSetMatrix(BlastScoringOptions* opts,
                                   const char* matrix_name);
 
@@ -902,6 +952,7 @@ Int2 BLAST_ValidateOptions(EBlastProgramType program_number,
  * @param threshold returns suggested value [in|out]
  * @return zero on success
  */
+NCBI_XBLAST_EXPORT
 Int2 BLAST_GetSuggestedThreshold(EBlastProgramType program_number, 
                                  const char* matrixName, 
                                  double* threshold);
@@ -914,6 +965,7 @@ Int2 BLAST_GetSuggestedThreshold(EBlastProgramType program_number,
  * @param window_size returns suggested value [in|out]
  * @return zero on success
  */
+NCBI_XBLAST_EXPORT
 Int2 BLAST_GetSuggestedWindowSize(EBlastProgramType program_number, 
                                  const char* matrixName, 
                                  Int4* window_size);
