@@ -1,6 +1,6 @@
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] =
-    "$Id: blast_psi.c,v 1.33 2006/11/21 17:07:09 papadopo Exp $";
+    "$Id: blast_psi.c,v 1.36 2008/04/15 15:55:45 kazimird Exp $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 /* ===========================================================================
  *
@@ -181,7 +181,9 @@ PSICreatePssmWithDiagnostics(const PSIMsa* msap,                    /* [in] */
     }
 
     status = _PSIComputeFreqRatios(msa, seq_weights, sbp, aligned_block, 
-                                   options->pseudo_count, internal_pssm);
+                                   options->pseudo_count, 
+                                   options->nsg_compatibility_mode,
+                                   internal_pssm);
     if (status != PSI_SUCCESS) {
         s_PSICreatePssmCleanUp(pssm, packed_msa, msa, aligned_block, 
                                seq_weights, internal_pssm);
@@ -357,6 +359,9 @@ s_PSISavePssm(const _PSIInternalPssmData* internal_pssm,
     pssm->lambda = sbp->kbp_gap_psi[0]->Lambda;
     pssm->kappa = sbp->kbp_gap_psi[0]->K;
     pssm->h = sbp->kbp_gap_psi[0]->H;
+    pssm->ung_lambda = sbp->kbp_psi[0]->Lambda;
+    pssm->ung_kappa = sbp->kbp_psi[0]->K;
+    pssm->ung_h = sbp->kbp_psi[0]->H;
 }
 
 /****************************************************************************/
@@ -401,6 +406,14 @@ PSIMsaNew(const PSIMsaDimensions* dimensions)
         }
     }
 
+#ifdef DEBUG_PSSM_ENGINE
+    retval->seqinfo = (PSISeqInfo*) calloc(dimensions->num_seqs + 1,
+                                           sizeof(PSISeqInfo));
+    if ( !retval->seqinfo ) {
+        return PSIMsaFree(retval);
+    }
+#endif /* DEBUG_PSSM_ENGINE */
+
     return retval;
 }
 
@@ -421,6 +434,12 @@ PSIMsaFree(PSIMsa* msa)
         sfree(msa->dimensions);
     }
 
+#ifdef DEBUG_PSSM_ENGINE
+    if ( msa->seqinfo ) {
+        sfree(msa->seqinfo);
+    }
+#endif /* DEBUG_PSSM_ENGINE */
+
     sfree(msa);
 
     return NULL;
@@ -447,6 +466,9 @@ PSIMatrixNew(Uint4 query_length, Uint4 alphabet_size)
     retval->lambda = 0.0;
     retval->kappa = 0.0;
     retval->h = 0.0;
+    retval->ung_lambda = 0.0;
+    retval->ung_kappa = 0.0;
+    retval->ung_h = 0.0;
 
     return retval;
 }
@@ -546,6 +568,28 @@ PSIDiagnosticsResponseNew(Uint4 query_length, Uint4 alphabet_size,
         }
     }
 
+    if (wants->sigma) {
+        retval->sigma = (double*) calloc(query_length, sizeof(double));
+        if ( !retval->sigma ) {
+            return PSIDiagnosticsResponseFree(retval);
+        }
+    }
+
+    if (wants->interval_sizes) {
+        retval->interval_sizes = (Uint4*) calloc(query_length, sizeof(Uint4));
+        if ( !retval->interval_sizes ) {
+            return PSIDiagnosticsResponseFree(retval);
+        }
+    }
+
+    if (wants->num_matching_seqs) {
+        retval->num_matching_seqs = 
+            (Uint4*) calloc(query_length, sizeof(Uint4));
+        if ( !retval->num_matching_seqs ) {
+            return PSIDiagnosticsResponseFree(retval);
+        }
+    }
+
     return retval;
 }
 
