@@ -1,6 +1,6 @@
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] =
-    "$Id: blast_psi_priv.c,v 1.67 2006/12/05 22:03:00 camacho Exp $";
+    "$Id: blast_psi_priv.c,v 1.72 2009/01/27 15:29:32 kazimird Exp $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 /* ===========================================================================
  *
@@ -41,6 +41,8 @@ static char const rcsid[] =
 #include <algo/blast/core/blast_util.h>
 #include "blast_dynarray.h"
 
+#include <algo/blast/composition_adjustment/matrix_frequency_data.h>
+
 /****************************************************************************/
 /* Use the following #define's to enable/disable functionality */
 
@@ -227,7 +229,7 @@ _PSIPackedMsaGetNumberOfAlignedSeqs(const _PSIPackedMsa* msa)
 
 /****************************************************************************/
 
-#if _DEBUG
+#ifdef DEBUG_PSSM_ENGINE 
 char GetResidue(char input)
 {
     return input > BLASTAA_SIZE ? '?' : NCBISTDAA_TO_AMINOACID[(int)input];
@@ -236,16 +238,26 @@ char GetResidue(char input)
 void
 PrintMsa(const char* filename, const PSIMsa* msa)
 {
-    Uint4 i, j;
     FILE* fp = NULL;
-
     ASSERT(msa);
     ASSERT(filename);
-
     fp = fopen(filename, "w");
+    PrintMsaFP(fp, msa);
+    fclose(fp);
+}
+
+void
+PrintMsaFP(FILE* fp, const PSIMsa* msa)
+{
+    Uint4 i, j;
+
+    ASSERT(msa);
 
     for (i = 0; i < msa->dimensions->num_seqs + 1; i++) {
-        fprintf(fp, "%3d\t", i);
+        fprintf(fp, "%3d\tGI=%10d\tEvalue=%2.0le\tBitScore=%4.1f\t", i,
+                msa->seqinfo[i].gi,
+                msa->seqinfo[i].evalue,
+                msa->seqinfo[i].bit_score);
         for (j = 0; j < msa->dimensions->query_length; j++) {
             if (msa->data[i][j].is_aligned) {
                 fprintf(fp, "%c", GetResidue(msa->data[i][j].letter));
@@ -255,19 +267,15 @@ PrintMsa(const char* filename, const PSIMsa* msa)
         }
         fprintf(fp, "\n");
     }
-    fclose(fp);
 }
 
 void
-__printMsa(const char* filename, const _PSIPackedMsa* msa)
+__printPackedMsaFP(FILE* fp, const _PSIPackedMsa* msa)
 {
     Uint4 i, j;
-    FILE* fp = NULL;
 
     ASSERT(msa);
-    ASSERT(filename);
-
-    fp = fopen(filename, "w");
+    ASSERT(fp);
 
     for (i = 0; i < msa->dimensions->num_seqs + 1; i++) {
         fprintf(fp, "%3d\t", i);
@@ -285,9 +293,19 @@ __printMsa(const char* filename, const _PSIPackedMsa* msa)
         }
         fprintf(fp, "\n");
     }
+}
+
+void
+__printPackedMsa(const char* filename, const _PSIPackedMsa* msa)
+{
+    FILE* fp = NULL;
+    ASSERT(msa);
+    ASSERT(filename);
+    fp = fopen(filename, "w");
+    __printPackedMsaFP(fp, msa);
     fclose(fp);
 }
-#endif /* _DEBUG */
+#endif /* DEBUG_PSSM_ENGINE */
 
 _PSIMsa*
 _PSIMsaNew(const _PSIPackedMsa* msa, Uint4 alphabet_size)
@@ -522,6 +540,7 @@ _PSIAlignedBlockFree(_PSIAlignedBlock* aligned_blocks)
     return NULL;
 }
 
+#define EFFECTIVE_ALPHABET 20 /**< size of alphabet used for pseudocounts calculations */
 _PSISequenceWeights*
 _PSISequenceWeightsNew(const PSIMsaDimensions* dimensions, 
                        const BlastScoreBlk* sbp)
@@ -573,6 +592,20 @@ _PSISequenceWeightsNew(const PSIMsaDimensions* dimensions,
         return _PSISequenceWeightsFree(retval);
     }
 
+    retval->posDistinctDistrib = (int**) 
+        _PSIAllocateMatrix(1+dimensions->query_length, 
+                           1+EFFECTIVE_ALPHABET, 
+                           sizeof(int));
+    retval->posDistinctDistrib_size = 1+dimensions->query_length;
+    if ( !retval->posDistinctDistrib ) {
+        return _PSISequenceWeightsFree(retval);
+    }
+
+    retval->posNumParticipating = (int*) calloc(1+dimensions->query_length, sizeof(int));
+    if ( !retval->posNumParticipating ) {
+        return _PSISequenceWeightsFree(retval);
+    }
+
     return retval;
 }
 
@@ -608,6 +641,15 @@ _PSISequenceWeightsFree(_PSISequenceWeights* seq_weights)
         sfree(seq_weights->gapless_column_weights);
     }
 
+    if (seq_weights->posDistinctDistrib) {
+        _PSIDeallocateMatrix((void**) seq_weights->posDistinctDistrib,
+                             seq_weights->posDistinctDistrib_size);
+    }
+
+    if (seq_weights->posNumParticipating) {
+        sfree(seq_weights->posNumParticipating);
+    }
+
     sfree(seq_weights);
 
     return NULL;
@@ -1488,11 +1530,16 @@ _PSIComputeSequenceWeights(const _PSIMsa* msa,                      /* [in] */
             _PSICalculateNormalizedSequenceWeights(msa, aligned_blocks, pos, 
                                                    aligned_seqs, seq_weights);
         } else {
+            int index;
             seq_weights->sigma[pos] = seq_weights->sigma[pos-1];
+            for (index = 0; index <= EFFECTIVE_ALPHABET; index++) {
+               seq_weights->posDistinctDistrib[pos][index] = seq_weights->posDistinctDistrib[pos-1][index];
+            }
             /* seq_weights->norm_seq_weights are unchanged from the previous
              * iteration, leaving them ready to be used in
              * _PSICalculateMatchWeights */
         }
+        seq_weights->posNumParticipating[pos] = aligned_seqs->num_used;
 
         /* Uses seq_weights->norm_seq_weights to populate match_weights */
         _PSICalculateMatchWeights(msa, pos, aligned_seqs, seq_weights);
@@ -1527,6 +1574,8 @@ _PSICalculateNormalizedSequenceWeights(
     const SDynamicUint4Array* aligned_seqs,             /* [in] */
     _PSISequenceWeights* seq_weights)       /* [out] norm_seq_weights, sigma */
 {
+    const Uint1 kGapResidue = AMINOACID_TO_NCBISTDAA['-'];
+    const Uint1 kXResidue = AMINOACID_TO_NCBISTDAA['X'];
     /* Index into aligned block for requested position */
     Uint4 i = 0;         
 
@@ -1563,6 +1612,7 @@ _PSICalculateNormalizedSequenceWeights(
         /* number of distinct residues found in a column of the alignment
          * extent correspoding to a query position */
         Uint4 num_distinct_residues_for_column = 0; 
+        Uint4 num_local_std_letters = 0; 
 
         /* Assert that the alignment extents have sane values */
         ASSERT(i < msa->dimensions->query_length);
@@ -1575,11 +1625,15 @@ _PSICalculateNormalizedSequenceWeights(
 
             if (residue_counts_for_column[kResidue] == 0) {
                 num_distinct_residues_for_column++;
+                if (kResidue != kGapResidue && kResidue != kXResidue)
+                    num_local_std_letters++;
             }
             residue_counts_for_column[kResidue]++;
         }
 
         sigma += num_distinct_residues_for_column;
+        num_local_std_letters = MIN(num_local_std_letters,EFFECTIVE_ALPHABET);
+        seq_weights->posDistinctDistrib[position][num_local_std_letters]++; 
         if (num_distinct_residues_for_column > 1) {
             /* num_distinct_residues_for_column == 1 means that all residues in
              * that column of the alignment extent are the same residue */
@@ -1792,15 +1846,62 @@ _PSICheckSequenceWeights(const _PSIMsa* msa,
     return PSI_SUCCESS;
 }
 
+/** initialize the expected number of observations
+  use background probabilities for this matrix
+  Calculate exp. # of distinct aa's as a function of independent trials
+  copy of posit.c:initializeExpNumObservations
+
+  @param expno table of expectations [out]
+  @param backgroundProbabilities residue background probs [in]
+*/
+static void s_initializeExpNumObservations(double *expno, const double *backgroundProbabilities);
+
+/** A method to estimate the effetive number of observations
+  in the interval for the specified columnNumber 
+  copy of posit.c:effectiveObservations
+  @param align_blk data structure describing the aligned blocks [in]
+  @param seq_weights data structure of sequence weights [in]
+  @param columnNumber column in the PSSM [in]
+  @param queryLength length of the query sequence
+  @param expno table of expectations [in]
+*/
+static double s_effectiveObservations(const _PSIAlignedBlock *align_blk,
+                                         const _PSISequenceWeights* seq_weights,
+                                         int columnNumber, int queryLength,
+                                         const double *expno);
+
+
+/** copy of posit.c:columnSpecificPseudocounts 
+ @param posSearch data structure of sequence weights [in]
+ @param columnNumber column in the PSSM [in]
+ @param backgroundProbabilities residue background probs [in]
+ @param observations for each column an estimate of observed residues [in]
+*/
+static double s_columnSpecificPseudocounts(const _PSISequenceWeights *posSearch,
+                                  int columnNumber,
+                                  const double *backgroundProbabilities,
+                                  const double observations);
+
+#define MAX_IND_OBSERVATIONS  400 /**< max number of independent observation for pseudocount calculation */
+#define PSEUDO_MAX 1000000 /**< effective infinity */
 /****************************************************************************/
 /******* Compute residue frequencies stage of PSSM creation *****************/
 
+#if defined(__INTEL_COMPILER)  &&  __INTEL_COMPILER >= 1000 \
+  &&  defined(__i386__)  &&  defined(__OPTIMIZE__)  &&  defined(__PIC__)
+/* Kludge: avoid hitting the error "hidden symbol `__svml_log2' in
+   /opt/intel/cc/10.1.021/lib/libsvml.a(svml_dln2.o) is referenced by DSO"
+   when building dynamic libraries of the Toolkit but proceeding to link
+   ICC's libraries staticly and only into applications. */
+#  pragma optimization_level 2
+#endif
 int
 _PSIComputeFreqRatios(const _PSIMsa* msa,
                       const _PSISequenceWeights* seq_weights,
                       const BlastScoreBlk* sbp,
                       const _PSIAlignedBlock* aligned_blocks,
                       Int4 pseudo_count,
+                      Boolean nsg_compatibility_mode,
                       _PSIInternalPssmData* internal_pssm)
 {
     /* Subscripts are indicated as follows: N_i, where i is a subscript of N */
@@ -1808,6 +1909,10 @@ _PSIComputeFreqRatios(const _PSIMsa* msa,
     SFreqRatios* freq_ratios = NULL;/* matrix-specific frequency ratios */
     Uint4 p = 0;                    /* index on positions */
     Uint4 r = 0;                    /* index on residues */
+    const double kZeroObsPseudo = 30.0; /*arbitrary constant to use for columns with
+                             zero observations in actual data (ZERO_OBS_PSEUDO in posit.c) */
+    double  expno[MAX_IND_OBSERVATIONS+1]; /*table of expectations*/
+    const double* backgroundProbabilities = Blast_GetMatrixBackgroundFreq(sbp->name);
 
     if ( !msa || !seq_weights || !sbp || !aligned_blocks || !internal_pssm ) {
         return PSIERR_BADPARAM;
@@ -1816,7 +1921,28 @@ _PSIComputeFreqRatios(const _PSIMsa* msa,
 
     freq_ratios = _PSIMatrixFrequencyRatiosNew(sbp->name);
 
+    s_initializeExpNumObservations(&(expno[0]),  backgroundProbabilities);
+
     for (p = 0; p < msa->dimensions->query_length; p++) {
+        double columnCounts = 0.0; /*column-specific pseudocounts*/
+        double observations = 0.0;
+        double pseudoWeight; /*multiplier for pseudocounts term*/
+        if (msa->cell[kQueryIndex][p].letter != kXResidue)
+        {
+           observations = s_effectiveObservations(aligned_blocks, seq_weights, p, msa->dimensions->query_length, expno);
+
+           if (0 == pseudo_count)
+              columnCounts = s_columnSpecificPseudocounts(seq_weights, p, backgroundProbabilities, observations);
+           else
+              columnCounts = pseudo_count;
+        }
+        if (columnCounts >= PSEUDO_MAX) {
+           pseudoWeight = kZeroObsPseudo;
+           observations = 0;
+        }
+        else {
+           pseudoWeight = columnCounts;
+        }
 
         for (r = 0; r < msa->alphabet_size; r++) {
 
@@ -1831,10 +1957,8 @@ _PSIComputeFreqRatios(const _PSIMsa* msa,
 
                 /* beta( Sum_j(f_j r_ij) ) in formula 2 */
                 double pseudo = 0.0;            
-                /* Effective number of independent observations for column p */
-                double alpha = 0.0;
                 /* Renamed to match the formula in the paper */
-                const double kBeta = pseudo_count;
+                const double kBeta = pseudoWeight;
                 double numerator = 0.0;         /* intermediate term */
                 double denominator = 0.0;       /* intermediate term */
                 double qOverPEstimate = 0.0;    /* intermediate term */
@@ -1849,16 +1973,18 @@ _PSIComputeFreqRatios(const _PSIMsa* msa,
                 }
                 pseudo *= kBeta;
 
-                alpha = seq_weights->sigma[p]/aligned_blocks->size[p]-1;
-
                 numerator =
-                    (alpha * seq_weights->match_weights[p][r] / 
+                    (observations * seq_weights->match_weights[p][r] / 
                      seq_weights->std_prob[r]) 
                     + pseudo;
 
-                denominator = alpha + kBeta;
+                denominator = observations + kBeta;
 
-                ASSERT(denominator != 0.0);
+                if (nsg_compatibility_mode && denominator == 0.0) {
+                    return PSIERR_UNKNOWN;
+                } else {
+                    ASSERT(denominator != 0.0);
+                }
                 qOverPEstimate = numerator/denominator;
 
                 /* Note artificial multiplication by standard probability
@@ -2260,6 +2386,9 @@ _PSIComputeScoreProbabilities(const int** pssm,                     /* [in] */
 
     /* Get the minimum and maximum scores */
     for (p = 0; p < query_length; p++) {
+        if (query[p] == kXResidue) {
+            continue;
+        }
         for (r = 0; r < alphabet_size; r++) {
             const int kScore = pssm[p][aa_alphabet[r]];
 
@@ -2447,243 +2576,232 @@ _PSISaveDiagnostics(const _PSIMsa* msa,
         }
     }
 
+    if (diagnostics->sigma) {
+        for (p = 0; p < diagnostics->query_length; p++) {
+            diagnostics->sigma[p] = seq_weights->sigma[p];
+        }
+    }
+
+    if (diagnostics->interval_sizes) {
+        for (p = 0; p < diagnostics->query_length; p++) {
+            diagnostics->interval_sizes[p] = aligned_block->size[p];
+        }
+    }
+
+    if (diagnostics->num_matching_seqs) {
+        for (p = 0; p < diagnostics->query_length; p++) {
+            diagnostics->num_matching_seqs[p] = msa->num_matching_seqs[p];
+        }
+    }
     return PSI_SUCCESS;
 }
 
-/*
- * ===========================================================================
- * $Log: blast_psi_priv.c,v $
- * Revision 1.67  2006/12/05 22:03:00  camacho
- * Minor fix for structure group customization
- *
- * Revision 1.66  2006/11/21 17:07:31  papadopo
- * rearrange headers
- *
- * Revision 1.65  2006/11/20 15:48:00  camacho
- * Relocation of dynamic array functions/definitions
- *
- * Revision 1.64  2006/11/15 16:19:53  camacho
- * Pacify compiler
- *
- * Revision 1.63  2006/11/14 17:24:12  camacho
- * eliminate compiler warning
- *
- * Revision 1.62  2006/11/14 15:37:19  camacho
- * PSSM engine performance optimizations
- *
- * Revision 1.61  2006/08/30 13:30:20  camacho
- * Remove inline
- *
- * Revision 1.60  2006/08/29 22:17:56  camacho
- * Use NCBISTDAA_TO_AMINOACID
- *
- * Revision 1.59  2006/04/19 19:16:49  camacho
- * Refactoring of structure group customization and addition of validation
- *
- * Revision 1.58  2006/02/16 18:47:18  camacho
- * + encoding translations for O and J
- *
- * Revision 1.57  2005/11/18 20:09:45  camacho
- * Fixes for backwards compatibility with C toolkit PSSM engine for certain corner
- * cases.
- *
- * Revision 1.56  2005/10/17 18:34:54  camacho
- * Remove abort() call when sequence weights are not checked
- *
- * Revision 1.55  2005/10/05 14:09:30  camacho
- * Port change in revision 6.76 of posit.c
- *
- * Revision 1.54  2005/10/03 20:42:41  camacho
- * Minor
- *
- * Revision 1.53  2005/04/21 20:26:57  camacho
- * Relax validation in s_PSIValidateAlignedColumns so that query sequence can be
- * the only aligned sequence for a given column of the multiple sequence
- * alignment.
- *
- * Revision 1.52  2005/03/22 15:35:10  camacho
- * Fix to enable backwards compatibility with old PSSM engine when the query is
- * the only sequence aligned for a given column of the multiple sequence
- * alignment.
- *
- * Revision 1.51  2005/03/07 18:46:05  camacho
- * Removed dead code
- *
- * Revision 1.50  2005/02/23 17:32:57  camacho
- * Minor
- *
- * Revision 1.49  2005/02/23 17:24:41  camacho
- * 1. Moved prototype of _PSIUpdateLambdaK to blast_psi_priv.h
- * 2. Removed unneeded fields from Kappa_compactSearchItems
- * 3. Doxygen fixes
- *
- * Revision 1.48  2005/02/22 22:49:55  camacho
- * + impala_scaling_factor, first cut
- *
- * Revision 1.47  2005/02/14 14:07:49  camacho
- * Changes to use SBlastScoreMatrix
- *
- * Revision 1.46  2005/01/31 16:50:21  camacho
- * 1. Moved constants to private header.
- * 2. Changed signature of functions to copy matrices.
- *
- * Revision 1.45  2004/12/15 16:19:27  camacho
- * Remove compiler warning
- *
- * Revision 1.44  2004/12/15 03:17:11  ucko
- * Revert previous change, as VERIFY is not necessarily #defined.
- *
- * Revision 1.43  2004/12/15 02:50:50  camacho
- * Replace ASSERT with VERIFY
- *
- * Revision 1.42  2004/12/13 22:26:59  camacho
- * Consolidated structure group customizations in option: nsg_compatibility_mode
- *
- * Revision 1.41  2004/12/09 15:22:56  dondosha
- * Renamed some functions dealing with BlastScoreBlk and Blast_KarlinBlk structures
- *
- * Revision 1.40  2004/12/08 15:06:01  camacho
- * Call _PSIUpdatePositionCounts is needed after purging query sequence for
- * structure group customization, thus this function has been changed to reset the
- * appropriate _PSIMsa fields so that residue counts/aligned sequences are
- * reported accurately.
- *
- * Revision 1.39  2004/12/07 22:07:34  camacho
- * Add sanity checks for purge identity threshold
- *
- * Revision 1.38  2004/11/29 13:53:23  camacho
- * Renamed Blast_ScoreFreq structure free function
- *
- * Revision 1.37  2004/11/23 21:48:21  camacho
- * Removed local initialization of ideal Karlin-Altschul parameters
- *
- * Revision 1.36  2004/11/22 14:38:48  camacho
- * + option to set % identity threshold to PSSM engine
- *
- * Revision 1.35  2004/11/18 16:25:32  camacho
- * Rename _PSIGetStandardProbabilities to BLAST_GetStandardAaProbabilities
- *
- * Revision 1.34  2004/11/15 16:54:36  camacho
- * break long lines
- *
- * Revision 1.33  2004/11/15 16:32:37  dondosha
- * Changed constant names and static functions names in accordance with C++ toolkit guidelines
- *
- * Revision 1.32  2004/11/15 14:21:43  camacho
- * Correction to _PSIValidateAlignedColumns when the query sequence should be accounted
- *
- * Revision 1.31  2004/11/02 20:37:30  camacho
- * Doxygen fixes
- *
- * Revision 1.30  2004/10/19 14:28:38  camacho
- * Fix memory access error
- *
- * Revision 1.29  2004/10/18 14:33:14  camacho
- * 1. Added validation routines
- * 2. Fixed bug in calculating sequence weights
- * 3. Expanded error codes
- *
- * Revision 1.28  2004/10/13 16:03:00  camacho
- * Add assertions as sanity checks
- *
- * Revision 1.27  2004/10/01 13:59:22  camacho
- * Use calloc where needed
- *
- * Revision 1.26  2004/09/23 19:51:44  camacho
- * Added sanity checks
- *
- * Revision 1.25  2004/09/17 02:06:34  camacho
- * Renaming of diagnostics structure fields
- *
- * Revision 1.24  2004/09/14 21:17:02  camacho
- * Add structure group customization to ignore query
- *
- * Revision 1.23  2004/08/31 16:13:28  camacho
- * Documentation changes
- *
- * Revision 1.22  2004/08/13 22:32:16  camacho
- * Refactoring of _PSIPurgeSimilarAlignments to use finite state machine
- *
- * Revision 1.21  2004/08/04 20:18:26  camacho
- * 1. Renaming of structures and functions that pertain to the internals of PSSM
- *    engine.
- * 2. Updated documentation (in progress)
- *
- * Revision 1.20  2004/08/02 13:25:49  camacho
- * 1. Various renaming of structures, in progress
- * 2. Addition of PSIDiagnostics structures, in progress
- *
- * Revision 1.19  2004/07/29 19:16:02  camacho
- * Moved PSIExtractQuerySequenceInfo
- *
- * Revision 1.18  2004/07/22 19:06:18  camacho
- * 1. Fix in _PSICheckSequenceWeights.
- * 2. Added functions to calculate information content.
- * 3. Cleaned up PSIComputeFreqRatios.
- * 4. Removed unneeded code to set populate extra column in PSSMs.
- * 5. Added collection of information content and gapless column weights to
- * diagnostics structure
- *
- * Revision 1.17  2004/07/06 15:23:54  camacho
- * Fix memory acccess error
- *
- * Revision 1.16  2004/07/02 19:40:48  camacho
- * Fixes for handling out-of-memory conditions
- *
- * Revision 1.15  2004/07/02 18:00:25  camacho
- * 1. Document rationale for order in which sequences are compared in
- *    _PSIPurgeNearIdenticalAlignments.
- * 2. Fix in _PSIPurgeSimilarAlignments to take into account the X residues
- * 3. Refactorings in sequence weight calculation functions:
- *    - Simplification of _PSICheckSequenceWeights
- *    - Addition of _PSISpreadGapWeights
- *    - Reorganization of _PSICalculateNormalizedSequenceWeights
- *
- * Revision 1.14  2004/06/25 20:31:11  camacho
- * Remove C++ comments
- *
- * Revision 1.13  2004/06/25 20:16:50  camacho
- * 1. Minor fixes to sequence weights calculation
- * 2. Add comments
- *
- * Revision 1.12  2004/06/21 12:52:44  camacho
- * Replace PSI_ALPHABET_SIZE for BLASTAA_SIZE
- *
- * Revision 1.11  2004/06/17 20:47:21  camacho
- * Minor fix to extent sizes
- *
- * Revision 1.10  2004/06/16 15:22:47  camacho
- * Fixes to add new unit tests
- *
- * Revision 1.9  2004/06/09 14:21:03  camacho
- * Removed msvc compiler warnings
- *
- * Revision 1.8  2004/06/08 17:30:06  dondosha
- * Compiler warnings fixes
- *
- * Revision 1.7  2004/06/07 14:18:24  dondosha
- * Added some variables initialization, to remove compiler warnings
- *
- * Revision 1.6  2004/05/28 17:35:03  camacho
- * Fix msvc6 warnings
- *
- * Revision 1.5  2004/05/28 16:00:09  camacho
- * + first port of PSSM generation engine
- *
- * Revision 1.4  2004/05/19 14:52:02  camacho
- * 1. Added doxygen tags to enable doxygen processing of algo/blast/core
- * 2. Standardized copyright, CVS $Id string, $Log and rcsid formatting and i
- *    location
- * 3. Added use of @todo doxygen keyword
- *
- * Revision 1.3  2004/05/06 14:01:40  camacho
- * + _PSICopyDoubleMatrix
- *
- * Revision 1.2  2004/04/07 22:08:37  kans
- * needed to include blast_def.h for sfree prototype
- *
- * Revision 1.1  2004/04/07 19:11:17  camacho
- * Initial revision
- *
- * ===========================================================================
- */
+
+
+/** Reorders in the same manner as returned by Blast_GetMatrixBackgroundFreq 
+   this function is a copy of posit.c:fillColumnProbabilities
+*/
+static void  s_fillColumnProbabilities(double *probabilities,
+                                     const _PSISequenceWeights *posSearch,
+                                     Int4 columnNumber)
+{
+   int charOrder[EFFECTIVE_ALPHABET]; /*standard order of letters according to S. Altschul*/
+   int c; /*loop index*/
+
+   charOrder[0] =  1;  /*A*/
+   charOrder[1] =  16; /*R*/
+   charOrder[2] =  13; /*N*/
+   charOrder[3] =  4;  /*D*/
+   charOrder[4] =  3;  /*C*/
+   charOrder[5] =  15; /*Q*/
+   charOrder[6] =  5;  /*E*/
+   charOrder[7] =  7;  /*G*/
+   charOrder[8] =  8;  /*H*/
+   charOrder[9] =  9;  /*I*/
+   charOrder[10] = 11; /*L*/
+   charOrder[11] = 10; /*K*/
+   charOrder[12] = 12; /*M*/
+   charOrder[13] =  6; /*F*/
+   charOrder[14] = 14; /*P*/
+   charOrder[15] = 17; /*S*/
+   charOrder[16] = 18; /*T*/
+   charOrder[17] = 20; /*W*/
+   charOrder[18] = 22; /*Y*/
+   charOrder[19] = 19; /*V*/
+
+   for(c = 0; c < EFFECTIVE_ALPHABET; c++)
+     probabilities[c] = posSearch->match_weights[columnNumber][charOrder[c]];
+}
+
+/** adjust the probabilities by assigning observations weight
+  to initialProbabilities and standardWeight to standardProbabilities
+  copy of posit.c:adjustColumnProbabilities 
+  @param initialProbabilities starting probabilities [in]
+  @param probabilitiesToReturn return value [out]
+  @param standardWeight small number of pseudocounts to
+                    avoid 0 probabilities [in]
+  @param standardProbabilities  background probabilities [in]
+  @param observations expected number of observations [in]
+*/
+static void s_adjustColumnProbabilities(double *initialProbabilities,
+                                      double *probabilitiesToReturn,
+                                      double standardWeight,
+                                      const double *standardProbabilities,
+                                      double observations)
+{
+  double intermediateSums[EFFECTIVE_ALPHABET]; /*weighted sums for each letter*/
+  double overallSum; /*overall sum of weightedSums*/
+  int c; /*loop index*/
+
+  overallSum = 0.0;
+  for(c = 0; c < EFFECTIVE_ALPHABET; c++) {
+    intermediateSums[c] =
+      (initialProbabilities[c] * observations) +
+      (standardProbabilities[c] * standardWeight);
+    overallSum += intermediateSums[c];
+  }
+  for(c = 0; c < EFFECTIVE_ALPHABET; c++)
+    probabilitiesToReturn[c] = intermediateSums[c]/overallSum;
+}
+
+/*compute relative entropy of first distribution to second distribution
+   copy of posit.c:computeRelativeEntropy */
+
+const double kPosEpsilon = 0.0001; /**< minimum return value of s_computeRelativeEntropy */
+
+/** compute relative entropy of first distribution to second distribution
+    A copy of posit.c:computeRelativeEntropy
+    @param newDistribution working set [in]
+    @param backgroundProbabilities standard set [in]
+*/
+static double s_computeRelativeEntropy(const double *newDistribution,
+                              const double *backgroundProbabilities)
+{
+   Int4 c; /*loop index*/
+   double returnValue; /*value to return*/
+
+   returnValue = 0.0;
+   for(c = 0; c < EFFECTIVE_ALPHABET; c++) {
+     if (newDistribution[c] > kPosEpsilon)
+       returnValue += (newDistribution[c] *
+                       log (newDistribution[c]/backgroundProbabilities[c]));
+   }
+   if (returnValue < kPosEpsilon)
+     returnValue = kPosEpsilon;
+   return(returnValue);
+}
+
+
+/*initialize the expected number of observations
+  use background probabilities for this matrix
+  Calculate exp. # of distinct aa's as a function of independent trials
+  copy of posit.c:initializeExpNumObservations
+*/
+static void s_initializeExpNumObservations(double *expno,
+                                    const double *backgroundProbabilities)
+
+{
+int     j,k ; /*loop indices*/
+double  weighted_sum; /*20 - this is how many distinct
+                         amino acids are expected*/
+
+   expno[0] = 0;
+   for (j=1;j<MAX_IND_OBSERVATIONS;++j) {
+     weighted_sum = 0;
+     for (k=0;k<EFFECTIVE_ALPHABET;++k)
+       weighted_sum += exp(j*log(1.0-backgroundProbabilities[k]));
+     expno[j] = EFFECTIVE_ALPHABET-weighted_sum;
+   }
+}
+
+/* copy of posit.c:columnSpecificPseudocounts */
+static double s_columnSpecificPseudocounts(const _PSISequenceWeights *posSearch,
+                                  int columnNumber,
+                                  const double *backgroundProbabilities,
+                                  const double observations)
+{
+  double columnProbabilitiesInitial[EFFECTIVE_ALPHABET];
+  double columnProbabilitiesAdjusted[EFFECTIVE_ALPHABET];
+  double relativeEntropy; /*relative entropy of this column to background probs.*/
+  double alpha; /*intermediate term*/
+  double pseudoDenominator; /*intermediate term*/
+  double returnValue;
+  /* Constant values, were #defines in posit.c */
+  const double kPseudoMult = 500.0;  /* Was PSEUDO_MULTIPLIER */
+  const double kPseudoNumerator = 0.0457;  /*numerator of entropy-based method, was PSEUDO_NUMERATOR */
+  const double kPseudoExponent = 0.8;  /*exponent of denominator, was PSEUDO_EXPONENT */
+  const double kPseudoSmallInitial = 5.5; /*small number of pseudocounts to
+                              avoid 0 probabilities in entropy-based method, was PSEUDO_SMALL_INITIAL */
+
+  s_fillColumnProbabilities(&(columnProbabilitiesInitial[0]), posSearch, columnNumber);
+  s_adjustColumnProbabilities(&(columnProbabilitiesInitial[0]),
+                            &(columnProbabilitiesAdjusted[0]),
+                              kPseudoSmallInitial,
+                              backgroundProbabilities, observations);
+  relativeEntropy = s_computeRelativeEntropy(&(columnProbabilitiesAdjusted[0]),
+                                           backgroundProbabilities);
+  pseudoDenominator = pow(relativeEntropy, kPseudoExponent);
+  alpha = kPseudoNumerator/pseudoDenominator;
+  if (alpha < (1.0 - kPosEpsilon))
+    returnValue = kPseudoMult * alpha/ (1- alpha);
+  else
+    returnValue = PSEUDO_MAX;
+
+  return(returnValue);
+}
+
+static double s_effectiveObservations(const _PSIAlignedBlock *align_blk,
+                                         const _PSISequenceWeights* seq_weights,
+                                         int columnNumber, int queryLength,
+                                         const double *expno)
+{
+int     i,k; /*loop indices*/
+double  indep; /*number of independent observations to return*/
+int halfNumColumns; /*half the number of columns in the interval, rounded
+                      down*/
+int totalDistinctCounts; /*total number of distinct letters in columns
+                     used*/
+double aveDistinctAA; /*average number of distinct letters in columns used*/
+int columnsAccountedFor; /*how many of the columns had their
+                            distinct count totaled so far*/
+
+
+ if (align_blk->pos_extnt[columnNumber].left < 0)
+   return(0);
+ if (align_blk->pos_extnt[columnNumber].right >= queryLength)
+   return(0);
+
+/*  Calculate the average number of distinct amino acids in the half of the
+    columns within the block in question with the most distinct amino acids;
+    +2 in the parentheses is for rounding up.*/
+
+ halfNumColumns = MAX(1,(align_blk->pos_extnt[columnNumber].right -
+                         align_blk->pos_extnt[columnNumber].left+2)/2);
+ k = EFFECTIVE_ALPHABET;
+ columnsAccountedFor = 0;
+ totalDistinctCounts = 0;
+ while (columnsAccountedFor < halfNumColumns) {
+   totalDistinctCounts += (seq_weights->posDistinctDistrib[columnNumber][k] *k);
+   columnsAccountedFor += seq_weights->posDistinctDistrib[columnNumber][k];
+   if (columnsAccountedFor > halfNumColumns) {
+     totalDistinctCounts -=
+       ((columnsAccountedFor - halfNumColumns) * k);
+     columnsAccountedFor = halfNumColumns;
+   }
+   k--;
+ }
+ aveDistinctAA = ((double) totalDistinctCounts)/
+   ((double) columnsAccountedFor);
+
+/*    Then use the following code to calculate the number of
+        independent observations corresponding to
+        aveDistinctAA.
+*/
+
+ for (i=1;i<MAX_IND_OBSERVATIONS && expno[i]<=aveDistinctAA;++i);
+ indep = (i==MAX_IND_OBSERVATIONS) ? i :
+   i-(expno[i]-aveDistinctAA)/(expno[i]-expno[i-1]);
+ indep = MIN(indep, seq_weights->posNumParticipating[columnNumber]);
+ indep = MAX(0,indep - 1);
+ return(indep);
+}
