@@ -1,4 +1,4 @@
-/* $Id: blast_traceback.c,v 1.206 2007/12/07 18:25:42 kazimird Exp $
+/* $Id: blast_traceback.c,v 1.209 2009/01/12 14:19:33 kazimird Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -50,7 +50,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: blast_traceback.c,v 1.206 2007/12/07 18:25:42 kazimird Exp $";
+    "$Id: blast_traceback.c,v 1.209 2009/01/12 14:19:33 kazimird Exp $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/blast_traceback.h>
@@ -343,7 +343,8 @@ Blast_TracebackFromHSPList(EBlastProgramType program_number,
 {
    Int4 index;
    BlastHSP* hsp;
-   Uint1* query,* subject;
+   Uint1* query;
+   const Uint1* subject = NULL;
    Int4 query_length;
    BlastHSP** hsp_array;
    Int4 subject_length=0;
@@ -353,7 +354,6 @@ Blast_TracebackFromHSPList(EBlastProgramType program_number,
    Uint1* translation_buffer = NULL;
    Int4 * frame_offsets   = NULL;
    Int4 * frame_offsets_a = NULL;
-   Boolean partial_translation = FALSE;
    const Boolean is_rpsblast = Blast_ProgramIsRpsBlast(program_number);
    const Boolean kIsOutOfFrame = score_options->is_ooframe;
    const Boolean kGreedyTraceback = (ext_options->eTbackExt == eGreedyTbck);
@@ -367,6 +367,7 @@ Blast_TracebackFromHSPList(EBlastProgramType program_number,
    BlastIntervalTree* tree = NULL;
    BlastHSPList * orig_hsplist = NULL;
    Boolean fence_error = FALSE;
+   SBlastTargetTranslation* target_t = NULL;
    
    if (num_initial_hsps == 0) {
       return 0;
@@ -395,23 +396,15 @@ Blast_TracebackFromHSPList(EBlastProgramType program_number,
           return -1;
       }
       
-      if (kIsOutOfFrame) {
-         Blast_SetUpSubjectTranslation(subject_blk, gen_code_string,
-                                       NULL, NULL, &partial_translation);
-         /* Length of a mixed-frame sequence, corresponding to each single 
-            strand of the nucleotide sequence, is equal to nucleotide 
-            length. */
-         subject_length = subject_blk->length;
-      } else if (program_number == eBlastTypeRpsTblastn) {
+      if (program_number == eBlastTypeRpsTblastn) {
           translation_buffer = subject_blk->sequence - 1;
           frame_offsets_a = frame_offsets =
               ContextOffsetsToOffsetArray(query_info_in);
-      } else {
-         Blast_SetUpSubjectTranslation(subject_blk, gen_code_string,
-            &translation_buffer, &frame_offsets, &partial_translation);
-         frame_offsets_a = frame_offsets;
-         /* subject and subject_length will be set later, for each HSP. */
-      }
+      } 
+      else
+      	BlastTargetTranslationNew(subject_blk, gen_code_string, program_number, kIsOutOfFrame, &target_t);
+      if (kIsOutOfFrame)
+        subject_length = subject_blk->length;
    } else {
       /* Subject is not translated */
       subject = subject_blk->sequence;
@@ -472,29 +465,22 @@ Blast_TracebackFromHSPList(EBlastProgramType program_number,
 
          Int4 start_shift = 0;
          Int4 adjusted_s_length;
-         Uint1* adjusted_subject;
+         const Uint1* adjusted_subject;
          Int4 cutoff;
 
          if (kTranslateSubject) {
-            if (!kIsOutOfFrame && !partial_translation) {
-               const Int4 context = BLAST_FrameToContext(hsp->subject.frame, 
-                                                   program_number);
-               subject = translation_buffer + frame_offsets[context] + 1;
-               subject_length = 
-                  frame_offsets[context+1] - frame_offsets[context] - 1;
-            } else { 
-                if (partial_translation) {
-                    Blast_HSPGetPartialSubjectTranslation(subject_blk, hsp, 
-                        kIsOutOfFrame, gen_code_string, &translation_buffer, 
-                        &subject, &subject_length, &start_shift);
-                } else {
-                    /* Out-of-frame with full translation; point subject to the
+            if (program_number == eBlastTypeRpsTblastn) {
+                 const Int4 context = BLAST_FrameToContext(hsp->subject.frame, program_number);
+                 subject = translation_buffer + frame_offsets[context] + 1;
+                 subject_length = frame_offsets[context+1] - frame_offsets[context] - 1;
+            } else if (kIsOutOfFrame) {
+                 /* Out-of-frame with full translation; point subject to the
                        start of the right strand in the mixed-frame sequence. */
-                    subject = subject_blk->oof_sequence + CODON_LENGTH;
-                    if (hsp->subject.frame < 0)
-                        subject += subject_length + 1;
-                }
-            }
+                 subject = subject_blk->oof_sequence + CODON_LENGTH;
+                 if (hsp->subject.frame < 0)
+                      subject += subject_length + 1;
+            } else
+            	subject = Blast_HSPGetTargetTranslation(target_t, hsp, &subject_length);
          }
 
          if (!kIsOutOfFrame && (((hsp->query.gapped_start == 0 && 
@@ -624,6 +610,8 @@ Blast_TracebackFromHSPList(EBlastProgramType program_number,
          sfree(translation_buffer);
       }
    }
+
+   target_t = BlastTargetTranslationFree(target_t);
    
    if (kSmithWaterman) {
        /* switch over to the result of the traceback */
@@ -1029,7 +1017,7 @@ Int2 s_RPSComputeTraceback(EBlastProgramType program_number,
          corresponding to this sequence. */
       seq_arg.oid = hsp_list->oid;
       seq_arg.encoding = encoding;
-      if (BlastSeqSrcGetSequence(seq_src, (void*) &seq_arg) < 0)
+      if (BlastSeqSrcGetSequence(seq_src, &seq_arg) < 0)
           continue;
 
       db_seq_start = concat_db_info->contexts[hsp_list->oid].query_offset;
@@ -1057,7 +1045,7 @@ Int2 s_RPSComputeTraceback(EBlastProgramType program_number,
              *        need a warning here
              */
             hsp_list = Blast_HSPListFree(hsp_list);
-            BlastSeqSrcReleaseSequence(seq_src, (void*)&seq_arg);
+            BlastSeqSrcReleaseSequence(seq_src, &seq_arg);
             continue;
          }
          
@@ -1073,7 +1061,7 @@ Int2 s_RPSComputeTraceback(EBlastProgramType program_number,
          one_query, one_query_info, gap_align, sbp, score_params, 
          ext_params->options, hit_params, NULL, NULL);
 
-      BlastSeqSrcReleaseSequence(seq_src, (void*)&seq_arg);
+      BlastSeqSrcReleaseSequence(seq_src, &seq_arg);
 
       if (program_number != eBlastTypeRpsTblastn) {
          _PSIDeallocateMatrix((void**)sbp->psi_matrix->pssm->data, 
@@ -1203,7 +1191,7 @@ BLAST_ComputeTraceback(EBlastProgramType program_number,
             seq_arg.check_oid_exclusion = TRUE;
             
             BlastSequenceBlkClean(seq_arg.seq);
-            if (BlastSeqSrcGetSequence(seq_src, (void*) &seq_arg) < 0) {
+            if (BlastSeqSrcGetSequence(seq_src, &seq_arg) < 0) {
                Blast_HSPStreamResultBatchReset(batch);
                continue;
             }
@@ -1274,8 +1262,8 @@ BLAST_ComputeTraceback(EBlastProgramType program_number,
                         (whole) subject sequence */
                         
                      seq_arg.enable_ranges = FALSE;
-                     BlastSeqSrcReleaseSequence(seq_src, (void*)&seq_arg);
-                     BlastSeqSrcGetSequence(seq_src, (void*) &seq_arg);
+                     BlastSeqSrcReleaseSequence(seq_src, &seq_arg);
+                     BlastSeqSrcGetSequence(seq_src, &seq_arg);
                         
                      /* Retry the alignment */
                        
@@ -1311,7 +1299,7 @@ BLAST_ComputeTraceback(EBlastProgramType program_number,
          }      /* loop over one HSPList batch */
 
          if (perform_traceback) {
-            BlastSeqSrcReleaseSequence(seq_src, (void*)&seq_arg);
+            BlastSeqSrcReleaseSequence(seq_src, &seq_arg);
          }
 
       }         /* loop over all batches */
