@@ -1,4 +1,4 @@
-/* $Id: blast_util.c,v 1.121 2007/01/21 08:45:13 kazimird Exp $
+/* $Id: blast_util.c,v 1.127 2009/02/03 18:14:30 kazimird Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -34,7 +34,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: blast_util.c,v 1.121 2007/01/21 08:45:13 kazimird Exp $";
+    "$Id: blast_util.c,v 1.127 2009/02/03 18:14:30 kazimird Exp $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/blast_util.h>
@@ -49,28 +49,116 @@ __sfree(void **x)
     return;
 }
 
+SSeqRange SSeqRangeNew(Int4 start, Int4 stop)
+{
+    SSeqRange retval;
+    retval.left = start;
+    retval.right = stop;
+    return retval;
+}
+
+Int4
+SSeqRangeArrayLessThanOrEqual(const SSeqRange* ranges, Int4 num_ranges, Int4
+                              target)
+{
+    Int4 retval = -1;       /* assume failure */
+    Int4 m = 0, b = 0, e = 0;
+
+    if (ranges == NULL || num_ranges <= 0) {
+        return retval;
+    }
+
+    b = 0;
+    e = num_ranges;
+    while (b < e - 1) {
+        m = (b + e) / 2;
+        if (ranges[m].left > target) {
+            e = m;
+        } else {
+            b = m;
+        }
+    }
+    /* if the target isn't in the range at index b and there is still more
+     * data, return the next element */
+    if (target > ranges[b].right && b < num_ranges) {
+        return b + 1;
+    } else {
+        return b;
+    }
+}
+
+/** Auxiliary function to free the BLAST_SequenceBlk::seq_ranges field if
+ * applicable
+ * @param seq_blk The sequence block structure to manipulate [in|out]
+ */
+static void 
+s_BlastSequenceBlkFreeSeqRanges(BLAST_SequenceBlk* seq_blk)
+{
+    ASSERT(seq_blk);
+    if (seq_blk->seq_ranges_allocated) {
+        sfree(seq_blk->seq_ranges);
+        seq_blk->num_seq_ranges = 0;
+        seq_blk->seq_ranges_allocated = FALSE;
+    }
+}
+
+/** Allocate and populate the seq_ranges field of the BLAST_SequenceBlk with
+ * default values as to indicate that the entire sequence should be searched 
+ * @note assumes the BLAST_SequenceBlk::length has been initialized
+ * @param seq_blk the data structure to populate [in|out]
+ * @return 0 on success, BLASTERR_MEMORY if memory allocation fails
+ */
+static Int2
+s_BlastSeqBlkSetLookupTableRangesToIndex(BLAST_SequenceBlk* seq_blk)
+{
+    static const Uint4 kDfltSize = 1;
+    if (seq_blk->seq_ranges != NULL) {
+        return 0;
+    }
+
+    s_BlastSequenceBlkFreeSeqRanges(seq_blk);
+    seq_blk->seq_ranges = (SSeqRange*) calloc(kDfltSize, 
+                                              sizeof(*seq_blk->seq_ranges));
+    if ( !seq_blk->seq_ranges ) {
+        return BLASTERR_MEMORY;
+    }
+    seq_blk->seq_ranges[0].left = 0;
+    seq_blk->seq_ranges[0].right = seq_blk->length;
+    seq_blk->num_seq_ranges = kDfltSize;
+    seq_blk->seq_ranges_allocated = TRUE;
+
+    return 0;
+}
+
 Int2
 BlastSetUp_SeqBlkNew (const Uint1* buffer, Int4 length,
    BLAST_SequenceBlk* *seq_blk, Boolean buffer_allocated)
 {
-   /* Check if BLAST_SequenceBlk itself needs to be allocated here or not */
-   if (*seq_blk == NULL) {
-      *seq_blk = calloc(1, sizeof(BLAST_SequenceBlk));
-   }
+    /* Check if BLAST_SequenceBlk itself needs to be allocated here or not */
+    if (*seq_blk == NULL) {
+        if (BlastSeqBlkNew(seq_blk) != 0) {
+            return -1;
+        }
+    }
+    ASSERT(seq_blk && *seq_blk);
 
-   if (buffer_allocated) {
-      (*seq_blk)->sequence_start_allocated = TRUE;
-      (*seq_blk)->sequence_start = (Uint1 *) buffer;
-      /* The first byte is a sentinel byte. */
-      (*seq_blk)->sequence = (*seq_blk)->sequence_start+1;
-   } else {
-      (*seq_blk)->sequence = (Uint1 *) buffer;
-      (*seq_blk)->sequence_start = NULL;
-   }
+    if (buffer_allocated) {
+        (*seq_blk)->sequence_start_allocated = TRUE;
+        (*seq_blk)->sequence_start = (Uint1 *) buffer;
+        /* The first byte is a sentinel byte. */
+        (*seq_blk)->sequence = (*seq_blk)->sequence_start+1;
+    } else {
+        (*seq_blk)->sequence = (Uint1 *) buffer;
+        (*seq_blk)->sequence_start = NULL;
+    }
+    
+    (*seq_blk)->length = length;
+
+    if (s_BlastSeqBlkSetLookupTableRangesToIndex(*seq_blk) != 0) {
+        return -1;
+    }
    
-   (*seq_blk)->length = length;
-   
-   return 0;
+    return 0;
 }
 
 Int2 BlastSeqBlkNew(BLAST_SequenceBlk** retval)
@@ -100,6 +188,9 @@ Int2 BlastSeqBlkSetSequence(BLAST_SequenceBlk* seq_blk,
     seq_blk->sequence = (Uint1*) sequence + 1;
     seq_blk->length = seqlen;
     seq_blk->oof_sequence = NULL;
+    if (s_BlastSeqBlkSetLookupTableRangesToIndex(seq_blk) != 0) {
+        return -1;
+    }
 
     return 0;
 }
@@ -115,6 +206,43 @@ Int2 BlastSeqBlkSetCompressedSequence(BLAST_SequenceBlk* seq_blk,
     seq_blk->sequence = (Uint1*) sequence;
     seq_blk->oof_sequence = NULL;
 
+    if (s_BlastSeqBlkSetLookupTableRangesToIndex(seq_blk) != 0) {
+        return -1;
+    }
+
+    return 0;
+}
+
+Int2
+BlastSeqBlkSetSeqRanges(BLAST_SequenceBlk* seq_blk,
+                        SSeqRange* seq_ranges,
+                        Uint4 num_seq_ranges,
+                        Boolean copy_seq_ranges)
+{
+    if ( !seq_blk || !seq_ranges ) {
+        return -1;
+    }
+    ASSERT(num_seq_ranges >= 1);
+
+    if (copy_seq_ranges) {
+        SSeqRange* tmp = (SSeqRange*) calloc(num_seq_ranges,
+                                             sizeof(*seq_ranges));
+        if ( !tmp ) {
+            return -1;
+        }
+        s_BlastSequenceBlkFreeSeqRanges(seq_blk);
+        seq_blk->seq_ranges = tmp;
+        memcpy((void*) seq_blk->seq_ranges,
+               (void*) seq_ranges,
+               num_seq_ranges * sizeof(*seq_ranges));
+        seq_blk->num_seq_ranges = num_seq_ranges;
+        seq_blk->seq_ranges_allocated = TRUE;
+    } else {
+        s_BlastSequenceBlkFreeSeqRanges(seq_blk);
+        seq_blk->seq_ranges = seq_ranges;
+        seq_blk->seq_ranges_allocated = FALSE;
+        seq_blk->num_seq_ranges = num_seq_ranges;
+    }
     return 0;
 }
 
@@ -135,7 +263,7 @@ void BlastSequenceBlkClean(BLAST_SequenceBlk* seq_blk)
        sfree(seq_blk->oof_sequence);
        seq_blk->oof_sequence_allocated = FALSE;
    }
-
+   s_BlastSequenceBlkFreeSeqRanges(seq_blk);
    return;
 }
 
@@ -156,18 +284,20 @@ BLAST_SequenceBlk* BlastSequenceBlkFree(BLAST_SequenceBlk* seq_blk)
 void BlastSequenceBlkCopy(BLAST_SequenceBlk** copy, 
                           BLAST_SequenceBlk* src) 
 {
-   ASSERT(copy);
-   ASSERT(src);
-   
-   if (*copy)
-      memcpy(*copy, src, sizeof(BLAST_SequenceBlk));
-   else 
-      *copy = BlastMemDup(src, sizeof(BLAST_SequenceBlk));
+    ASSERT(copy);
+    ASSERT(src);
+    
+    if (*copy) {
+        memcpy(*copy, src, sizeof(BLAST_SequenceBlk));
+    } else {
+        *copy = BlastMemDup(src, sizeof(BLAST_SequenceBlk));
+    }
 
-   (*copy)->sequence_allocated = FALSE;
-   (*copy)->sequence_start_allocated = FALSE;
-   (*copy)->oof_sequence_allocated = FALSE;
-   (*copy)->lcase_mask_allocated = FALSE;
+    (*copy)->sequence_allocated = FALSE;
+    (*copy)->sequence_start_allocated = FALSE;
+    (*copy)->oof_sequence_allocated = FALSE;
+    (*copy)->lcase_mask_allocated = FALSE;
+    (*copy)->seq_ranges_allocated = FALSE;
 }
 
 Int2 BlastProgram2Number(const char *program, EBlastProgramType *number)
@@ -321,9 +451,9 @@ BLAST_GetTranslation(const Uint1* query_seq, const Uint1* query_seq_rev,
 	Uint1 codon[CODON_LENGTH];
 	Int4 index, index_prot;
 	Uint1 residue;
-   Uint1* nucl_seq;
+        Uint1* nucl_seq;
 
-   nucl_seq = (frame >= 0 ? (Uint1 *)query_seq : (Uint1 *)(query_seq_rev+1));
+        nucl_seq = (frame >= 0 ? (Uint1 *)query_seq : (Uint1 *)(query_seq_rev+1));
 
 	/* The first character in the protein is the NULLB sentinel. */
 	prot_seq[0] = NULLB;
@@ -805,8 +935,7 @@ Int2 BLAST_PackDNA(const Uint1* buffer, Int4 length, EBlastEncoding encoding,
 size_t
 BLAST_GetTranslatedProteinLength(size_t nucleotide_length, unsigned int context)
 {
-    ASSERT(context >= 0 && context < NUM_FRAMES);
-    if (nucleotide_length == 0) {
+    if (nucleotide_length == 0 || nucleotide_length <= context % CODON_LENGTH) {
         return 0;
     }
     return (nucleotide_length - context % CODON_LENGTH) / CODON_LENGTH;
@@ -940,7 +1069,7 @@ Int2 BLAST_GetAllTranslations(const Uint1* nucl_seq, EBlastEncoding encoding,
       return -1;
 
    if ((translation_buffer = 
-        (Uint1*) malloc(2*(nucl_length+1)+1)) == NULL)
+        (Uint1*) malloc(2*(nucl_length+1)+2)) == NULL)
       return -1;
 
    if (encoding == eBlastEncodingNcbi4na) {
@@ -1156,34 +1285,108 @@ Blast_SetUpSubjectTranslation(BLAST_SequenceBlk* subject_blk,
    return 0;
 }
 
+SBlastTargetTranslation*
+BlastTargetTranslationFree(SBlastTargetTranslation* target_t)
+{
+
+        if (target_t)
+        {
+                if (target_t->translations)
+                {
+			int index;
+			for (index=0; index<target_t->num_frames; index++)
+                           sfree(target_t->translations[index]);
+                        sfree(target_t->translations);
+                }
+                if (target_t->range)
+                	sfree(target_t->range);
+                sfree(target_t);
+        }
+        return NULL;
+}
+
+Int2 
+BlastTargetTranslationNew(BLAST_SequenceBlk* subject_blk,
+                              const Uint1* gen_code_string,
+                              EBlastProgramType program_number,
+                              Boolean is_ooframe,
+                              SBlastTargetTranslation** target)
+{
+      SBlastTargetTranslation* retval = (SBlastTargetTranslation*) calloc(1, sizeof(SBlastTargetTranslation));
+      Int4 num_frames = retval->num_frames = NUM_FRAMES;
+      *target = retval;
+
+      retval->gen_code_string = gen_code_string;
+      retval->program_number = program_number;
+
+      /* If target is OOF do translation now, otherwise do it as needed. */
+      retval->partial = !is_ooframe;
+
+      retval->translations = (Uint1**) calloc(num_frames, sizeof(Uint1*));
+
+      if (!retval->partial)
+      {
+         if (is_ooframe) {
+             BLAST_GetAllTranslations(subject_blk->sequence_start,
+                eBlastEncodingNcbi4na, subject_blk->length, gen_code_string,
+                NULL, NULL, &subject_blk->oof_sequence);
+             subject_blk->oof_sequence_allocated = TRUE;
+         }
+         else
+         {
+              int context = 0;
+              Uint1* nucl_seq_rev = NULL;
+
+               /* First produce the reverse strand of the nucleotide sequence */
+              GetReverseNuclSequence(subject_blk->sequence_start, subject_blk->length, 
+                    &nucl_seq_rev);
+
+              for (context = 0; context < num_frames; ++context) {
+                 int frame = BLAST_ContextToFrame(eBlastTypeBlastx, context);
+                 retval->translations[context] = (Uint1*) malloc((2+subject_blk->length/3)*sizeof(Uint1));
+                 BLAST_GetTranslation(subject_blk->sequence_start, nucl_seq_rev,
+                    subject_blk->length, frame, retval->translations[context], gen_code_string);
+              }
+              sfree(nucl_seq_rev);
+         }
+      }
+      else
+      {
+           retval->range = (Int4*) calloc(2*num_frames, sizeof(Int4));
+           retval->subject_blk = subject_blk; /* Get pointer for later translations. */
+      }
+      
+
+      return 0;
+}
+
 double* 
 BLAST_GetStandardAaProbabilities()
 {
     Blast_ResFreq* standard_probabilities = NULL;
     Uint4 i = 0;
     double* retval = NULL;
-    BlastScoreBlk* sbp = BlastScoreBlkNew(BLASTAA_SEQ_CODE, 1);
 
-    if ( !sbp ) {
-        return NULL;
-    }
+    /* Manually build a BlastScoreBlk, we only need a few fields populated */
+    BlastScoreBlk sbp;
+    memset((void*)&sbp, 0, sizeof(BlastScoreBlk));
+    sbp.alphabet_code = BLASTAA_SEQ_CODE;
+    sbp.alphabet_size = BLASTAA_SIZE;
+    sbp.protein_alphabet = TRUE;
 
-    ASSERT(sbp->alphabet_size == BLASTAA_SIZE);
-    retval = (double*) malloc(sbp->alphabet_size * sizeof(double));
+    retval = (double*) malloc(sbp.alphabet_size * sizeof(double));
     if ( !retval ) {
-        sbp = BlastScoreBlkFree(sbp);
         return NULL;
     }
 
-    standard_probabilities = Blast_ResFreqNew(sbp);
-    Blast_ResFreqStdComp(sbp, standard_probabilities);
+    standard_probabilities = Blast_ResFreqNew(&sbp);
+    Blast_ResFreqStdComp(&sbp, standard_probabilities);
 
-    for (i = 0; i < (Uint4) sbp->alphabet_size; i++) {
+    for (i = 0; i < (Uint4) sbp.alphabet_size; i++) {
         retval[i] = standard_probabilities->prob[i];
     }
 
     Blast_ResFreqFree(standard_probabilities);
-    sbp = BlastScoreBlkFree(sbp);
     return retval;
 }
 
