@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   12/30/03
 *
-* $Revision: 1.82 $
+* $Revision: 1.108 $
 *
 * File Description:  New GenBank flatfile generator, internal header
 *
@@ -65,6 +65,7 @@ extern "C" {
 #define TILDE_TO_SPACES  1
 #define TILDE_EXPAND     2
 #define TILDE_OLD_EXPAND 3
+#define TILDE_SEMICOLON  4
 
 
 /* flags set by mode to customize behavior */
@@ -92,12 +93,13 @@ typedef struct asn2gbflags {
   Boolean             srcQualsToNote;
   Boolean             hideEmptySource;
   Boolean             goQualsToNote;
-  Boolean             geneSynsToNote;
+  Boolean             separateGeneSyns;
   Boolean             refSeqQualsToNote;
   Boolean             selenocysteineToNote;
   Boolean             pyrrolysineToNote;
   Boolean             extraProductsToNote;
   Boolean             codonRecognizedToNote;
+  Boolean             hideSpecificGeneMaps;
   Boolean             forGbRelease;
 } Asn2gbFlags, PNTR Asn2gbFlagsPtr;
 
@@ -165,19 +167,22 @@ typedef struct int_asn2gb_job {
 /* should be allocated to MAX (ASN2GNBK_TOTAL_SOURCE, ASN2GNBK_TOTAL_FEATUR) */
 
 typedef union qualval {
-  CharPtr        str;
-  Boolean        ble;
-  Int4           num;
-  ValNodePtr     vnp;
-  GBQualPtr      gbq;
-  OrgModPtr      omp;
-  SubSourcePtr   ssp;
-  CodeBreakPtr   cbp;
-  SeqLocPtr      slp;
-  SeqIdPtr       sip;
-  tRNAPtr        trp;
-  UserObjectPtr  uop;
-  UserFieldPtr   ufp;
+  CharPtr              str;
+  Boolean              ble;
+  Int4                 num;
+  ValNodePtr           vnp;
+  GBQualPtr            gbq;
+  OrgModPtr            omp;
+  SubSourcePtr         ssp;
+  CodeBreakPtr         cbp;
+  SeqLocPtr            slp;
+  SeqIdPtr             sip;
+  tRNAPtr              trp;
+  UserObjectPtr        uop;
+  UserFieldPtr         ufp;
+  RNAGenPtr            rgp;
+  GeneNomenclaturePtr  gnp;
+  PCRReactionSetPtr    prp;
 } QualVal, PNTR QualValPtr;
 
 /* structure passed to individual paragraph format functions */
@@ -276,6 +281,7 @@ typedef struct asn2gbwork {
   Boolean          hideImpFeats;
   Boolean          hideVariations;
   Boolean          hideRepeatRegions;
+  Boolean          hideSitesBondsRegions;
   Boolean          hideCddFeats;
   Boolean          hideCdsProdFeats;
 
@@ -443,7 +449,8 @@ typedef enum {
   Qual_class_method,
   Qual_class_pubset,
   Qual_class_db_xref,
-  Qual_class_seq_id,
+  Qual_class_nuc_id,
+  Qual_class_prt_id,
   Qual_class_seq_loc,
   Qual_class_its,
   Qual_class_sec_str,
@@ -459,13 +466,19 @@ typedef enum {
   Qual_class_product,
   Qual_class_model_ev,
   Qual_class_gene_syn,
+  Qual_class_sep_gene_syn,
   Qual_class_locus_tag,
+  Qual_class_map,
   Qual_class_go,
   Qual_class_nomenclature,
+  Qual_class_gene_nomen,
   Qual_class_pcr,
+  Qual_class_pcr_react,
   Qual_class_mol_wt,
   Qual_class_voucher,
-  Qual_class_mobile_element
+  Qual_class_lat_lon,
+  Qual_class_mobile_element,
+  Qual_class_tag_peptide
 }  QualType;
 
 /* source 'feature' */
@@ -515,6 +528,7 @@ typedef enum {
   SCQUAL_genotype,
   SCQUAL_germline,
   SCQUAL_group,
+  SCQUAL_haplogroup,
   SCQUAL_haplotype,
   SCQUAL_identified_by,
   SCQUAL_ins_seq_name,
@@ -523,8 +537,10 @@ typedef enum {
   SCQUAL_lab_host,
   SCQUAL_label,
   SCQUAL_lat_lon,
+  SCQUAL_linkage_group,
   SCQUAL_macronuclear,
   SCQUAL_map,
+  SCQUAL_mating_type,
   SCQUAL_metagenome_note,
   SCQUAL_metagenome_source,
   SCQUAL_metagenomic,
@@ -538,6 +554,7 @@ typedef enum {
   SCQUAL_pathovar,
   SCQUAL_PCR_primers,
   SCQUAL_PCR_primer_note,
+  SCQUAL_PCR_reaction,
   SCQUAL_plasmid_name,
   SCQUAL_plastid_name,
   SCQUAL_pop_variant,
@@ -576,7 +593,7 @@ typedef enum {
   ASN2GNBK_TOTAL_SOURCE
 }  SourceType;
 
-NLM_EXTERN SourceType orgModToSourceIdx [42];
+NLM_EXTERN SourceType orgModToSourceIdx [41];
 
 typedef enum {
   FTQUAL_allele = 1,
@@ -612,6 +629,9 @@ typedef enum {
   FTQUAL_gene_desc,
   FTQUAL_gene_allele,
   FTQUAL_gene_map,
+  FTQUAL_gene_cyt_map,
+  FTQUAL_gene_gen_map,
+  FTQUAL_gene_rad_map,
   FTQUAL_gene_syn,
   FTQUAL_gene_syn_refseq,
   FTQUAL_gene_note,
@@ -635,7 +655,10 @@ typedef enum {
   FTQUAL_modelev,
   FTQUAL_mol_wt,
   FTQUAL_ncRNA_class,
+  FTQUAL_ncRNA_note,
+  FTQUAL_ncRNA_other,
   FTQUAL_nomenclature,
+  FTQUAL_gene_nomen,
   FTQUAL_note,
   FTQUAL_number,
   FTQUAL_old_locus_tag,
@@ -671,6 +694,7 @@ typedef enum {
   FTQUAL_rpt_unit_range,
   FTQUAL_rpt_unit_seq,
   FTQUAL_rrna_its,
+  FTQUAL_satellite,
   FTQUAL_sec_str_type,
   FTQUAL_selenocysteine,
   FTQUAL_selenocysteine_note,
@@ -680,6 +704,7 @@ typedef enum {
   FTQUAL_site_type,
   FTQUAL_standard_name,
   FTQUAL_tag_peptide,
+  FTQUAL_tag_peptide_str,
   FTQUAL_transcription,
   FTQUAL_transcript_id,
   FTQUAL_transcript_id_note,
@@ -696,28 +721,6 @@ typedef enum {
   ASN2GNBK_TOTAL_FEATUR
 }  FtQualType;
 
-#define MAX_WWWBUF 328
-
-NLM_EXTERN Char link_feat [MAX_WWWBUF];
-NLM_EXTERN Char link_featc [MAX_WWWBUF];
-NLM_EXTERN Char link_seq [MAX_WWWBUF];
-NLM_EXTERN Char link_projid [MAX_WWWBUF];
-NLM_EXTERN Char link_wgs [MAX_WWWBUF];
-NLM_EXTERN Char link_omim [MAX_WWWBUF];
-NLM_EXTERN Char ref_link [MAX_WWWBUF];
-NLM_EXTERN Char nt_link [MAX_WWWBUF];
-NLM_EXTERN Char doc_link [MAX_WWWBUF];
-NLM_EXTERN Char ev_link [MAX_WWWBUF];
-NLM_EXTERN Char ec_link [MAX_WWWBUF];
-NLM_EXTERN Char ec_ambig [MAX_WWWBUF];
-NLM_EXTERN Char link_tax [MAX_WWWBUF];
-NLM_EXTERN Char link_muid [MAX_WWWBUF];
-NLM_EXTERN Char link_code [MAX_WWWBUF];
-NLM_EXTERN Char link_encode [MAX_WWWBUF];
-NLM_EXTERN Char link_go [MAX_WWWBUF];
-NLM_EXTERN Char link_go_ref [MAX_WWWBUF];
-NLM_EXTERN Char link_sp [MAX_WWWBUF];
-
 NLM_EXTERN void FF_www_db_xref(
   IntAsn2gbJobPtr ajp,
   StringItemPtr ffstring,
@@ -726,6 +729,17 @@ NLM_EXTERN void FF_www_db_xref(
   BioseqPtr bsp
 );
 
+NLM_EXTERN void FF_www_specimen_voucher (
+  IntAsn2gbJobPtr ajp,
+  StringItemPtr ffstring,
+  CharPtr subname
+);
+
+NLM_EXTERN void FF_Add_NCBI_Base_URL (
+  StringItemPtr ffstring,
+  CharPtr url
+);
+
 NLM_EXTERN Boolean StringIsJustQuotes (
   CharPtr str
 );
@@ -738,7 +752,7 @@ typedef struct sourcequal {
 
 NLM_EXTERN SourceQual asn2gnbk_source_quals [ASN2GNBK_TOTAL_SOURCE];
 
-NLM_EXTERN SourceType subSourceToSourceIdx [39];
+NLM_EXTERN SourceType subSourceToSourceIdx [42];
 
 NLM_EXTERN void DoOneSection (
   BioseqPtr target,
@@ -784,6 +798,7 @@ NLM_EXTERN void FFAddNChar (
   Boolean convertQuotes
 );
 NLM_EXTERN void FFExpandTildes (StringItemPtr sip, CharPtr PNTR cpp);
+NLM_EXTERN void FFSemicolonSeparateTildes (StringItemPtr sip, CharPtr PNTR cpp);
 NLM_EXTERN void FFReplaceTildesWithSpaces (StringItemPtr ffstring, CharPtr PNTR cpp);
 NLM_EXTERN void FFOldExpand (StringItemPtr sip, CharPtr PNTR cpp);
 NLM_EXTERN void AddCommentStringWithTildes (StringItemPtr ffstring, CharPtr string);
@@ -823,7 +838,16 @@ NLM_EXTERN void FFTrim (
     StringItemPtr line_start,
     Int4 line_pos,
     Int4 line_prefix_len
-);NLM_EXTERN void FFCalculateLineBreak (
+);
+NLM_EXTERN int FFNextChar(
+  StringItemPtr start_sip,
+  Int4 start_pos
+);
+NLM_EXTERN void FFAdvanceChar(
+  StringItemPtr* start_sip,
+  Int4* start_pos
+);
+NLM_EXTERN void FFCalculateLineBreak (
   StringItemPtr PNTR break_sip, Int4 PNTR break_pos,
   Int4 init_indent, Int4 visible
 );
@@ -966,7 +990,8 @@ NLM_EXTERN void FFAddString_NoRedund (
   StringItemPtr unique,
   CharPtr prefix,
   CharPtr string,
-  CharPtr suffix
+  CharPtr suffix,
+  Boolean convertQuotes
 );
 NLM_EXTERN void s_AddPeriodToEnd (CharPtr someString);
 NLM_EXTERN Boolean s_RemovePeriodFromEnd (CharPtr someString);
@@ -1069,8 +1094,20 @@ NLM_EXTERN CharPtr goQualType [];
 NLM_EXTERN CharPtr goFieldType [];
 
 NLM_EXTERN CharPtr legalDbXrefs [];
+NLM_EXTERN CharPtr legalSrcDbXrefs [];
 NLM_EXTERN CharPtr legalRefSeqDbXrefs [];
 
+NLM_EXTERN Boolean DbxrefIsValid (
+  CharPtr name,
+  BoolPtr is_refseq_P,
+  BoolPtr is_source_P,
+  BoolPtr is_badcap_P,
+  CharPtr PNTR goodcapP
+);
+
+NLM_EXTERN void AddAllDbxrefsToBioseq (
+  BioseqPtr bsp
+);
 
 NLM_EXTERN void AddRefStatsBlock (
   Asn2gbWorkPtr awp
@@ -1118,7 +1155,8 @@ NLM_EXTERN void AddKeywordsBlock (
 );
 NLM_EXTERN void AddSegmentBlock (
   Asn2gbWorkPtr awp,
-  Boolean onePartOfSeg
+  Boolean onePartOfSeg,
+  Boolean is_na
 );
 NLM_EXTERN void AddSourceBlock (
   Asn2gbWorkPtr awp
