@@ -29,261 +29,12 @@
 *   
 * Version Creation Date: 4/1/91
 *
-* $Revision: 6.52 $
+* $Revision: 6.56 $
 *
 * File Description:  Sequence Utilities for objseq and objsset
 *
-* Modifications:  
-* --------------------------------------------------------------------------
-* $Log: sequtil.h,v $
-* Revision 6.52  2007/03/13 18:03:45  kans
-* added FreeSeqLocSetComponents to free sequence records referenced in a chain of SeqLocs
-*
-* Revision 6.51  2007/02/21 17:23:59  bollin
-* Added IsSkippableDbtag function, to use when TMSMART, BankIt, and NCBIFILE
-* IDs should be ignored (for displays, formatting, etc.)
-*
-* Revision 6.50  2006/11/15 18:01:22  kans
-* added SeqLocPartialCheckEx to control fetching of far location
-*
-* Revision 6.49  2006/01/05 14:11:56  bollin
-* added SeqLocPrintUseBestID function, which prints out the sequence location
-* but uses the "best" sequence ID instead of the one actually stored in the
-* SeqLoc.
-*
-* Revision 6.48  2005/08/03 18:28:36  kans
-* ValidateAccnDotVer returns -5 for missing version and -6 for bad version (not just digits)
-*
-* Revision 6.47  2005/01/26 23:38:46  kans
-* added ValidateAccnDotVer, check /compare in flatfile generator and validator
-*
-* Revision 6.46  2004/12/08 15:14:51  kans
-* added ACCN_DDBJ_GSS, accession prefix DE
-*
-* Revision 6.45  2004/08/04 17:15:16  kans
-* added AccnInUniProt - still need AccnIsSWISSPROT for old style
-*
-* Revision 6.44  2004/07/14 22:46:08  dondosha
-* Added GetAccessionVersionFromSeqId function to extract Accession.version from a Seq-id
-*
-* Revision 6.43  2004/04/01 13:43:05  lavr
-* Spell "occurred", "occurrence", and "occurring"
-*
-* Revision 6.42  2003/01/21 16:51:53  kans
-* added PRINTID_FASTA_ALL
-*
-* Revision 6.41  2002/10/29 21:17:31  kans
-* ValidateAccn and ValidateSeqID are public - implemented in asn2gnbk.c but prototyped in sequtil.h
-*
-* Revision 6.40  2002/06/20 18:38:43  kans
-* added FAA and GAA, ACCN_NCBI_WGS_PROT, ACCN_EMBL_WGS_PROT, ACCN_DDBJ_WGS_PROT, and ACCN_IS_WGS
-*
-* Revision 6.39  2002/06/19 17:13:48  kans
-* added ACCN_PDB, support for PDB in SeqIdFromAccessionDotVersion
-*
-* Revision 6.38  2002/05/15 19:49:50  cavanaug
-* Added WGS types to ACCN_IS_GENBANK, ACCN_IS_EMBL, ACCN_IS_DDBJ
-*
-* Revision 6.37  2002/01/22 18:49:15  kans
-* added ACCN_NCBI_WGS, ACCN_EMBL_WGS, and ACCN_DDBJ_WGS
-*
-* Revision 6.36  2002/01/16 16:59:38  camacho
-* Changed the type of buflen parameter in SeqIdWrite from Int2 to Uint4
-*
-* Revision 6.35  2001/09/28 22:42:49  vakatov
-* Renamed "new" to "x_new" -- to avoid clash with the C++ "operator new"
-*
-* Revision 6.34  2001/09/28 14:31:01  madden
-* Added functions BSCompressDNANew and GenericCompressDNAEx for long sequences with ambiguity
-*
-* Revision 6.33  2001/08/07 18:12:05  kans
-* added macros for EMBL and DDBJ TPA and TPA_PROT prefixes
-*
-* Revision 6.32  2001/07/03 21:42:02  kans
-* added macros and accession prefixes for TPA (third-party annotation) records
-*
-* Revision 6.31  2001/05/30 22:47:23  kans
-* fixed Mac compiler warnings about unwanted assignments, moved function prototypes to header, removed unused variables
-*
-* Revision 6.30  2001/04/27 15:46:20  madden
-* Add function RebuildDNA_4na
-*
-* Revision 6.29  2001/01/04 15:05:33  sicotte
-* fix ACCN_IS_AMBIGOUSDB macro
-*
-* Revision 6.28  2000/12/07 16:34:43  sicotte
-* Updated WHICH_db_accession and corresponding macros: I* accessions can no longer be proteins (they were PIR) and have completed hardcoding of N000?? accessions which can belong to twoDB. Added SeqIdFromAccessionEx, ACCN_PIR_FORMAT, and AccnIsSWISSPROT functions
-*
-* Revision 6.27  2000/11/16 17:23:26  sicotte
-* IS_protdb_accession is now true for any 3 letter accession and IS_ntdb_accession is now also true for any unknown accession-looking accession number
-*
-* Revision 6.26  2000/11/14 20:49:48  sicotte
-* add XM_ refseq prefix
-*
-* Revision 6.25  2000/10/31 21:20:06  vakatov
-* [WIN32] DLL'zation
-*
-* Revision 6.24  2000/10/27 20:10:58  shavirin
-* Added new function MakeNewProteinSeqIdExMT for MT save operation.
-*
-* Revision 6.23  2000/10/24 19:01:11  dondosha
-* Added prototype for UniqueLocalId(), previously in mblast.h
-*
-* Revision 6.22  2000/10/11 21:59:36  kans
-* added PRINTID_FASTA_GENERAL
-*
-* Revision 6.21  2000/07/11 15:01:50  kans
-* added SeqIdFromAccessionDotVersion for genome mapping project
-*
-* Revision 6.20  2000/05/17 17:22:44  dondosha
-* Added prototype for GetAccessionFromSeqId
-*
-* Revision 6.19  2000/05/03 17:56:05  sicotte
-* Added BC prefix ; changed refseq SeqID
-*
-* Revision 6.18  2000/02/04 16:13:17  kans
-* added prefix parameter to MakeNewProteinSeqIdEx
-*
-* Revision 6.17  1999/08/03 20:00:35  sicotte
-* Add SeqIdFindBestAccession
-*
-* Revision 6.16  1999/07/30 09:28:18  sicotte
-* Transfered fns from salutil.c(chappey) AddSeqId,SeqIdDupList,SeqIdDupBestList,SeqIdListfromSeqLoc
-*
-* Revision 6.15  1999/04/08 14:12:03  sicotte
-* Add SeqIdOrderInBioseqIdList (consider synomymous SeqIds)
-*
-* Revision 6.14  1999/04/02 17:32:00  vakatov
-* Added NLM_EXTERN for FindNuc() and FindProt() proto
-*
-* Revision 6.13  1999/04/01 22:23:23  sicotte
-* Fixed doubly occurring prototype.
-*
-* Revision 6.12  1999/04/01 17:41:21  sicotte
-* Added SeqIdInSeqLocList : Check if the Bioseq of SeqId sip is in list: may try to fetch the Bioseq
-*
-* Revision 6.11  1999/04/01 13:54:48  sicotte
-* Added SeqIdOrderInList(To find the position of a SeqId in a Chain)
-*       ExtractAccession ( To parse an accession into it's version and acc.)
-*       SeqIdFromAccession (To make the proper type of SeqId given an
-*                           accession string. (uses WHICH_db_accession)
-*       moved IS_ntdb_accession, IS_protdb_accession, WHICH_db_accession
-*          from accutils.ch to sequtil.ch
-*
-* Revision 6.10  1999/03/31 16:53:40  madden
-* Added FindNuc and FindProt functions for SeqEntryExplore
-*
-* Revision 6.9  1999/03/04 19:38:42  kans
-* now showing versions in accession numbers
-*
-* Revision 6.8  1999/02/02 21:38:12  kans
-* moved SHOWVERSION to header, SeqMgrAddIndexElement with and without version (JO)
-*
-* Revision 6.7  1999/01/27 22:04:46  kans
-* PRINTID_REPORT once again the end of the list
-*
-* Revision 6.6  1999/01/12 18:00:19  kans
-* SeqIdComp now ignores version if < 1, and added PRINTID_TEXTID_ACC_VER and PRINTID_TEXTID_ACC_ONLY formats for SeqIdWrite
-*
-* Revision 6.5  1998/07/28 16:41:09  kans
-* added MakeNewProteinSeqIdEx for faster creation of many new protein products
-*
-* Revision 6.4  1998/02/27 17:10:15  vakatov
-* [WIN32 DLL]  Declared some functions as NLM_EXTERN(DLL-exportable)
-*
-* Revision 6.3  1998/02/26 19:14:08  madden
-* Added AdjustOffSetsInSeqAlign
-*
-* Revision 6.2  1998/01/26 17:25:36  madden
-* Added TxGetQueryIdFromSeqAlign function
-*
-* Revision 6.1  1997/10/06 14:01:28  zjing
-* move TxGetSubjectId, GetScoreAndEvalue to sequtil.ch
-*
-* Revision 6.0  1997/08/25 18:07:27  madden
-* Revision changed to 6.0
-*
-* Revision 5.17  1997/08/12 21:25:19  kans
-* removed semicolons from SeqEntryList and BioseqList
-*
-* Revision 5.16  1997/06/19 18:39:03  vakatov
-* [WIN32,MSVC++]  Adopted for the "NCBIOBJ.LIB" DLL'ization
-*
-* Revision 5.15  1997/05/19 19:25:53  shavirin
-* Added definition for the function CorrectGeneFeatLocation()
-*
- * Revision 5.14  1997/04/09  18:43:50  tatiana
- * *** empty log message ***
- *
- * Revision 5.13  1997/04/09  18:00:34  tatiana
- * added MolTypeForGI()
- *
- * Revision 5.12  1997/03/28  21:18:38  shavirin
- * Added definition for a function BSRebuildDNA_4na()
- *
- * Revision 5.11  1997/03/06  22:48:21  shavirin
- * Moved definitions of SPCompress functions to seqport.h
- *
- * Revision 5.10  1997/03/06  21:28:25  shavirin
- * Added definitions for new set of functions: SPCompressDNA(), SPRebuildDNA(),
- * SPCompressNew(), SPCompressFree()
- *
- * Revision 5.9  1997/03/04  21:59:08  shavirin
- * Added definition of function GenericCompressDNA()
- *
- * Revision 5.8  1997/03/03  22:28:14  shavirin
- * Added definition for Compress DNA read/Write function
- *
- * Revision 5.7  1996/10/15  14:48:08  shavirin
- * Added definitions for new functions BSCompressDNA() and
- * BSRebuildDNA() handling ambiguity characters.
- *
- * Revision 5.6  1996/10/10  21:08:25  shavirin
- * Added definition for new function Convert4NaRandom(), that randomly
- * convert ncbi4na to ncbi2na encoding without SeqMapTable usage
- *
- * Revision 5.5  1996/08/07  20:12:59  epstein
- * move MuskSeqIdWrite, seqid_name, local_id_make and update_seq_loc from jzmisc to sequtil, to untangle the desktop and tools libraries from one another
- *
- * Revision 5.4  1996/07/15  14:42:42  epstein
- * add SeqEntryContainsSeqIdOfMolType() and FindCodingRegion()
- *
- * Revision 5.3  1996/06/24  18:38:33  epstein
- * move splicing functionality to sequtil.c
- *
- * Revision 5.2  1996/06/16  15:19:31  ostell
- * added delta seq support to SeqEntryPack
- *
- * Revision 5.1  1996/06/12  18:29:25  epstein
- * move SeqLocIntNew() and SeqLocPntNew() from edutil to sequtil
- *
- * Revision 5.0  1996/05/28  13:23:23  ostell
- * Set to revision 5.0
- *
- * Revision 4.3  1996/03/11  19:06:43  ostell
- * added MakeNewProteinSeqId()
- *
- * Revision 4.2  1996/01/31  16:09:15  tatiana
- * StringForSeqTech() added by Tatiana
- *
- * Revision 4.1  1995/10/17  04:15:37  ostell
- * added SeqLocFindPart()
- * made SeqLocFindNext() call it
- * added equiv_is_one to IS_one_loc()
- *
- * Revision 4.0  1995/07/26  13:49:01  ostell
- * force revision to 4.0
- *
- * Revision 2.30  1995/07/18  19:54:32  tatiana
- * add SeqIdWrite() to replace SeqIdPrint()
- *
- * Revision 2.29  1995/06/14  16:31:59  kans
- * EntrezASN1Detected function added (to be used by Sequin and ID)
- *
- * Revision 2.28  1995/05/19  04:02:08  ostell
- * added SeqLocAinB()
- * ==========================================================================
- */
+* ==========================================================================
+*/
 
 #ifndef _NCBI_SeqUtil_
 #define _NCBI_SeqUtil_
@@ -854,6 +605,8 @@ NLM_EXTERN Boolean SeqLocRevCmp(SeqLocPtr anp);
 
 NLM_EXTERN Int4 GetOffsetInLoc(SeqLocPtr of, SeqLocPtr in, Uint1 which_end);
 NLM_EXTERN Int4 GetOffsetInBioseq(SeqLocPtr of, BioseqPtr in, Uint1 which_end);
+NLM_EXTERN Int4 GetOffsetInBioseqEx (SeqLocPtr of, BioseqPtr in, Uint1 which_end, Boolean is_circular);
+NLM_EXTERN void GetLeftAndRightOffsetsInBioseq (SeqLocPtr of, BioseqPtr in, Int4Ptr left, Int4Ptr right, Boolean is_circular);
 NLM_EXTERN Int2 SeqLocOrder(SeqLocPtr a, SeqLocPtr b, BioseqPtr in);
 
 NLM_EXTERN Int2 SeqLocMol(SeqLocPtr seqloc);
@@ -1218,6 +971,15 @@ NLM_EXTERN Boolean LIBCALL NAccnIsDDBJ (CharPtr s);
 
 #define ACCN_DDBJ_GSS 61
 
+#define ACCN_NCBI_TSA 62
+#define ACCN_NCBI_TSA_PROT 63
+#define ACCN_EMBL_TSA 64
+#define ACCN_EMBL_TSA_PROT 65
+#define ACCN_DDBJ_TSA 66
+#define ACCN_DDBJ_TSA_PROT 67
+
+#define ACCN_REFSEQ_ARTIFICIAL_ASSEMBLY 68
+
 
 /* Some accessions prefix can be either protein or nucleotide 
    such as NCBI PATENT I, AR .. or segmented set Bioseqs 'AH'
@@ -1247,18 +1009,20 @@ NLM_EXTERN Boolean LIBCALL NAccnIsDDBJ (CharPtr s);
    Define to detect Genbank's accessions: Genbank-subsumed GSDB accession numbers
    are defined to be Genbank's as well as GSDB DIRSUB records.
 */
-#define ACCN_IS_GENBANK(c) ((((c)&65535) == ACCN_NCBI_GSDB) ||  (((c)&65535)==ACCN_GSDB_DIRSUB) || (((c)&65535) == ACCN_NCBI_EST) ||  (((c)&65535) == ACCN_NCBI_DIRSUB) ||  (((c)&65535) == ACCN_NCBI_GENOME) ||  (((c)&65535) == ACCN_NCBI_PATENT) ||  (((c)&65535) == ACCN_NCBI_HTGS) ||  (((c)&65535) == ACCN_NCBI_GSS) ||  (((c)&65535) == ACCN_NCBI_STS) ||  (((c)&65535) == ACCN_NCBI_BACKBONE) ||  (((c)&65535) == ACCN_NCBI_SEGSET)  ||  (((c)&65535) == ACCN_NCBI_WGS) ||  (((c)&65535) == ACCN_NCBI_OTHER)  || (((c)&65535) == ACCN_NCBI_PROT) || (((c)&65535) == ACCN_NCBI_cDNA) || (((c)&65535) == ACCN_EMBL_GB) || (((c)&65535) == ACCN_EMBL_GB_DDBJ || (((c)&65535) == ACCN_GB_DDBJ)) )
+#define ACCN_IS_GENBANK(c) ((((c)&65535) == ACCN_NCBI_GSDB) ||  (((c)&65535)==ACCN_GSDB_DIRSUB) || (((c)&65535) == ACCN_NCBI_EST) ||  (((c)&65535) == ACCN_NCBI_DIRSUB) ||  (((c)&65535) == ACCN_NCBI_GENOME) ||  (((c)&65535) == ACCN_NCBI_PATENT) ||  (((c)&65535) == ACCN_NCBI_HTGS) ||  (((c)&65535) == ACCN_NCBI_GSS) ||  (((c)&65535) == ACCN_NCBI_STS) ||  (((c)&65535) == ACCN_NCBI_BACKBONE) ||  (((c)&65535) == ACCN_NCBI_SEGSET)  ||  (((c)&65535) == ACCN_NCBI_WGS) ||  (((c)&65535) == ACCN_NCBI_OTHER)  || (((c)&65535) == ACCN_NCBI_PROT) || (((c)&65535) == ACCN_NCBI_cDNA) || (((c)&65535) == ACCN_NCBI_TSA) || (((c)&65535) == ACCN_NCBI_TSA_PROT) || (((c)&65535) == ACCN_EMBL_GB) || (((c)&65535) == ACCN_EMBL_GB_DDBJ || (((c)&65535) == ACCN_GB_DDBJ)) )
 
 /* XM_,NP_,NM_,NT_,NC_ reference sequence records created and curated by NCBI 
    REFSEQ project
 */
-#define ACCN_IS_REFSEQ(c) (((c)== ACCN_REFSEQ_PROT) || ((c)== ACCN_REFSEQ_mRNA) || ((c)== ACCN_REFSEQ_CONTIG) || ((c)== ACCN_REFSEQ_CHROMOSOME) || ((c)== ACCN_REFSEQ_mRNA_PREDICTED) || ((c)== ACCN_REFSEQ_PROT_PREDICTED) || ((c)== ACCN_REFSEQ_GENOMIC) || (((c)&65535)== ACCN_REFSEQ) )
+#define ACCN_IS_REFSEQ(c) (((c)== ACCN_REFSEQ_PROT) || ((c)== ACCN_REFSEQ_mRNA) || ((c)== ACCN_REFSEQ_CONTIG) || ((c)== ACCN_REFSEQ_CHROMOSOME) || ((c)== ACCN_REFSEQ_mRNA_PREDICTED) || ((c)== ACCN_REFSEQ_PROT_PREDICTED) || ((c)== ACCN_REFSEQ_GENOMIC) || ((c)== ACCN_REFSEQ_ARTIFICIAL_ASSEMBLY) || (((c)&65535)== ACCN_REFSEQ) )
 
 #define ACCN_IS_TPA(c) (((c)== ACCN_NCBI_TPA) || ((c)== ACCN_NCBI_TPA_PROT) || ((c)== ACCN_EMBL_TPA) || ((c)== ACCN_EMBL_TPA_PROT) || ((c)== ACCN_DDBJ_TPA) || ((c)== ACCN_DDBJ_TPA_PROT))
 
 #define ACCN_IS_WGS(c) (((c)== ACCN_NCBI_WGS) || ((c)== ACCN_NCBI_WGS_PROT) || ((c)== ACCN_EMBL_WGS) || ((c)== ACCN_EMBL_WGS_PROT) || ((c)== ACCN_DDBJ_WGS) || ((c)== ACCN_DDBJ_WGS_PROT))
 
-#define ACCN_IS_NCBI(c) (ACCN_IS_REFSEQ((c)) || ACCN_IS_GENBANK((c)) || ((c)== ACCN_NCBI_TPA) || ((c)== ACCN_NCBI_TPA_PROT) || ((c)== ACCN_NCBI_WGS) || ((c)== ACCN_NCBI_WGS_PROT))
+#define ACCN_IS_TSA(c) (((c)== ACCN_NCBI_TSA) || ((c)== ACCN_NCBI_TSA_PROT) || ((c)== ACCN_EMBL_TSA) || ((c)== ACCN_EMBL_TSA_PROT) || ((c)== ACCN_DDBJ_TSA) || ((c)== ACCN_DDBJ_TSA_PROT))
+
+#define ACCN_IS_NCBI(c) (ACCN_IS_REFSEQ((c)) || ACCN_IS_GENBANK((c)) || ((c)== ACCN_NCBI_TPA) || ((c)== ACCN_NCBI_TPA_PROT) || ((c)== ACCN_NCBI_WGS) || ((c)== ACCN_NCBI_WGS_PROT) || ((c)== ACCN_NCBI_TSA))
 
 /*
   Macro to detect EMBL accession numbers  (can also belong to another DB)
