@@ -64,6 +64,10 @@
 #define ERR_SEQ_INST_InternalGapsInSeqRaw  1,60
 #define ERR_SEQ_INST_SelfReferentialSequence  1,61
 #define ERR_SEQ_INST_WholeComponent  1,62
+#define ERR_SEQ_INST_TSAHistAssemblyMissing  1,63
+#define ERR_SEQ_INST_ProteinsHaveGeneralID  1,64
+#define ERR_SEQ_INST_HighNContent  1,65
+#define ERR_SEQ_INST_SeqLitDataLength0  1,66
 #define ERR_SEQ_DESCR  2,0
 #define ERR_SEQ_DESCR_BioSourceMissing  2,1
 #define ERR_SEQ_DESCR_InvalidForType  2,2
@@ -116,6 +120,18 @@
 #define ERR_SEQ_DESCR_LatLonState  2,49
 #define ERR_SEQ_DESCR_BadSpecificHost  2,50
 #define ERR_SEQ_DESCR_RefGeneTrackingIllegalStatus  2,51
+#define ERR_SEQ_DESCR_ReplacedCountryCode  2,52
+#define ERR_SEQ_DESCR_BadInstitutionCode  2,53
+#define ERR_SEQ_DESCR_BadCollectionCode  2,54
+#define ERR_SEQ_DESCR_BadVoucherID  2,55
+#define ERR_SEQ_DESCR_UnstructuredVoucher  2,56
+#define ERR_SEQ_DESCR_ChromosomeLocation  2,57
+#define ERR_SEQ_DESCR_MultipleSourceQualifiers  2,58
+#define ERR_SEQ_DESCR_UnbalancedParentheses  2,59
+#define ERR_SEQ_DESCR_MultipleSourceVouchers  2,60
+#define ERR_SEQ_DESCR_BadCountryCapitalization  2,61
+#define ERR_SEQ_DESCR_WrongVoucherType  2,62
+#define ERR_SEQ_DESCR_UserObjectProblem  2,63
 #define ERR_GENERIC  3,0
 #define ERR_GENERIC_NonAsciiAsn  3,1
 #define ERR_GENERIC_Spell  3,2
@@ -146,6 +162,11 @@
 #define ERR_SEQ_PKG_ArchaicFeatureProduct  4,14
 #define ERR_SEQ_PKG_GraphPackagingProblem  4,15
 #define ERR_SEQ_PKG_InternalGenBankSet  4,16
+#define ERR_SEQ_PKG_ConSetProblem  4,17
+#define ERR_SEQ_PKG_NoBioseqFound  4,18
+#define ERR_SEQ_PKG_INSDRefSeqPackaging  4,19
+#define ERR_SEQ_PKG_GPSnonGPSPackaging  4,20
+#define ERR_SEQ_PKG_RefSeqPopSet  4,21
 #define ERR_SEQ_FEAT  5,0
 #define ERR_SEQ_FEAT_InvalidForType  5,1
 #define ERR_SEQ_FEAT_PartialProblem  5,2
@@ -298,6 +319,20 @@
 #define ERR_SEQ_FEAT_UndesiredProteinName  5,149
 #define ERR_SEQ_FEAT_FeatureBeginsOrEndsInGap  5,150
 #define ERR_SEQ_FEAT_GeneOntologyTermMissingGOID  5,151
+#define ERR_SEQ_FEAT_PseudoRnaHasProduct  5,152
+#define ERR_SEQ_FEAT_PseudoRnaViaGeneHasProduct  5,153
+#define ERR_SEQ_FEAT_BadRRNAcomponentOrder  5,154
+#define ERR_SEQ_FEAT_BadRRNAcomponentOverlap  5,155
+#define ERR_SEQ_FEAT_MissingGeneLocusTag  5,156
+#define ERR_SEQ_FEAT_MultipleProtRefs  5,157
+#define ERR_SEQ_FEAT_BadInternalCharacter  5,158
+#define ERR_SEQ_FEAT_BadTrailingCharacter  5,159
+#define ERR_SEQ_FEAT_BadTrailingHyphen  5,160
+#define ERR_SEQ_FEAT_MultipleGeneOverlap  5,161
+#define ERR_SEQ_FEAT_BadCharInAuthorLastName  5,162
+#define ERR_SEQ_FEAT_PseudoCDSmRNArange  5,163
+#define ERR_SEQ_FEAT_ExtendablePartialProblem  5,164
+#define ERR_SEQ_FEAT_GeneXrefNeeded  5,165
 #define ERR_SEQ_ALIGN  6,0
 #define ERR_SEQ_ALIGN_SeqIdProblem  6,1
 #define ERR_SEQ_ALIGN_StrandRev  6,2
@@ -337,6 +372,8 @@
 #define ERR_SEQ_GRAPH_GraphGapScore  7,15
 #define ERR_SEQ_GRAPH_GraphOverlap  7,16
 #define ERR_SEQ_GRAPH_GraphBioseqId  7,17
+#define ERR_SEQ_GRAPH_GraphACGTScoreMany  7,18
+#define ERR_SEQ_GRAPH_GraphNScoreMany  7,19
 #define ERR_SEQ_ANNOT  8,0
 #define ERR_SEQ_ANNOT_AnnotIDs  8,1
 #define ERR_SEQ_ANNOT_AnnotLOCs  8,2
