@@ -9,8 +9,8 @@
 #include <asn.h>
 #endif
 
-static char * asnfilename = "asnentgene.h29";
-static AsnValxNode avnx[47] = {
+static char * asnfilename = "asnentgene.h30";
+static AsnValxNode avnx[50] = {
     {20,"live" ,0,0.0,&avnx[1] } ,
     {20,"secondary" ,1,0.0,&avnx[2] } ,
     {20,"discontinued" ,2,0.0,&avnx[3] } ,
@@ -26,64 +26,67 @@ static AsnValxNode avnx[47] = {
     {20,"pseudo" ,7,0.0,&avnx[13] } ,
     {20,"transposon" ,8,0.0,&avnx[14] } ,
     {20,"miscRNA" ,9,0.0,&avnx[15] } ,
+    {20,"ncRNA" ,10,0.0,&avnx[16] } ,
     {20,"other" ,255,0.0,NULL } ,
-    {20,"cyto" ,0,0.0,&avnx[17] } ,
-    {20,"bp" ,1,0.0,&avnx[18] } ,
-    {20,"cM" ,2,0.0,&avnx[19] } ,
-    {20,"cR" ,3,0.0,&avnx[20] } ,
+    {20,"cyto" ,0,0.0,&avnx[18] } ,
+    {20,"bp" ,1,0.0,&avnx[19] } ,
+    {20,"cM" ,2,0.0,&avnx[20] } ,
+    {20,"cR" ,3,0.0,&avnx[21] } ,
     {20,"min" ,4,0.0,NULL } ,
     {2,NULL,0,0.0,NULL } ,
     {2,NULL,0,0.0,NULL } ,
     {2,NULL,0,0.0,NULL } ,
-    {20,"genomic" ,1,0.0,&avnx[25] } ,
-    {20,"pre-RNA" ,2,0.0,&avnx[26] } ,
-    {20,"mRNA" ,3,0.0,&avnx[27] } ,
-    {20,"rRNA" ,4,0.0,&avnx[28] } ,
-    {20,"tRNA" ,5,0.0,&avnx[29] } ,
-    {20,"snRNA" ,6,0.0,&avnx[30] } ,
-    {20,"scRNA" ,7,0.0,&avnx[31] } ,
-    {20,"peptide" ,8,0.0,&avnx[32] } ,
-    {20,"other-genetic" ,9,0.0,&avnx[33] } ,
-    {20,"genomic-mRNA" ,10,0.0,&avnx[34] } ,
-    {20,"cRNA" ,11,0.0,&avnx[35] } ,
-    {20,"mature-peptide" ,12,0.0,&avnx[36] } ,
-    {20,"pre-protein" ,13,0.0,&avnx[37] } ,
-    {20,"miscRNA" ,14,0.0,&avnx[38] } ,
-    {20,"snoRNA" ,15,0.0,&avnx[39] } ,
-    {20,"property" ,16,0.0,&avnx[40] } ,
-    {20,"reference" ,17,0.0,&avnx[41] } ,
-    {20,"generif" ,18,0.0,&avnx[42] } ,
-    {20,"phenotype" ,19,0.0,&avnx[43] } ,
-    {20,"complex" ,20,0.0,&avnx[44] } ,
-    {20,"compound" ,21,0.0,&avnx[45] } ,
-    {20,"comment" ,254,0.0,&avnx[46] } ,
+    {20,"genomic" ,1,0.0,&avnx[26] } ,
+    {20,"pre-RNA" ,2,0.0,&avnx[27] } ,
+    {20,"mRNA" ,3,0.0,&avnx[28] } ,
+    {20,"rRNA" ,4,0.0,&avnx[29] } ,
+    {20,"tRNA" ,5,0.0,&avnx[30] } ,
+    {20,"snRNA" ,6,0.0,&avnx[31] } ,
+    {20,"scRNA" ,7,0.0,&avnx[32] } ,
+    {20,"peptide" ,8,0.0,&avnx[33] } ,
+    {20,"other-genetic" ,9,0.0,&avnx[34] } ,
+    {20,"genomic-mRNA" ,10,0.0,&avnx[35] } ,
+    {20,"cRNA" ,11,0.0,&avnx[36] } ,
+    {20,"mature-peptide" ,12,0.0,&avnx[37] } ,
+    {20,"pre-protein" ,13,0.0,&avnx[38] } ,
+    {20,"miscRNA" ,14,0.0,&avnx[39] } ,
+    {20,"snoRNA" ,15,0.0,&avnx[40] } ,
+    {20,"property" ,16,0.0,&avnx[41] } ,
+    {20,"reference" ,17,0.0,&avnx[42] } ,
+    {20,"generif" ,18,0.0,&avnx[43] } ,
+    {20,"phenotype" ,19,0.0,&avnx[44] } ,
+    {20,"complex" ,20,0.0,&avnx[45] } ,
+    {20,"compound" ,21,0.0,&avnx[46] } ,
+    {20,"ncRNA" ,22,0.0,&avnx[47] } ,
+    {20,"gene-group" ,23,0.0,&avnx[48] } ,
+    {20,"comment" ,254,0.0,&avnx[49] } ,
     {20,"other" ,255,0.0,NULL } };
 
 static AsnType atx[104] = {
   {401, "Entrezgene" ,1,0,0,0,0,1,0,0,NULL,&atx[14],&atx[1],0,&atx[101]} ,
   {0, "track-info" ,128,0,0,1,0,0,0,0,NULL,&atx[2],NULL,0,&atx[15]} ,
-  {411, "Gene-track" ,1,0,0,0,0,0,0,0,NULL,&atx[14],&atx[3],0,&atx[28]} ,
+  {403, "Gene-track" ,1,0,0,0,0,1,0,0,NULL,&atx[14],&atx[3],0,&atx[19]} ,
   {0, "geneid" ,128,0,0,0,0,0,0,0,NULL,&atx[4],NULL,0,&atx[5]} ,
   {302, "INTEGER" ,0,2,0,0,0,0,0,0,NULL,NULL,NULL,0,NULL} ,
   {0, "status" ,128,1,0,0,1,0,0,0,&avnx[4],&atx[4],&avnx[0],0,&atx[6]} ,
   {0, "current-id" ,128,2,0,1,0,0,0,0,NULL,&atx[9],&atx[7],0,&atx[10]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[8],NULL,0,NULL} ,
-  {407, "Dbtag" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[11]} ,
+  {408, "Dbtag" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[11]} ,
   {312, "SEQUENCE OF" ,0,16,0,0,0,0,0,0,NULL,NULL,NULL,0,NULL} ,
   {0, "create-date" ,128,3,0,0,0,0,0,0,NULL,&atx[11],NULL,0,&atx[12]} ,
-  {408, "Date" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[72]} ,
+  {409, "Date" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[72]} ,
   {0, "update-date" ,128,4,0,0,0,0,0,0,NULL,&atx[11],NULL,0,&atx[13]} ,
   {0, "discontinue-date" ,128,5,0,1,0,0,0,0,NULL,&atx[11],NULL,0,NULL} ,
   {311, "SEQUENCE" ,0,16,0,0,0,0,0,0,NULL,NULL,NULL,0,NULL} ,
   {0, "type" ,128,1,0,0,0,0,0,0,NULL,&atx[4],&avnx[5],0,&atx[16]} ,
   {0, "source" ,128,2,0,0,0,0,0,0,NULL,&atx[17],NULL,0,&atx[18]} ,
-  {405, "BioSource" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[23]} ,
+  {406, "BioSource" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[23]} ,
   {0, "gene" ,128,3,0,0,0,0,0,0,NULL,&atx[19],NULL,0,&atx[20]} ,
-  {403, "Gene-ref" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[21]} ,
+  {404, "Gene-ref" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[21]} ,
   {0, "prot" ,128,4,0,1,0,0,0,0,NULL,&atx[21],NULL,0,&atx[22]} ,
-  {404, "Prot-ref" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[17]} ,
+  {405, "Prot-ref" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[17]} ,
   {0, "rna" ,128,5,0,1,0,0,0,0,NULL,&atx[23],NULL,0,&atx[24]} ,
-  {406, "RNA-ref" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[8]} ,
+  {407, "RNA-ref" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[8]} ,
   {0, "summary" ,128,6,0,1,0,0,0,0,NULL,&atx[25],NULL,0,&atx[26]} ,
   {323, "VisibleString" ,0,26,0,0,0,0,0,0,NULL,NULL,NULL,0,NULL} ,
   {0, "location" ,128,7,0,1,0,0,0,0,NULL,&atx[9],&atx[27],0,&atx[35]} ,
@@ -92,7 +95,7 @@ static AsnType atx[104] = {
   {0, "display-str" ,128,0,0,0,0,0,0,0,NULL,&atx[25],NULL,0,&atx[30]} ,
   {0, "method" ,128,1,0,0,0,0,0,0,NULL,&atx[34],&atx[31],0,NULL} ,
   {0, "proxy" ,128,0,0,0,0,0,0,0,NULL,&atx[25],NULL,0,&atx[32]} ,
-  {0, "map-type" ,128,1,0,0,0,0,0,0,NULL,&atx[33],&avnx[16],0,NULL} ,
+  {0, "map-type" ,128,1,0,0,0,0,0,0,NULL,&atx[33],&avnx[17],0,NULL} ,
   {310, "ENUMERATED" ,0,10,0,0,0,0,0,0,NULL,NULL,NULL,0,NULL} ,
   {315, "CHOICE" ,0,-1,0,0,0,0,0,0,NULL,NULL,NULL,0,NULL} ,
   {0, "gene-source" ,128,8,0,1,0,0,0,0,NULL,&atx[36],NULL,0,&atx[45]} ,
@@ -101,14 +104,14 @@ static AsnType atx[104] = {
   {0, "src-int" ,128,1,0,1,0,0,0,0,NULL,&atx[4],NULL,0,&atx[39]} ,
   {0, "src-str1" ,128,2,0,1,0,0,0,0,NULL,&atx[25],NULL,0,&atx[40]} ,
   {0, "src-str2" ,128,3,0,1,0,0,0,0,NULL,&atx[25],NULL,0,&atx[41]} ,
-  {0, "gene-display" ,128,4,0,0,1,0,0,0,&avnx[21],&atx[42],NULL,0,&atx[43]} ,
+  {0, "gene-display" ,128,4,0,0,1,0,0,0,&avnx[22],&atx[42],NULL,0,&atx[43]} ,
   {301, "BOOLEAN" ,0,1,0,0,0,0,0,0,NULL,NULL,NULL,0,NULL} ,
-  {0, "locus-display" ,128,5,0,0,1,0,0,0,&avnx[22],&atx[42],NULL,0,&atx[44]} ,
-  {0, "extra-terms" ,128,6,0,0,1,0,0,0,&avnx[23],&atx[42],NULL,0,NULL} ,
+  {0, "locus-display" ,128,5,0,0,1,0,0,0,&avnx[23],&atx[42],NULL,0,&atx[44]} ,
+  {0, "extra-terms" ,128,6,0,0,1,0,0,0,&avnx[24],&atx[42],NULL,0,NULL} ,
   {0, "locus" ,128,9,0,1,0,0,0,0,NULL,&atx[9],&atx[46],0,&atx[83]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[47],NULL,0,NULL} ,
   {414, "Gene-commentary" ,1,0,0,0,0,0,0,0,NULL,&atx[14],&atx[48],0,&atx[56]} ,
-  {0, "type" ,128,0,0,0,0,0,0,0,NULL,&atx[4],&avnx[24],0,&atx[49]} ,
+  {0, "type" ,128,0,0,0,0,0,0,0,NULL,&atx[4],&avnx[25],0,&atx[49]} ,
   {0, "heading" ,128,1,0,1,0,0,0,0,NULL,&atx[25],NULL,0,&atx[50]} ,
   {0, "label" ,128,2,0,1,0,0,0,0,NULL,&atx[25],NULL,0,&atx[51]} ,
   {0, "text" ,128,3,0,1,0,0,0,0,NULL,&atx[25],NULL,0,&atx[52]} ,
@@ -121,7 +124,7 @@ static AsnType atx[104] = {
   {0, "value" ,128,1,0,0,0,0,0,0,NULL,&atx[25],NULL,0,NULL} ,
   {0, "refs" ,128,7,0,1,0,0,0,0,NULL,&atx[9],&atx[60],0,&atx[62]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[61],NULL,0,NULL} ,
-  {410, "Pub" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[2]} ,
+  {411, "Pub" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[28]} ,
   {0, "source" ,128,8,0,1,0,0,0,0,NULL,&atx[9],&atx[63],0,&atx[70]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[64],NULL,0,NULL} ,
   {416, "Other-source" ,1,0,0,0,0,0,0,0,NULL,&atx[14],&atx[65],0,NULL} ,
@@ -132,7 +135,7 @@ static AsnType atx[104] = {
   {0, "post-text" ,128,4,0,1,0,0,0,0,NULL,&atx[25],NULL,0,NULL} ,
   {0, "genomic-coords" ,128,9,0,1,0,0,0,0,NULL,&atx[9],&atx[71],0,&atx[73]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[72],NULL,0,NULL} ,
-  {409, "Seq-loc" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[61]} ,
+  {410, "Seq-loc" ,1,0,0,0,0,0,1,0,NULL,NULL,NULL,0,&atx[61]} ,
   {0, "seqs" ,128,10,0,1,0,0,0,0,NULL,&atx[9],&atx[74],0,&atx[75]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[72],NULL,0,NULL} ,
   {0, "products" ,128,11,0,1,0,0,0,0,NULL,&atx[9],&atx[76],0,&atx[77]} ,
@@ -161,12 +164,12 @@ static AsnType atx[104] = {
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[56],NULL,0,NULL} ,
   {0, "non-unique-keys" ,128,18,0,1,0,0,0,0,NULL,&atx[9],&atx[100],0,NULL} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[8],NULL,0,NULL} ,
-  {402, "Entrezgene-Set" ,1,0,0,0,0,1,0,0,NULL,&atx[103],&atx[102],0,&atx[19]} ,
+  {402, "Entrezgene-Set" ,1,0,0,0,0,1,0,0,NULL,&atx[103],&atx[102],0,&atx[2]} ,
   {0, NULL,1,-1,0,0,0,0,0,0,NULL,&atx[0],NULL,0,NULL} ,
   {314, "SET OF" ,0,17,0,0,0,0,0,0,NULL,NULL,NULL,0,NULL} };
 
 static AsnModule ampx[1] = {
-  { "NCBI-Entrezgene" , "asnentgene.h29",&atx[0],NULL,NULL,0,0} };
+  { "NCBI-Entrezgene" , "asnentgene.h30",&atx[0],NULL,NULL,0,0} };
 
 static AsnValxNodePtr avn = avnx;
 static AsnTypePtr at = atx;
