@@ -1,4 +1,4 @@
-/*  $Id: ncbi_connection.c,v 6.49 2007/10/17 15:25:43 kazimird Exp $
+/* $Id: ncbi_connection.c,v 6.55 2008/11/19 16:34:48 kazimird Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -37,24 +37,23 @@
 #include <stdlib.h>
 #include <string.h>
 
+#define NCBI_USE_ERRCODE_X   Connect_Connection
+
 
 /***********************************************************************
  *  INTERNAL
  ***********************************************************************/
 
-#define NCBI_USE_ERRCODE_X   Connect_Connection
-
-
 /* Standard logging message
  */
-#define CONN_LOG_EX(subcode, level, descr, status)         \
-  CORE_LOGF_X(subcode, level,                                \
-            ("%s (connector \"%s\", error \"%s\")", descr, \
-             conn->meta.get_type                           \
-             ? conn->meta.get_type(conn->meta.c_get_type)  \
-             : "Unknown", IO_StatusStr(status)))
+#define CONN_LOG_EX(subcode, level, message, status)                \
+  CORE_LOGF_X(subcode, level,                                       \
+              ("%s (connector \"%s\", status \"%s\")", message,     \
+               conn->meta.get_type                                  \
+               ? conn->meta.get_type(conn->meta.c_get_type)         \
+               : "Unknown", IO_StatusStr(status)))
 
-#define CONN_LOG(subcode, level, descr)   \
+#define CONN_LOG(subcode, level, descr)         \
     CONN_LOG_EX(subcode, level, descr, status)
 
 #ifdef _DEBUG
@@ -65,12 +64,12 @@
 
 /* Standard macros to verify that the passed connection handle is not NULL
  */
-#define CONN_NOT_NULL_EX(subcode, func_name, status)            \
-  if ( !conn ) {                                                \
+#define CONN_NOT_NULL_EX(subcode, func_name, status)              \
+    if (!conn) {                                                  \
       CORE_LOG_X(subcode, eLOG_Error, "CONN_" #func_name          \
-               "(conn, ...) -- passed NULL connection handle"); \
-      assert(conn);                                             \
-      return status;                                            \
+                 "(conn, ...) -- NULL connection handle");        \
+      assert(conn);                                               \
+      return status;                                              \
   }
 
 #define CONN_NOT_NULL(subcode, func_name)   \
@@ -124,7 +123,7 @@ extern EIO_Status CONN_Create
     CONN conn = (SConnection*) calloc(1, sizeof(SConnection));
     EIO_Status status = eIO_Unknown;
 
-    if ( conn ) {
+    if (conn) {
         conn->state     = eCONN_Unusable;
         conn->o_timeout = kDefaultTimeout;
         conn->r_timeout = kDefaultTimeout;
@@ -160,7 +159,7 @@ extern EIO_Status CONN_ReInit
     }
 
     /* reset and close current connector(s), if any */
-    if ( conn->meta.list ) {
+    if (conn->meta.list) {
 #ifdef IMPLEMENTED__CONN_WaitAsync
         /* cancel async. i/o event handler */
         CONN_WaitAsync(conn, eIO_ReadWrite, 0, 0, 0);
@@ -171,12 +170,12 @@ extern EIO_Status CONN_ReInit
         }}
         /* call current connector's "FLUSH" and "CLOSE" methods */
         if (conn->state == eCONN_Open) {
-            if ( conn->meta.flush ) {
+            if (conn->meta.flush) {
                 conn->meta.flush(conn->meta.c_flush,
                                  conn->c_timeout == kDefaultTimeout ?
                                  conn->meta.default_timeout : conn->c_timeout);
             }
-            if ( conn->meta.close ) {
+            if (conn->meta.close) {
                 status = conn->meta.close(conn->meta.c_close,
                                           conn->c_timeout == kDefaultTimeout
                                           ? conn->meta.default_timeout
@@ -203,7 +202,7 @@ extern EIO_Status CONN_ReInit
             }
         }
 
-        if ( !x_conn ) {
+        if (!x_conn) {
             /* Entirely new connector - remove the old connector stack first */
             METACONN_Remove(&conn->meta, 0);
             assert(conn->meta.list == 0);
@@ -380,7 +379,7 @@ extern EIO_Status CONN_Wait
     assert(conn->state == eCONN_Open  &&  conn->meta.list != 0);
 
     /* check if there is a PEEK'ed data in the input */
-    if (event == eIO_Read && BUF_Size(conn->buf))
+    if (event == eIO_Read  &&  BUF_Size(conn->buf))
         return eIO_Success;
 
     /* call current connector's "WAIT" method */
@@ -389,12 +388,16 @@ extern EIO_Status CONN_Wait
         : eIO_NotSupported;
 
     if (status != eIO_Success) {
-        if (status != eIO_Timeout)
-            CONN_LOG(14, eLOG_Error, "[CONN_Wait]  Error waiting on I/O");
-        else if (!timeout || timeout->sec || timeout->usec)
-            CONN_LOG(15, eLOG_Warning, "[CONN_Wait]  I/O timed out");
+        if (status != eIO_Timeout) {
+            CONN_LOG(14, eLOG_Error, event == eIO_Read
+                     ? "[CONN_Wait]  Error waiting on read"
+                     : "[CONN_Wait]  Error waiting on write");
+        } else if (!timeout  ||  (timeout->sec | timeout->usec)) {
+            CONN_LOG(15, eLOG_Warning, event == eIO_Read
+                     ? "[CONN_Wait]  Read timed out"
+                     : "[CONN_Wait]  Write timed out");
+        }
     }
-
     return status;
 }
 
@@ -405,28 +408,37 @@ static EIO_Status s_CONN_Write
  size_t      size,
  size_t*     n_written)
 {
+    const STimeout* wto;
     EIO_Status status;
 
     assert(*n_written == 0);
 
     /* check if the write method is specified at all */
-    if ( !conn->meta.write ) {
+    if (!conn->meta.write) {
         status = eIO_NotSupported;
         CONN_LOG(16, eLOG_Error, "[CONN_Write]  Unable to write data");
         return status;
     }
 
     /* call current connector's "WRITE" method */
-    status = conn->meta.write(conn->meta.c_write, buf, size, n_written,
-                              conn->w_timeout == kDefaultTimeout ?
-                              conn->meta.default_timeout : conn->w_timeout);
+    wto = (conn->w_timeout == kDefaultTimeout
+           ? conn->meta.default_timeout : conn->w_timeout);
+    status = conn->meta.write(conn->meta.c_write, buf, size, n_written, wto);
 
     if (status != eIO_Success) {
-        if ( *n_written ) {
+        if (*n_written) {
             CONN_TRACE("[CONN_Write]  Write error");
             status = eIO_Success;
-        } else if ( size )
-            CONN_LOG(17, eLOG_Error, "[CONN_Write]  Cannot write data");
+        } else if (size) {
+            ELOG_Level level;
+            if (status != eIO_Timeout  ||  conn->w_timeout == kDefaultTimeout)
+                level = eLOG_Error;
+            else if (wto  &&  (wto->sec | wto->usec))
+                level = eLOG_Warning;
+            else
+                level = eLOG_Trace;
+            CONN_LOG(17, level, "[CONN_Write]  Cannot write data");
+        }
     }
     return status;
 }
@@ -442,14 +454,12 @@ static EIO_Status s_CONN_WritePersist
 
     assert(*n_written == 0);
 
-    for (;;) {
+    do {
         size_t x_written = 0;
         status = s_CONN_Write(conn, (char*) buf + *n_written,
                               size - *n_written, &x_written);
         *n_written += x_written;
-        if (*n_written == size  ||  status != eIO_Success)
-            break;
-    }
+    } while (*n_written != size  &&  status == eIO_Success);
 
     return status;
 }
@@ -521,7 +531,7 @@ extern EIO_Status CONN_Flush
     assert(conn->state == eCONN_Open  &&  conn->meta.list != 0);
 
     /* call current connector's "FLUSH" method */
-    if ( !conn->meta.flush )
+    if (!conn->meta.flush)
         return eIO_Success;
     status = conn->meta.flush(conn->meta.c_flush,
                               conn->w_timeout == kDefaultTimeout ?
@@ -541,19 +551,20 @@ static EIO_Status s_CONN_Read
  size_t*     n_read,
  int/*bool*/ peek)
 {
+    const STimeout* rto;
     EIO_Status status;
 
     assert(*n_read == 0);
 
     /* check if the read method is specified at all */
-    if ( !conn->meta.read ) {
+    if (!conn->meta.read) {
         status = eIO_NotSupported;
         CONN_LOG(22, eLOG_Error, "[CONN_Read]  Unable to read data");
         return status;
     }
 
     /* read data from the internal peek buffer, if any */
-    if ( size ) {
+    if (size) {
         *n_read = peek
             ? BUF_Peek(conn->buf, buf, size) : BUF_Read(conn->buf, buf, size);
         if (*n_read == size)
@@ -564,22 +575,33 @@ static EIO_Status s_CONN_Read
     /* read data from the connection */
     {{
         size_t x_read = 0;
+        rto = (conn->r_timeout == kDefaultTimeout
+               ? conn->meta.default_timeout : conn->r_timeout);
         /* call current connector's "READ" method */
-        status = conn->meta.read(conn->meta.c_read, buf, size- *n_read,&x_read,
-                                 conn->r_timeout == kDefaultTimeout ?
-                                 conn->meta.default_timeout : conn->r_timeout);
+        status = conn->meta.read(conn->meta.c_read, buf, size - *n_read,
+                                 &x_read, rto);
         *n_read += x_read;
-        if ( peek )  /* save data in the internal peek buffer */
-            verify(BUF_Write(&conn->buf, buf, x_read));
+
+        /* save data in the internal peek buffer */
+        if (peek  &&  !BUF_Write(&conn->buf, buf, x_read)) {
+            CONN_LOG_EX(32, eLOG_Error, "[CONN_Read]  Cannot save peek data",
+                        eIO_Unknown);
+        }
     }}
 
     if (status != eIO_Success) {
-        if ( *n_read ) {
+        if (*n_read) {
             CONN_TRACE("[CONN_Read]  Read error");
             status = eIO_Success;
         } else if (size  &&  status != eIO_Closed) {
-            CONN_LOG(23, status == eIO_Timeout ? eLOG_Warning : eLOG_Error,
-                     "[CONN_Read]  Cannot read data");
+            ELOG_Level level;
+            if (status != eIO_Timeout  ||  conn->r_timeout == kDefaultTimeout)
+                level = eLOG_Error;
+            else if (rto  &&  (rto->sec | rto->usec))
+                level = eLOG_Warning;
+            else
+                level = eLOG_Trace;
+            CONN_LOG(23, level, "[CONN_Read]  Cannot read data");
         }
     }
     return status;
@@ -606,7 +628,7 @@ static EIO_Status s_CONN_ReadPersist
         if (*n_read == size  ||  status != eIO_Success)
             break;
         /* flush the unwritten output data (if any) */
-        if ( conn->meta.flush ) {
+        if (conn->meta.flush) {
             conn->meta.flush(conn->meta.c_flush,
                              conn->r_timeout == kDefaultTimeout ?
                              conn->meta.default_timeout : conn->r_timeout);
@@ -643,7 +665,7 @@ extern EIO_Status CONN_Read
     assert(conn->state == eCONN_Open  &&  conn->meta.list != 0);
 
     /* flush the unwritten output data (if any) */
-    if ( conn->meta.flush ) {
+    if (conn->meta.flush) {
         conn->meta.flush(conn->meta.c_flush,
                          conn->r_timeout == kDefaultTimeout ?
                          conn->meta.default_timeout : conn->r_timeout);
@@ -690,7 +712,7 @@ extern EIO_Status CONN_ReadLine
     if (status == eIO_Success) {
         assert(conn->state == eCONN_Open  &&  conn->meta.list != 0);
         /* flush the unwritten output data (if any) */
-        if ( conn->meta.flush ) {
+        if (conn->meta.flush) {
             conn->meta.flush(conn->meta.c_flush,
                              conn->r_timeout == kDefaultTimeout ?
                              conn->meta.default_timeout : conn->r_timeout);
@@ -718,8 +740,11 @@ extern EIO_Status CONN_ReadLine
                 line[len] = c;
             len++;
         }
-        if (i < x_read  &&  !BUF_PushBack(&conn->buf, x_buf + i, x_read - i))
-            status = eIO_Unknown;
+        if (i < x_read  &&  !BUF_PushBack(&conn->buf, x_buf + i, x_read - i)) {
+            CONN_LOG_EX(33, eLOG_Error,
+                        "[CONN_ReadLine]  Cannot push extra data back",
+                        eIO_Unknown);
+        }
         if (done  ||  len >= size)
             break;
     }
@@ -744,7 +769,7 @@ extern EIO_Status CONN_Status(CONN conn, EIO_Event dir)
     if (conn->state == eCONN_Closed)
         return eIO_Closed;
 
-    if ( !conn->meta.status )
+    if (!conn->meta.status)
         return eIO_NotSupported;
 
     return conn->meta.status(conn->meta.c_status, dir);
@@ -765,10 +790,10 @@ extern EIO_Status CONN_Close(CONN conn)
     /* allow close CB only once */
     memset(&conn->cbs[eCONN_OnClose], 0, sizeof(conn->cbs[eCONN_OnClose]));
     /* call it! */
-    if ( func )
-        (*func)(conn, eCONN_OnClose, data);
+    if (func)
+        func(conn, eCONN_OnClose, data);
     /* now close the connection - this also makes it "eCONN_Unusable" */
-    if ( conn->meta.list )
+    if (conn->meta.list)
         CONN_ReInit(conn, 0);
     BUF_Destroy(conn->buf);
     conn->buf = 0;
@@ -789,9 +814,9 @@ extern EIO_Status CONN_SetCallback
 
     CONN_NOT_NULL(28, SetCallback);
 
-    if ( old_cb )
+    if (old_cb)
         *old_cb = conn->cbs[i];
-    if ( new_cb )
+    if (new_cb)
         conn->cbs[i] = *new_cb;
     return eIO_Success;
 }
@@ -836,16 +861,16 @@ extern EIO_Status CONN_WaitAsync
         x_connector->vtable.wait_async(x_connector->handle, 0, 0) :
         eIO_NotSupported;
     if (status != eIO_Success) {
-        CONN_LOG(30, eLOG_Error, "[CONN_WaitAsync]  Cannot reset the handler");
+        CONN_LOG(30, eLOG_Error, "[CONN_WaitAsync]  Cannot reset handler");
         return status;
     }
-    if ( x_data->cleanup )
+    if (x_data->cleanup)
         x_data->cleanup(x_data->data);
     memset(x_data, '\0', sizeof(*x_data));
 
     /* set new handler, if specified */
     /* (call current connector's "WAIT_ASYNC" method with new handler/data) */
-    if ( !handler )
+    if (!handler)
         return eIO_Success;
 
     x_data->conn       = conn;
