@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   3/4/04
 *
-* $Revision: 1.35 $
+* $Revision: 1.48 $
 *
 * File Description:
 *
@@ -61,7 +61,7 @@
 #include <accpubseq.h>
 #endif
 
-#define ASN2FSA_APP_VER "1.8"
+#define ASN2FSA_APP_VER "3.1"
 
 CharPtr ASN2FSA_APPLICATION = ASN2FSA_APP_VER;
 
@@ -578,7 +578,7 @@ static void ProcessSingleRecord (
       return;
     }
 
-    dataptr = ReadAsnFastaOrFlatFile (fp, &datatype, NULL, FALSE, FALSE, FALSE, FALSE);
+    dataptr = ReadAsnFastaOrFlatFile (fp, &datatype, NULL, FALSE, FALSE, TRUE, FALSE);
 
     FileClose (fp);
 
@@ -695,8 +695,11 @@ static void ProcessSingleRecord (
 
   omp = ObjMgrGet ();
   ObjMgrReapOne (omp);
+  SeqMgrClearBioseqIndex ();
   ObjMgrFreeCache (0);
   FreeSeqIdGiCache ();
+
+  SeqEntrySetScope (NULL);
 }
 
 static void ProcessMultipleRecord (
@@ -712,7 +715,7 @@ static void ProcessMultipleRecord (
   AsnTypePtr     atp, atp_bss, atp_desc, atp_se;
   BioseqPtr      bsp;
   ValNodePtr     bsplist;
-  Char           buf [64], cmmd [256], file [FILENAME_MAX], path [PATH_MAX], longest [64];
+  Char           buf [64], file [FILENAME_MAX], path [PATH_MAX], longest [64];
   StreamFlgType  flags = STREAM_CORRECT_INVAL;
   FILE           *fp;
   Int4           numrecords = 0;
@@ -720,6 +723,7 @@ static void ProcessMultipleRecord (
   ObjMgrPtr      omp;
   time_t         starttime, stoptime, worsttime;
 #ifdef OS_UNIX
+  Char           cmmd [256];
   CharPtr        gzcatprog;
   int            ret;
   Boolean        usedPopen = FALSE;
@@ -876,8 +880,11 @@ static void ProcessMultipleRecord (
       SeqEntryFree (sep);
       omp = ObjMgrGet ();
       ObjMgrReapOne (omp);
+      SeqMgrClearBioseqIndex ();
       ObjMgrFreeCache (0);
       FreeSeqIdGiCache ();
+
+      SeqEntrySetScope (NULL);
     } else {
       AsnReadVal (aip, atp, NULL);
     }
