@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   1/23/07
 *
-* $Revision: 1.2 $
+* $Revision: 1.5 $
 *
 * File Description:
 *
@@ -81,7 +81,6 @@ typedef struct brflags {
   Int4     numrecords;
   ValNodePtr            sep_list;
   ValNodePtr            bsplist;
-  ValNodePtr            filename_list;
   BarcodeTestConfigData bcd;
 } BRFlagData, PNTR BRFlagPtr;
 
@@ -492,21 +491,6 @@ static ValNodePtr DoLockFarComponents (
   return rsult;
 }
 
-static ValNodePtr FreeFilenameList (ValNodePtr filename_list)
-{
-  ValNodePtr vnp_next;
-  if (filename_list == NULL) return NULL;
-  vnp_next = filename_list->next;
-  filename_list->next = NULL;
-  if (filename_list->choice == 2) {
-    filename_list = ValNodeFreeData (filename_list);
-  } else {
-    filename_list = ValNodeFree (filename_list);
-  }
-  vnp_next = FreeFilenameList (vnp_next);
-  return NULL;
-}
-
 static void ProcessSeqEntryList (BRFlagPtr drfp, CharPtr filename)
 {
   ValNodePtr  pass_fail_list, vnp;
@@ -583,8 +567,6 @@ static void ProcessSeqEntryList (BRFlagPtr drfp, CharPtr filename)
   SeqEntrySetScope (NULL);
   drfp->sep_list = ValNodeFree (drfp->sep_list);
   
-  drfp->filename_list = FreeFilenameList (drfp->filename_list);
-
   drfp->bsplist = UnlockFarComponents (drfp->bsplist);
 
   if (ofp != NULL && need_ofp_close) {
@@ -697,8 +679,6 @@ static void ProcessSingleRecord (
 
     if (sep != NULL) {
       ValNodeAddPointer (&(drfp->sep_list), 0, sep);
-      ValNodeAddInt (&(drfp->filename_list), FILENAME_LIST_ENTITY_ID_ITEM, (Int4) entityID);
-      ValNodeAddPointer (&(drfp->filename_list), FILENAME_LIST_FILENAME_ITEM, StringSave (filename));
 
     }
   } else {
@@ -1130,6 +1110,9 @@ Int2 Main (void)
   /* minimum length */
   dfd.bcd.min_length = myargs[l_argMinLength].intvalue;
 
+  /* require keyword to be present */
+  dfd.bcd.require_keyword = TRUE;
+
   /* set up Barcode Report Configuration */
   enabled_list = (CharPtr) myargs [e_argEnableTests].strvalue;
   disabled_list = (CharPtr) myargs [d_argDisableTests].strvalue;
