@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   1/22/95
 *
-* $Revision: 6.477 $
+* $Revision: 6.565 $
 *
 * File Description: 
 *
@@ -167,7 +167,6 @@ extern void ExtendFastaSet (IteM i);
 extern void ExtendAllSequencesInSet (IteM i);
 extern void SeqLocAdjustByOffset (SeqLocPtr slp, Int4 offset);
 extern void SplitSegmentedFeatsMenuItem (IteM i);
-extern SeqLocPtr SeqLocWholeNew (BioseqPtr bsp);
 extern SeqFeatPtr SeqFeatCopy (SeqFeatPtr sfp);
 extern SeqLocPtr SeqLocReplaceLocalID (SeqLocPtr slp,
 				       SeqIdPtr  new_sip);
@@ -248,6 +247,7 @@ extern void EditPubsEx (BaseFormPtr bfp);
 extern void RemovePubConsortiums (IteM i);
 
 extern void ExtendPartialFeatures (IteM i);
+extern void ExtendPartialFeaturesWithConstraint (IteM i);
 extern void TrimOrganismName (IteM i);
 extern void SUCSubmitterProc (IteM i);
 
@@ -348,7 +348,6 @@ extern Uint2 PackageFormResults (SequinBlockPtr sbp, SeqEntryPtr sep,
 extern void EnableFeaturesPerTarget (BaseFormPtr bfp);
 extern void EnableAnalysisItems (BaseFormPtr bfp, Boolean isDocSum);
 
-extern Int4 ExtendSeqLocToEnd (SeqLocPtr slp, BioseqPtr bsp, Boolean end5);
 extern void ExtendSeqLocToPosition (SeqLocPtr slp, Boolean end5, Int4 pos);
 
 #define REGISTER_BIOSEQ_SEG_EDIT ObjMgrProcLoad(OMPROC_EDIT,"Edit Bioseq Seg","BioseqSegEditor",OBJ_BIOSEQ_SEG,0,OBJ_BIOSEQ_SEG,0,NULL,BioseqSegEditFunc,PROC_PRIORITY_DEFAULT)
@@ -411,7 +410,6 @@ extern void MRnaFromCdsProc (Uint2 entityID);
 extern void BioseqViewFormToolBar (GrouP h);
 extern Boolean DoBuildContig (void);
 extern void SetGenome (PopuP p);
-extern PopuP ReplaceBioSourceGenomePopup (DialoG d, PopuP genome);
 extern PopuP ReplaceBioSourceGencodePopup (DialoG d, PopuP gencode);
 extern CharPtr NameStdPtrToAuthorSpreadsheetString (NameStdPtr nsp);
 extern NameStdPtr AuthorSpreadsheetStringToNameStdPtr (CharPtr txt);
@@ -426,7 +424,6 @@ extern void InsertGeneLocusTagPrefix (IteM i);
 extern void ReplaceRepeatRegionLocusTagWithDbxref (IteM i);
 extern void FindGeneAndProtForCDS (Uint2 entityID, SeqFeatPtr cds,
                                    SeqFeatPtr PNTR gene, SeqFeatPtr PNTR prot);
-extern SeqFeatPtr FindBestProtein (Uint2 entityID, SeqLocPtr product);
 extern void ExportAlignmentInterleave (IteM i);
 extern void ExportAlignmentContiguous (IteM i);
 extern void FixFeatureIntervals (IteM i);
@@ -435,7 +432,6 @@ extern void CombineMultipleCDS (IteM i);
 
 extern void NewDescriptorMenuFunc (ObjMgrProcPtr ompp, BaseFormPtr bfp, Uint2 descsubtype);
 extern Boolean PropagateFromGenBankBioseqSet (SeqEntryPtr sep, Boolean ask);
-extern CharPtr MergeValNodeStrings (ValNodePtr list, Boolean useReturn);
 extern int LIBCALLBACK SortByVnpChoice (VoidPtr ptr1, VoidPtr ptr2);
 extern void PrepareToConvertToCDS (SeqEntryPtr sep, Uint2 entityID,
                                    Uint2 subtype, CharPtr findthis);
@@ -449,23 +445,21 @@ extern void ChangeGenBankNameToLocal (IteM i);
 extern void RemoveGBIDsFromBioseqs (IteM i);
 extern void RemoveGBIDsFromProteins (IteM i);
 extern void RemoveGIsFromBioseqs (IteM i);
-extern CharPtr MergeValNodeStrings (ValNodePtr list, Boolean useReturn);
 
 extern void CommonApplyToAllProc (BaseFormPtr bfp, Int2 type);
 extern void ApplyTitle (IteM i);
 extern void ApplyCDS (IteM i);
 extern void ApplyRRNA (IteM i);
 extern void ApplyImpFeat (IteM i);
-extern void AdjustCDSLocationsForUnknownGapsCallback (SeqFeatPtr sfp, Pointer userdata);
 extern void AdjustCDSLocationsForKnownAndUnknownGapsCallback (SeqFeatPtr sfp, Pointer userdata);
 extern void AdjustFeaturesForGaps (IteM i);
 extern void LoadTPAAccessionNumbersFromFile (IteM i);
 extern void LoadSecondaryAccessionNumbersFromFile (IteM i);
 extern void LoadHistoryAccessionNumbersFromFile (IteM i);
 extern void LoadOrganismModifierTable (IteM i);
+extern void LoadTaxConsult (IteM i);
 extern void ExportOrganismTable (IteM i);
 extern void LoadFeatureQualifierTable (IteM i);
-extern CharPtr GetModifierPopupPositionName (Int2 val);
 
 extern void AddCodonListTotRNA (tRNAPtr trna, ValNodePtr codons);
 
@@ -500,15 +494,21 @@ extern void CorrectCDSGenCodes (IteM i);
 /* extern Boolean RetranslateOneCDS (SeqFeatPtr sfp, Uint2 entityID, Boolean include_stop); */
 extern void UpdateProteinsFromCDS (IteM i);
 
-extern void testAutoDef (IteM i);
-extern void testAutoDefWithOptions (IteM i);
+extern void AutoDef (IteM i);
+extern void AutoDefWithOptions (IteM i);
 extern void AutoDefWithoutModifiers (IteM i);
 extern void AutoDefBaseFormCommon (BaseFormPtr bfp, Boolean use_form, Boolean use_modifiers);
+extern void AutoDefStrain (BaseFormPtr bfp);
+extern void AutoDefMiscFeat (BaseFormPtr bfp);
 extern void AutoDefToolBtn (ButtoN b);
 extern void AutoDefOptionsToolBtn (ButtoN b);
+extern void AutoDefStrainToolBtn (ButtoN b);
+extern void AutoDefMiscFeatToolBtn (ButtoN b);
+extern void AutoDefEntityIDNoOptions (Uint2 entityID, Boolean use_modifiers);
 
 extern void RemoveDefLinesToolBtn (ButtoN b);
 extern void FindStringProcToolBtn (ButtoN b);
+extern void FindFlatfileProcToolBtn (ButtoN b);
 extern void ResolveExistingLocalIDsToolBtn (ButtoN b);
 extern void GroupExplodeToolBtn (ButtoN b);
 
@@ -542,6 +542,7 @@ extern Int2 LIBCALLBACK CreateSegregateByFeatureWindow (Pointer data);
 extern Int2 LIBCALLBACK CreateSegregateByDescriptorWindow (Pointer data);
 extern Int2 LIBCALLBACK CreateSegregateByMoleculeTypeWindow (Pointer data);
 extern Int2 LIBCALLBACK CreateSegregateByIdWindow (Pointer data);
+extern Int2 LIBCALLBACK SequesterSequences (Pointer data);
 extern Int2 LIBCALLBACK RemoveExtraneousSets (Pointer data);
 extern void ReverseComplementBioseqAndFeats (BioseqPtr bsp, Uint2 entityID);
 extern void RemoveOrphanProteins (Uint2 entityID, SeqEntryPtr sep);
@@ -622,8 +623,6 @@ extern void SqnNewAlign (BioseqPtr bsp1, BioseqPtr bsp2, SeqAlignPtr PNTR salp);
 
 extern void ProduceAlignmentNotes (TAlignmentFilePtr afp, TErrorInfoPtr error_list);
 
-extern Boolean CreateUpdateCitSubFromBestTemplate (SeqEntryPtr top_sep, SeqEntryPtr upd_sep);
-
 extern void RemoveAlignmentsWithSequence (BioseqPtr bsp, Uint2 input_entityID);
 extern void FlipEntireAlignmentIfAllSequencesFlipped (SeqAnnotPtr sap, Pointer userdata);
 
@@ -731,11 +730,6 @@ extern void MergeToPartsOrdered (IteM i);
 
 extern void ConvertInnerCDSsToMatPeptidesCallback (BioseqPtr bsp, Pointer userdata);
 extern void MergeCDS (IteM i);
-extern Boolean RetranslateOneCDS 
-( SeqFeatPtr sfp,
-  Uint2 entityID,
-  Boolean include_stop,
-  Boolean no_stop_at_end_of_complete_cds);
 
 extern void InitValNodePopup (ValNodePtr list, PopuP p);
 extern Int2 GetValNodePopup (PopuP p, ValNodePtr list);
@@ -787,7 +781,12 @@ extern ValNodePtr GetSelectedSequenceList (LisT l);
 extern void SelectAllSequencesInListCtrl (LisT l);
 extern void UnSelectAllSequencesInListCtrl (LisT l);
 extern void OffsetLocation (SeqLocPtr loc, Int4 offset, SeqIdPtr sip);
-extern void AddCitSubToUpdatedSequence (BioseqPtr upd_bsp, Uint2 input_entityID);
+
+extern CharPtr kSubmitterUpdateText;
+extern CharPtr kIndexerUpdateVecScreenText;
+extern Boolean CreateUpdateCitSubFromBestTemplate (SeqEntryPtr top_sep, SeqEntryPtr upd_sep, CharPtr update_txt);
+extern void AddCitSubToUpdatedSequence (BioseqPtr upd_bsp, Uint2 input_entityID, CharPtr update_txt);
+
 extern Boolean AlistMessage (EnumFieldAssocPtr al, UIEnumPtr val, UIEnum dflt, CharPtr mssg);
 
 typedef struct loginfo 
@@ -807,13 +806,6 @@ extern void LogCDSAmbiguousFrame (LogInfoPtr lip, SeqFeatPtr sfp);
 extern void LoadGenomeProjectIDsFromFile (IteM i);
 extern void RemoveEmptyGenomeProjectIDs (IteM i);
 
-typedef struct sourcequaldesc
-{
-  CharPtr       name;
-  Boolean       isOrgMod;
-  Uint1         subtype;
-} SourceQualDescData, PNTR SourceQualDescPtr;
-
 extern CharPtr SourceQualValNodeName (ValNodePtr vnp);
 extern ValNodePtr SourceQualValNodeDataCopy (ValNodePtr vnp);
 extern Boolean SourceQualValNodeMatch (ValNodePtr vnp1, ValNodePtr vnp2);
@@ -867,6 +859,38 @@ extern void RemoveGBQual (IteM i);
 extern void ConvertLocusTagToOldLocusTag (IteM i);
 extern void ExportLastLineage (IteM i);
 
+extern void MacroApplyGBQual (IteM i);
+extern void MacroApplySourceQual (IteM i);
+extern void MacroApplyCDSGeneProt (IteM i);
+extern void PublicMacroApplyCDSGeneProt (IteM i);
+extern void MacroApplyRNAQual (IteM i);
+
+extern void MacroRemoveGBQual (IteM i);
+extern void MacroRemoveSourceQual (IteM i);
+extern void MacroRemoveCDSGeneProt (IteM i);
+extern void MacroRemoveRNAQual (IteM i);
+
+extern void MacroConvertGBQual (IteM i);
+extern void MacroConvertSourceQual (IteM i);
+extern void MacroConvertCDSGeneProt (IteM i);
+extern void MacroConvertRNAQual (IteM i);
+
+extern void MacroSwapGBQual (IteM i);
+extern void MacroSwapSourceQual (IteM i);
+extern void MacroSwapCDSGeneProt (IteM i);
+extern void MacroSwapRNAQual (IteM i);
+
+extern void MacroEditGBQual (IteM i);
+extern void MacroEditSourceQual (IteM i);
+extern void MacroEditCDSGeneProt (IteM i);
+extern void PublicMacroEditCDSGeneProt (IteM i);
+extern void MacroEditRNAQual (IteM i);
+
+extern void MacroApplyStructuredComment (IteM i);
+extern void MacroEditStructuredComment (IteM i);
+extern void MacroRemoveStructuredComment (IteM i);
+
+
 /* constraint values */
 #define LOCATION_CONSTRAINT_WHOLE_INTERVAL  1
 #define LOCATION_CONSTRAINT_START_ENDPOINT  2
@@ -888,7 +912,7 @@ extern void ExportLastLineage (IteM i);
 #define LOCATION_CONSTRAINT_NUC_SEQ      2
 #define LOCATION_CONSTRAINT_PROT_SEQ     3
 
-typedef struct locationconstraint
+typedef struct LocationConstraintX
 {
   Int4      left;
   Int4      right;
@@ -896,7 +920,7 @@ typedef struct locationconstraint
   Int4      match_choice;
   Int4      strand;
   Int4      sequence_type;
-} LocationConstraintData, PNTR LocationConstraintPtr;
+} LocationConstraintXData, PNTR LocationConstraintXPtr;
 
 typedef enum
 {
@@ -914,9 +938,9 @@ typedef struct stringconstraint
   Boolean insensitive;
   Boolean whole_word;
   Boolean not_present;
-} StringConstraintData, PNTR StringConstraintPtr;
+} StringConstraintData, PNTR StringConstraintXPtr;
 
-extern StringConstraintPtr StringConstraintFree (StringConstraintPtr scp);
+extern StringConstraintXPtr StringConstraintXFree (StringConstraintXPtr scp);
 
 typedef struct pseudoconstraint
 {
@@ -935,7 +959,7 @@ typedef struct choiceconstraint
   Int4                constraint_type;
   ValNodePtr          qual_choice;
   ValNodePtr          qual_choice_match;
-  StringConstraintPtr string_constraint;
+  StringConstraintXPtr string_constraint;
   PseudoConstraintPtr pseudo_constraint;
   FreeValNodeProc     free_vn_proc;
   CopyValNodeDataProc copy_vn_proc;
@@ -943,20 +967,38 @@ typedef struct choiceconstraint
 
 extern ChoiceConstraintPtr ChoiceConstraintFree (ChoiceConstraintPtr scp);
 
+typedef struct sequenceconstraint 
+{
+  Boolean nucs_ok;
+  Boolean prots_ok;
+  
+  Int4                other_constraint_type;
+  StringConstraintXPtr string_constraint;
+  ChoiceConstraintPtr source_constraint;
+  ValNodePtr          feature_list;
+  
+} SequenceConstraintXData, PNTR SequenceConstraintXPtr;
+
+extern SequenceConstraintXPtr SequenceConstraintXFree (SequenceConstraintXPtr scp);
+extern DialoG SequenceConstraintXDialog (GrouP g);
+extern Boolean DoesSequenceMatchSequenceConstraintX (BioseqPtr bsp, SequenceConstraintXPtr scp);
+
+
+
 typedef struct filterset 
 {
-  StringConstraintPtr   scp;
+  StringConstraintXPtr   scp;
   ChoiceConstraintPtr   ccp;
-  LocationConstraintPtr lcp;
+  LocationConstraintXPtr lcp;
   ChoiceConstraintPtr   cgp;
-  StringConstraintPtr   id_list;
+  StringConstraintXPtr   id_list;
 } FilterSetData, PNTR FilterSetPtr;
 
 extern void FilterSetClearText (FilterSetPtr fsp);
 extern FilterSetPtr FilterSetNew (void);
 extern FilterSetPtr FilterSetFree (FilterSetPtr fsp);
 
-extern Boolean DoesStringMatchConstraint (CharPtr pchSource, StringConstraintPtr scp);
+extern Boolean DoesStringMatchConstraintX (CharPtr pchSource, StringConstraintXPtr scp);
 
 typedef CharPtr (*GetFeatureFieldString) PROTO ((SeqFeatPtr, ValNodePtr, FilterSetPtr));
 typedef void (*SetFeatureFieldString) PROTO ((SeqFeatPtr, Pointer, FilterSetPtr));
@@ -999,18 +1041,18 @@ typedef struct textportion
   CharPtr end_text;
   Boolean insensitive;
   Boolean whole_word;
-} TextPortionData, PNTR TextPortionPtr;
+} TextPortionXData, PNTR TextPortionXPtr;
 
-extern TextPortionPtr TextPortionFree (TextPortionPtr tp);
+extern TextPortionXPtr TextPortionXFree (TextPortionXPtr tp);
 extern void 
-FindTextPortionInString 
+FindTextPortionXInString 
 (CharPtr        str, 
- TextPortionPtr tp, 
+ TextPortionXPtr tp, 
  CharPtr PNTR   ploc, 
  Int4Ptr        plen);
 
-extern DialoG TextPortionDialogEx (GrouP h, Boolean inside, Nlm_ChangeNotifyProc change_notify, Pointer change_userdata);
-extern DialoG TextPortionDialog (GrouP h);
+extern DialoG TextPortionXDialogEx (GrouP h, Boolean inside, Nlm_ChangeNotifyProc change_notify, Pointer change_userdata);
+extern DialoG TextPortionXDialog (GrouP h);
 
 #define CONVERT_TYPE_MOVE  0
 #define CONVERT_TYPE_COPY  1
@@ -1031,13 +1073,13 @@ typedef struct convertfield
   RemoveDescriptorFieldString remove_d_str_func;
   NameFromValNodeProc         name_field_func;
   FilterSetPtr                fsp;
-  TextPortionPtr              text_portion;
+  TextPortionXPtr              text_portion;
   Boolean                     strip_name_from_text;
   Boolean                     remove_parsed;
 } ConvertFieldData, PNTR ConvertFieldPtr;
 
-extern DialoG StringConstraintDialog (GrouP h, CharPtr label, Boolean clear_btn);
-extern DialoG LocationConstraintDialog (GrouP h, Boolean show_interval_controls, Boolean clear_btn);
+extern DialoG StringConstraintDialogX (GrouP h, CharPtr label, Boolean clear_btn);
+extern DialoG LocationConstraintXDialog (GrouP h, Boolean show_interval_controls, Boolean clear_btn);
 
 enum pub_field_nums 
 {
@@ -1235,7 +1277,7 @@ ConstraintChoiceDialog
  CharPtr                   text_name,
  Boolean                   clear_btn,
  Boolean                   use_pseudo);
-extern DialoG SourceConstraintDialog (GrouP h, Boolean clear_btn);
+extern DialoG SourceConstraintDialogX (GrouP h, Boolean clear_btn);
 extern Boolean DoesOneSourceMatchConstraint (BioSourcePtr biop, ChoiceConstraintPtr scp);
 extern DialoG CDSGeneProtConstraintDialog (GrouP h, Boolean clear_btn);
 extern DialoG 
@@ -1259,6 +1301,12 @@ typedef struct parsefield
 
 extern ParseFieldPtr ParseFieldFree (ParseFieldPtr pfp);
  
+extern DialoG ParseFieldDestDialogEx 
+(GrouP                    h,
+ Nlm_ChangeNotifyProc     change_notify,
+ Pointer                  change_userdata,
+ Boolean                  is_search_field,
+ Boolean                  include_dbxref);
 extern DialoG ParseFieldDestDialog 
 (GrouP                    h, 
  Nlm_ChangeNotifyProc     change_notify,
@@ -1321,11 +1369,13 @@ GetSampleForSeqEntry
 extern void ApplyGDSKeyword (IteM i);
 extern void ApplyTPAInferentialKeyword (IteM i);
 extern void ApplyTPAExperimentalKeyword (IteM i);
+extern void ApplyTPAReassemblyKeyword (IteM i);
 extern void ApplyKeywordWithStringConstraint (IteM i);
 extern void RemoveKeywordWithStringConstraint (IteM i);
 
 #if defined(OS_UNIX) || defined(OS_MSWIN)
 extern Int2 LIBCALLBACK CorrectRNAStrandedness (Pointer data);
+extern Int2 LIBCALLBACK CorrectRNAStrandednessUseSmart (Pointer data);
 #endif
 
 extern void AddGeneFeatureFromTitle (SeqEntryPtr nucsep, CharPtr ttl,  SeqLocPtr slp);
@@ -1333,6 +1383,17 @@ extern SeqFeatPtr AddProteinFeatureFromDefline (SeqEntryPtr psep, CharPtr title)
 extern void AddCodingRegionFieldsFromProteinTitle (CdRegionPtr crp, CharPtr title, CharPtr PNTR pcomment);
 extern Boolean ReplaceImportModifierName (CharPtr PNTR orig_name, Int4 col_num);
 
+extern SeqEntryPtr 
+ImportSequencesFromFileEx
+(FILE           *fp, 
+ SeqEntryPtr     sep_list,
+ Boolean         is_na, 
+ Boolean         parse_id,
+ CharPtr         supplied_id_txt,
+ ValNodePtr PNTR err_msg_list,
+ BoolPtr         chars_stripped,
+ Boolean         allow_char_stripping);
+
 extern SeqEntryPtr 
 ImportSequencesFromFile
 (FILE           *fp, 
@@ -1425,6 +1486,7 @@ extern void RemoveNomenclature (IteM i);
 extern void ParseCollectionDateMonthFirst (IteM i);
 extern void ParseCollectionDateDayFirst (IteM i);
 extern void RemoveUnpublishedPublications (IteM i);
+extern void RemovePublishedPublications (IteM i);
 extern void RemoveUnindexedFeatures (IteM i);
 extern void CopyLocusToLocusTag (IteM i);
 
@@ -1452,7 +1514,7 @@ extern void ConvertCodingRegionsWithInternalKnownGapToMiscFeat (IteM i);
 extern void FixOneAlignmentOverGaps (SeqAlignPtr salp, Uint2 entityID);
 extern void ConsolidateSegmentsOverKnownLengthGaps (SeqAlignPtr salp);
 
-extern void CreateDiscrepancyReportWindow ();
+extern void CreateReportWindow (EDiscrepancyReportType report_type);
 extern void ScrollToDiscrepancyItem (ValNodePtr vnp, Pointer userdata);
 extern void EditDiscrepancyItem (ValNodePtr vnp, Pointer userdata);
 extern void WriteClickableListReport (FILE *fp, ValNodePtr discrepancy_list, Boolean show_all, Boolean use_feature_table_fmt);
@@ -1462,9 +1524,8 @@ extern void ChangeKnownGapLength (IteM i);
 extern void AddFlankingNsToKnownLengthGaps (IteM i);
 
 extern Int2 GetSequinAppParam (CharPtr section, CharPtr type, CharPtr dflt, CharPtr buf, Int2 buflen);
-extern Boolean DoBioseqFeaturesMatchSequenceConstraint (BioseqPtr bsp, ValNodePtr feat_list, StringConstraintPtr scp);
-extern Boolean DoesIDListMeetStringConstraint (SeqIdPtr sip, StringConstraintPtr string_constraint);
-extern void ResetCapitalization (Boolean first_is_upper, CharPtr pString);
+extern Boolean DoBioseqFeaturesMatchSequenceConstraintX (BioseqPtr bsp, ValNodePtr feat_list, StringConstraintXPtr scp);
+extern Boolean DoesIDListMeetStringConstraint (SeqIdPtr sip, StringConstraintXPtr string_constraint);
 extern Int2 LIBCALLBACK ReorderSetByAccession (Pointer data);
 
 extern Int2 LIBCALLBACK CopyDescriptorToList (Pointer data);
@@ -1502,6 +1563,7 @@ extern ValNodePtr ChooseFeaturesForConversion (ValNodePtr clickable_list, BaseFo
 extern void RemoveBadPubs (IteM i);
 extern void VecScreenTool (IteM i);
 extern void LogTrimmedLocation (LogInfoPtr lip, SeqLocPtr slp);
+extern void CalculateVectorDescription (ClickableItemPtr cip);
 
 extern void BarcodeTestTool (IteM i);
 
@@ -1516,7 +1578,7 @@ extern void MakeGeneralIDsFromLocusTags (IteM i);
 extern void ShowClickableItemList (ValNodePtr clickable_list, BaseFormPtr bfp, CharPtr win_title, CharPtr label1, CharPtr label2);
 
 extern Int4 CountChosenDiscrepancies (ValNodePtr discrepancy_list, Boolean count_all);
-extern void AddTranslExcept (SeqFeatPtr sfp, CharPtr cds_comment, Boolean use_strict);
+extern void AddTranslExcept (SeqFeatPtr sfp, CharPtr cds_comment, Boolean use_strict, Boolean extend, Boolean adjust_gene);
 
 enum table_data_errors 
 {
@@ -1588,6 +1650,7 @@ extern void BulkEditRNA (IteM i);
 extern void BulkEditorFeatList (Uint2 entityID, ValNodePtr feat_list);
 extern void BulkEditorDescrList (Uint2 entityID, ValNodePtr descr_list);
 extern void BulkEditDiscrepancy (ValNodePtr vnp, Pointer userdata);
+extern void BulkEditorCheckAllDialog (DialoG dlg);
 
 extern Uint1 GetSubtypeForBulkEdit (ValNodePtr feat_list);
 
@@ -1603,17 +1666,24 @@ ConvertOneCDSToMiscFeat
 
 extern void SuppressGenesOnFeaturesInsideMobileElements (IteM i);
 
-extern Int4 GetDeltaSeqLen (DeltaSeqPtr dsp);
 extern BaseFormPtr GetBaseFormForEntityID (Uint2 entityID);
 
 extern ValNodePtr ParseAccessionNumberListFromString (CharPtr list_str, SeqEntryPtr sep);
-extern SeqIdPtr CreateSeqIdFromText (CharPtr id_str, SeqEntryPtr sep);
 
 extern SubSourcePtr FindBadLatLon (BioSourcePtr biop);
 extern DialoG LatLonTestResultsDisplay (GrouP h);
 extern void LatLonTool (IteM i);
 extern DialoG SpecificHostResultsDisplay (GrouP h);
 extern void FixSpecificHostValues (IteM i);
+extern DialoG LatLonCountryResultsDisplay (GrouP h);
+extern Pointer GetLatLonCountryCorrection (Uint1 data_choice, Pointer data, Pointer metadata);
+extern void LatLonCountryTool (IteM i);
+extern DialoG CountryTestResultsDisplay (GrouP h, Pointer metadata);
+extern void CountryFixupTool (IteM i);
+
+extern DialoG TaxFixDisplay (GrouP h);
+extern Boolean IsTaxNameBad (OrgRefPtr org);
+extern void TaxFixTool (IteM i);
 
 extern void SetTransgenicOnSourceDescWhenSourceFeatPresent (IteM i);
 extern void SetFocusOnSourceDescWhenSourceFeatPresent (IteM i);
@@ -1624,7 +1694,7 @@ extern const char *nucleotide_alphabet;
 extern const char *protein_alphabet;
 
 extern DialoG AlnSettingsDlg (GrouP h, Boolean allow_sequence_type);
-extern TSequenceInfoPtr GetDefaultSequenceInfo ();
+extern TSequenceInfoPtr GetDefaultSequenceInfo (void);
 extern TSequenceInfoPtr GetAlignmentOptions (Uint1Ptr moltype, TSequenceInfoPtr sequence_info);
 
 
@@ -1634,6 +1704,135 @@ extern SeqFeatPtr GetGeneForFeature (SeqFeatPtr sfp);
 
 extern void LoadFeatureFieldTable (IteM i);
 
+/* structure used for taxname options for loading values froma table or the Apply/Edit/Convert/Remove dialogs */
+typedef struct taxnameoptions {
+ Boolean remove_taxref;
+ Boolean remove_old_name;
+ Boolean remove_common;
+} TaxnameOptionsData, PNTR TaxnameOptionsPtr;
+
+extern void ApplyTaxnameOptionsToBioSource (BioSourcePtr biop, TaxnameOptionsPtr top);
+extern DialoG TaxnameOptionsDialog (GrouP h, Nlm_ChangeNotifyProc change_notify, Pointer change_userdata);
+
+extern void CleanTableLine (CharPtr line);
+
+extern void ApplyTagToCodingRegionsInSourceFeatures (IteM i);
+extern Boolean EntityIDAlreadyInList (Uint2 entityID, ValNodePtr entityIDList);
+
+extern void ResolveFeatureOverlaps (IteM i);
+extern void ConvertGeneralIdToLocalID (IteM i);
+
+extern void UpdateFeatures (IteM i);
+
+NLM_EXTERN void RemoveDupGenBankSets (SeqEntryPtr sep);
+NLM_EXTERN BioseqSetPtr InsetNewSet (BioseqSetPtr bssp, Uint1 _class);
+
+extern void AbbreviateCitSubAffilStates (IteM i);
+extern void RemoveQualityScores (BioseqPtr bsp, FILE *log_fp, BoolPtr data_in_log);
+extern void NewLoadFeatureQualifierTable (IteM i);
+extern void NewLoadSourceQualifierTable (IteM i);
+extern void ListAllSequences (BioseqPtr bsp, Pointer userdata);
+extern void ChooseCategories (ValNodePtr value_list, Boolean do_choose);
+extern void ChooseCategoriesByStringConstraint (ValNodePtr value_list, StringConstraintXPtr scp, Boolean do_choose);
+extern void CapitalizeFirstLetterOfEveryWord (CharPtr pString);
+
+typedef void (*BulkSetFieldFunc) PROTO ((Pointer, Pointer));
+typedef Pointer (*BulkSetFieldStringFunc) PROTO ((Pointer, ApplyValuePtr));
+typedef Pointer (*BulkGetFieldFunc) PROTO ((Uint1, Pointer, Pointer));
+typedef CharPtr (*BulkDisplayFieldFunc) PROTO ((Pointer));
+typedef void (*BulkFreeFieldFunc) PROTO ((Pointer));
+typedef DialoG (*BulkCreateDlgFunc) PROTO ((GrouP, CharPtr, SeqEntryPtr));
+typedef Int4 (*BulkFormatColumnFunc) PROTO ((ColPtr, CharPtr));
+typedef void (*BulkDrawColumnFunc) PROTO ((Pointer, RectPtr));
+typedef Pointer (*BulkReleaseCellFunc) PROTO ((Pointer));
+typedef Pointer (*BulkCopyFieldFunc) PROTO ((Pointer));
+
+typedef struct bulkedfield {
+  CharPtr name;
+  BulkSetFieldFunc set_func;
+  BulkSetFieldStringFunc set_str_func;
+  BulkGetFieldFunc get_func;
+  BulkDisplayFieldFunc display_func;
+  BulkFreeFieldFunc free_func;
+  BulkCreateDlgFunc create_dlg_func;
+  BulkFormatColumnFunc format_col_func;
+  BulkDrawColumnFunc  draw_col_func;
+  BulkReleaseCellFunc release_cell_func;
+  BulkCopyFieldFunc   copy_func;
+} BulkEdFieldData, PNTR BulkEdFieldPtr;
+
+NLM_EXTERN Pointer BulkSetSimpleTextString (Pointer curr_val, ApplyValuePtr avp);
+NLM_EXTERN CharPtr BulkDisplaySimpleText (Pointer data);
+NLM_EXTERN void BulkFreeSimpleText (Pointer data);
+NLM_EXTERN Int4 BulkFormatSimpleText (ColPtr col, CharPtr name);
+NLM_EXTERN Pointer BulkSimpleTextCopy (Pointer data);
+NLM_EXTERN DialoG BulkSimpleTextDialog (GrouP g, CharPtr name, SeqEntryPtr sep);
+
+NLM_EXTERN void BulkEditorObjectList (Uint2 entityID, CharPtr title, ValNodePtr feat_list, BulkEdFieldPtr field_list);
+NLM_EXTERN DialoG 
+CreateBulkEditorDialog 
+(GrouP               h,
+ BulkEdFieldPtr      field_list,
+ ValNodePtr          feat_list,
+ SeqEntryPtr         sep, 
+ Boolean             collapse_by_default,
+ ClickableCallback   single_click_func,
+ ClickableCallback   double_click_func);
+
+NLM_EXTERN void ApplyBulkEditorToObjectList (DialoG d);
+
+NLM_EXTERN void FlipSequenceIntervals (IteM i);
+
+NLM_EXTERN void CreateRefSeqProteinIDs (IteM i);
+
+extern void ConvertBioSourceDbxrefToFeatureDbxref (IteM i);
+
+NLM_EXTERN void AddFluComments (IteM i);
+NLM_EXTERN void CreateStructuredCommentsItem (IteM i);
+NLM_EXTERN void SubmitterCreateStructuredComments (IteM i);
+
+NLM_EXTERN SeqAlignPtr GetSeqAlignTSA (BioseqPtr bsp1, BioseqPtr bsp2);
+NLM_EXTERN void EditTSAAssembly (IteM i);
+
+NLM_EXTERN void MergeBiosources (IteM i);
+NLM_EXTERN Boolean FixIDsAndTitles (SeqEntryPtr new_list, SeqEntryPtr current_list, Boolean is_nuc);
+
+NLM_EXTERN void ExportQualifiers (IteM i);
+
+extern Boolean IsUnwantedFeatureType (Uint1 key);
+extern void ExportBankitComments (IteM i);
+
+extern void ImportAlignmentForSeqHistInterval (IteM i);
+extern void AdvancedAssemblyAlignmentEditor (IteM i);
+extern void AssemblyAlignmentIntervalResolution (IteM i);
+
+extern void ExternalSourceQualifierTableReader (IteM i);
+
+extern void TrimPrimerSeqJunk (IteM i);
+
+extern void SequinSeqViewFormMessage (ForM f, Int2 mssg);
+extern Boolean WriteTheEntityID (Uint2 entityID, CharPtr path, Boolean binary);
+
+extern BioseqSetPtr FindTopLevelSetForDesktopFunction (BioseqSetPtr bssp);
+extern void ReorderSetByAccessionMenuItem (IteM i);
+extern void DescriptorPropagateMenuItem (IteM i);
+NLM_EXTERN void RepackagePartsMenuItem (IteM i);
+extern void NewSegregateBioseqSetMenuItem (IteM i);
+extern void SequesterSequencesMenuItem (IteM i);
+extern void GetRidOfSegGapMenuItem (IteM i);
+extern void GenerateSeqAlignMenuItem (IteM i);
+extern void UpdateSeqAlignMenuItem (IteM i);
+extern void CorrectRNAStrandednessMenuItem (IteM i);
+extern void BioseqRevCompByIDMenuItem (IteM i);
+extern void RemoveSetsInSetMenuItem (IteM i);
+
+extern void LoadTaxTableReader (IteM i);
+
+#ifdef OS_MSWIN
+NLM_EXTERN Int4 RunSilent(const char *cmdline);
+#endif
+
+
 #ifdef __cplusplus
 }
 #endif
